package org.objectweb.telosys.common;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.objectweb.telosys.util.DateUtil;
import org.objectweb.telosys.util.XmlUtil;


/**
 * Utility class to write standard Java objects as XML tag
 * 
 * @author Laurent Guerin
 *
 */
public class JavaObjectWriter
{
    private final static String END_OF_LINE = "\n" ;
        
    /**
     * Writes the given object in XML in the given OutputStream.<br>
     * Output : (tagname value="..." /)
     * @param outputStream
     * @param obj
     * @param sTagName
     * @return true if a tag has been generated, else false
     */
    public static boolean toXml(OutputStream outputStream, Object obj, String sTagName )
    {
        if ( outputStream == null )
        {
            throw new TelosysRuntimeException("JavaObjectWriter.toXml() : OutputStream is null");
        }
        Writer writer = new OutputStreamWriter(outputStream);
        return toXml(writer, obj, sTagName);
    }
    
    /**
     * Writes the given object in XML using the given Writer.<br>
     * Output : (tagname value="..." /)
     * @param writer
     * @param obj
     * @param sTagName
     * @return true if a tag has been generated, else false
     */
    public static boolean toXml(Writer writer, Object obj, String sTagName )
    {
        boolean bKnown = false ;
        if ( writer == null )
        {
            throw new TelosysRuntimeException("JavaObjectWriter.toXml() : Writer is null");
        }
        if (obj != null)
        {
            String sXmlValue = null ;
	        if (obj instanceof String)
	        {
	            bKnown = true ;
	            printValue( writer, sTagName, XmlUtil.xmlString( (String) obj ) );
			}
	        else if (obj instanceof java.lang.Number) // Byte, Short, Interger, Long, Double, Float, BigDecimal, BigInterger
	        { 
	            bKnown = true ;
	            printValue( writer, sTagName, ((java.lang.Number)obj).toString() );
			}
	        else if (obj instanceof java.lang.Boolean)
	        {
	            bKnown = true ;
	            printValue( writer, sTagName, ((java.lang.Boolean)obj).toString() );
			}
	        else if (obj instanceof java.util.Date)
	        {
	            bKnown = true ;
	            java.util.Date date = (java.util.Date)obj;
	            String sDateTime = DateUtil.dateTimeISO(date);
	            String sOther = " date=\"" + DateUtil.dateISO(date) + "\" time=\"" + DateUtil.timeISO(date) + "\" ";
	            printValue( writer, sTagName, sDateTime, sOther );
			}
	        //--- Print ...
	        if ( ! bKnown  )
	        {
	            printComment(writer, "Unknown object type '" + obj.getClass().getName() + "'" );
	        }
	        return bKnown ;
        }
        else
        {
            // object is null => pseudo 'null' value = '' 
	        printValue( writer, sTagName, "" ); 
	        return true ; // Unknown, but generated
        }
    }
    
    private static void printValue(Writer writer, String sTagName , String sXmlString )
    {
        printValue(writer, sTagName, sXmlString, null );
    }
    
    private static void printValue(Writer writer, String sTagName , String sXmlString, String sOtherAttributes )
    {
        String s = "<" + sTagName + " value=\"" + sXmlString + "\" " 
        		+ ( sOtherAttributes != null ? sOtherAttributes : "" )
        		+ "/>" ;
        
        try {
	        writer.write(s);
            writer.write(END_OF_LINE);
	        
	    } catch (IOException ex)
	    {
	        throw new TelosysRuntimeException("JavaObjectWriter.toXml() : IOException", ex);
	    }        
    }
    
    private static void printComment(Writer writer, String sComment )
    {
        try {
	        writer.write("<!-- " + sComment + " -->");
            writer.write(END_OF_LINE);
	        
	    } catch (IOException ex)
	    {
	        throw new TelosysRuntimeException("JavaObjectWriter.toXml() : IOException", ex);
	    }        
    }
}
