package org.objectweb.telosys.common;

/**
 * 
 * Object used to write messages on all there defined outputs <br>
 * The "CONSOLE" output is always defined <br>
 * 
 * <br>
 * There's 3 kinds of outputs : <br>
 * . error output : for "error" messages <br>
 * . info output : for "info" and "warn" messages<br>
 * . trace output : for "trace" messages <br>
 * 
 * @author Laurent GUERIN
 * 
 */
import java.io.*;
import java.util.*;

public final class LogWriter
{
    private static int STACK_MAX_CALLERS = 10 ;
    
    //-----------------------------------------------------------------------------
    //--- PRIVATE ATTRIBUTES
    //-----------------------------------------------------------------------------
    // No log for the LogWriter ( v 0.9.9 )
    //private static final TelosysClassLogger log             = new TelosysClassLogger(LogWriter.class);

    private final static String             CONSOLE         = "CONSOLE";

    private final static String             LINE            = "------------------------------------------------------------";

    private final static int                LF              = 1;

    private final static int                CRLF            = 2;

    //--- Type de fin de ligne
    private int                             _iEndOfLine     = LF;

    //--- Collection des flux de sortie utilisables
    private Hashtable                       _htOutputs      = new Hashtable(5);

    //--- Collection des flux de sortie pour chaque mthode
    private OutputStream                    _errorOutputs[] = null;

    private OutputStream                    _infoOutputs[]  = null;

    private OutputStream                    _traceOutputs[] = null;

    public final static int                 INFO            = 1;

    public final static int                 TRACE           = 2;

    public final static int                 ERROR           = 3;

    //-----------------------------------------------------------------------------
    //--- CONSTRUCTORS
    //-----------------------------------------------------------------------------
    /**
     * Constructs a new instance using a properties file to define the outputs
     * @param sFileName
     */
    public LogWriter(String sFileName)
    {
        init(sFileName);
    }

    /**
     * Constructs a new instance using the given properties to define the outputs
     * @param prop
     */
    public LogWriter(Properties prop)
    {
        init(prop);
    }

    /**
     * Constructs a new instance with "CONSOLE" as the only output for all kinds of messages
     * 
     */
    public LogWriter()
    {
        Properties prop = new Properties();
        prop.put("method.trace", "CONSOLE");
        prop.put("method.info", "CONSOLE");
        prop.put("method.error", "CONSOLE");
        init(prop);
    }

    //-----------------------------------------------------------------------------
    //--- PRIVATE METHODS
    //-----------------------------------------------------------------------------
    synchronized private void init(String sFileName)
    {
        if (sFileName != null)
        {
            FileInputStream fis = null;
            try
            {
                fis = new FileInputStream(sFileName);
                init(fis);
            }
            catch (FileNotFoundException ex)
            {
                internalError("init(" + sFileName + ")", ex);
            } finally
            {
                Tool.closeInputStream(fis);
            }
        }
    }

    //-----------------------------------------------------------------------------
    synchronized private void init(InputStream in)
    {
        if (in != null)
        {
            Properties prop = new Properties();
            try
            {
                prop.load(in);
                //log.trace("Properties loaded : ");
                Tool.showProperties(prop);
                // Telosys.trace(CLASS_NAME, prop.toString());
                init(prop);
            } catch (IOException ex)
            {
                internalError("init(InputStream)", ex);
            }
        }
    }

    //-----------------------------------------------------------------------------
    synchronized private void init(Properties prop)
    {
        //--- initialisation du type de fin de ligne (dans les flux de log)
        String sEndOfLine = prop.getProperty("endofline");
        if (sEndOfLine != null)
        {
            if (sEndOfLine.toUpperCase().trim().equals("CRLF"))
                _iEndOfLine = CRLF;
            if (sEndOfLine.toUpperCase().trim().equals("LF"))
                _iEndOfLine = LF;
        }
        //--- initialisation des flux de sortie utilisables
        initAllOutputs(prop);
        //--- initialisation des flux de sortie de chaque mthode
        _infoOutputs  = initMethodOutputs("info",  prop);
        _traceOutputs = initMethodOutputs("trace", prop);
        _errorOutputs = initMethodOutputs("error", prop);
    }

    //-----------------------------------------------------------------------------
    synchronized private void internalError(String sMsg)
    {
        internalError(sMsg, null);
    }

    //-----------------------------------------------------------------------------
    synchronized private void internalError(String sMsg, Exception ex)
    {
        System.out.println("LogWritter internal error : " + sMsg );
        System.out.flush();
        
        if (ex != null)
        {
            System.out.println("LogWritter Exception : " + ex.getMessage() );
            System.out.flush();
            ex.printStackTrace(Console.out);
            System.out.flush();
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Init all the usable outputs defined in the given properties <br>
     * After calling this method the outputs are open and ready to use <br>
     * <br>
     * Remark : The "CONSOLE" output is always added in the usable outputs list<br>
     * <br>
     * e.g. : <br>
     *  output.file1=C:/tmp/monfichier1.log <br>
     *  output.file2=C:/tmp/monfichier2.log <br>
     *  
     * @param prop
     */
    private void initAllOutputs(Properties prop)
    {
        //log.trace("initOutputs(prop)");
        //--- Stockage de la CONSOLE (systmatiquement)
        addOuput(CONSOLE, null);

        //--- Stockage de tous les autres flux de sortie dclars dans les
        // properties
        Enumeration enumKeys = prop.keys();
        String sKey = null;
        String sKeyMinus = null;
        while (enumKeys.hasMoreElements()) // Pour chaque "cl" dans les
        // properties
        {
            sKey = (String) enumKeys.nextElement();
            //sKeyMinus = sKey.trim().toLowerCase() ;
            // Si la cl de la proprit commence par "output", c'est un nom de
            // fichier
            if (sKey.trim().toLowerCase().startsWith("output"))
            {
                addOuput(sKey, prop);
            }
        }
    }

    //-----------------------------------------------------------------------------
    private void addOuput(String sKey, Properties prop)
    {
        //log.trace("addOuput(" + sKey + ")");
        if ( sKey == null ) 
        {
            internalError("addOutput(null,...) : key is null");
            return ;
        }
        Object obj = _htOutputs.get(sKey);
        if (obj == null) // Non trouv => A AJOUTER
        {
            if (sKey.equals(CONSOLE))
            {
                //--- Sortie CONSOLE
                // NB: NullPointerException if the key or value is null
                if ( Console.out != null )
                {
                    _htOutputs.put(sKey, Console.out);                    
                }
            }
            else
            {
                //--- Sortie FICHIER
                String sFileName = prop.getProperty(sKey);
                //log.trace("sFileName = " + sFileName);
                if (sFileName != null)
                {
                    FileOutputStream fos = null ;
                    try
                    {
                        fos = new FileOutputStream(sFileName);
                        println(fos, LINE);
                        println(fos, " LOG INITIALIZED ");
                        println(fos, LINE);
                    } catch (FileNotFoundException ex) // Si cration impossible
                    {
                        // ERREUR
                        internalError("addOutput(" + sKey + ")", ex);
                    }
                    // NB: NullPointerException if the key or value is null 
                    if ( fos != null )
                    {
                        _htOutputs.put(sKey, fos);
                    }
                }
            }
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Defines all the outputs to use for the given method name 
     * @param sMethode the name of the method : "error", "info", "trace" ("warn" goes in "info" output)
     * @param prop the properties
     * @return an array of the outputs to use
     */
    private OutputStream[] initMethodOutputs(String sMethode, Properties prop)
    {
        Vector v = new Vector(5);
        String sOutputName;
        String sList = prop.getProperty("method." + sMethode);

        if (sList != null) // if this method is defined in the properties
        {
            //--- List of all the outputs to use ( separator : " " or "," or ";" )
            StringTokenizer st = new StringTokenizer(sList, " ,;");
            int i = 0;
            while (st.hasMoreTokens())
            {
                sOutputName = st.nextToken();
                //--- Is this a defined output with this name ?
                Object obj = _htOutputs.get(sOutputName);
                if (obj != null) // YES => Take it
                {
                    //log.trace(" v.add(" + sOutputName + " stream )");
                    v.add(i++, obj);
                }
                else
                {
                    internalError("initMethodeOutputs(" + sMethode + ", prop) : Stream object is null !");
                }
            }
        }

        //--- Get the "OutputStream[]" array from the vector
        OutputStream[] typeForCasting = new OutputStream[0];
        OutputStream[] returnValue = (OutputStream[]) v.toArray(typeForCasting);
        return returnValue;
    }

    //-----------------------------------------------------------------------------
    //--- Ecriture dans chacun des flux de sortie associs au type de trace
    //-----------------------------------------------------------------------------
    private void println(OutputStream outputs[], String msg)
    {
        if (outputs != null)
        {
            //--- Pour chaque flux de sortie
            for ( int i = 0 ; i < outputs.length ; i++ )
            {
                println(outputs[i], msg);
            }
        }
    }

    //-----------------------------------------------------------------------------
    synchronized private void println(OutputStream out, String msg)
    {
        if (out != null)
        {
            try
            {
                out.write(msg.getBytes());
                if (_iEndOfLine == CRLF)
                {
                    out.write('\r');
                    out.write('\n');
                }
                else
                {
                    out.write('\n');
                }
                out.flush();
            } catch (IOException ex)
            {
                internalError("println(outputs[], msg) ", ex);
            }
        }
    }

    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------
    /**
     * Prints an "ERROR" message on all the "error" outputs
     * @param sClassName
     * @param sMsg
     */
    public void error(String sClassName, String sMsg)
    {
        println(_errorOutputs, "ERROR : [" + sClassName + "] " + sMsg);
    }

    /**
     * Prints an "ERROR" message on all the "error" outputs
     * @param sMsg
     */
    public void error(String sMsg)
    {
        println(_errorOutputs, "ERROR : " + sMsg);
    }

    //-----------------------------------------------------------------------------
    /**
     * Prints a "WARN" message on all the "info" outputs
     * @param sClassName
     * @param sMsg
     */
    public void warn(String sClassName, String sMsg)
    {
        println(_infoOutputs, "WARN  : [" + sClassName + "] " + sMsg);
    }

    /**
     * Prints a "WARN" message on all the "info" outputs
     * @param sMsg
     */
    public void warn(String sMsg)
    {
        println(_infoOutputs, "WARN  : " + sMsg);
    }

    //-----------------------------------------------------------------------------
    /**
     * Prints an "info" message on all the "info" outputs
     * @param sClassName
     * @param sMsg
     */
    public void info(String sClassName, String sMsg)
    {
        println(_infoOutputs, "info  : [" + sClassName + "] " + sMsg);
    }

    /**
     * Prints an "info" message on all the "info" outputs
     * @param sMsg
     */
    public void info(String sMsg)
    {
        println(_infoOutputs, "info  : " + sMsg);
    }

    //-----------------------------------------------------------------------------
    /**
     * Prints an "Trace" message on all the "trace" outputs
     * @param sClassName
     * @param sMsg
     */
    public void trace(String sClassName, String sMsg)
    {
        println(_traceOutputs, "Trace : [" + sClassName + "] " + sMsg);
    }

    /**
     * Prints an "Trace" message on all the "trace" outputs
     * @param sMsg
     */
    public void trace(String sMsg)
    {
        println(_traceOutputs, "Trace : " + sMsg);
    }

    //-----------------------------------------------------------------------------
    /**
     * Prints the current outputs configuration 
     * @param out where to print the configuration
     */
    public void showConfig(PrintStream out)
    {
        out.println("------------------------------");
        out.println("Logger configuration : ");
        out.println(" Outputs declared : ");
        Enumeration enumKeys = _htOutputs.keys();
        String sKey = null;
        while (enumKeys.hasMoreElements()) // Pour chaque "cl" dans les
        // properties
        {
            sKey = (String) enumKeys.nextElement();
            out.println("  . " + sKey);
        }
        out.println(" number of outputs for 'info'  : " + (_infoOutputs == null ? 0 : _infoOutputs.length));
        out.println(" number of outputs for 'trace' : " + (_traceOutputs == null ? 0 : _traceOutputs.length));
        out.println(" number of outputs for 'error' : " + (_errorOutputs == null ? 0 : _errorOutputs.length));
        out.println("------------------------------");
    }
    
    //-------------------------------------------------------------------------------------
    /**
     * Reports an exception (including causes and stack trace) on all the "error" outputs
     * @param ex
     */
    public void exeptionReport( Throwable ex )
    {
        exeptionReport( ex, false );
    }
    
    //-------------------------------------------------------------------------------------
    /**
     * Reports an exception (or an exception cause recursively) on all the "error" outputs
     * @param ex
     * @param bCause true if the given exception is a cause, else false
     */
    // public void exeptionReport( Throwable ex, boolean bCause )
    private void exeptionReport( Throwable ex, boolean bCause ) // private since v 1.1.0
    {
        String sCause = bCause ? "CAUSE " : "" ;
        println(_errorOutputs, "EXCEPTION " + sCause + ": [" + ex.getClass().getName() + "] : \"" + ex.getMessage() + "\"" );
        StackTraceElement st[] = ex.getStackTrace();
        StackTraceElement loc = null;
        if ( st.length > 0 )
        {
            //--- Exception origin ( 0 ) 
	        loc = st[0];
            //println(" in '" + loc.getFileName() + "', line " + loc.getLineNumber() + ", method '" + loc.getMethodName() + "'" );
            //println(" in '" + loc.getClassName() + "', line " + loc.getLineNumber() + ", method '" + loc.getMethodName() + "', callers :" );
            //--- Callers stack ( 1 to N )
	        for ( int i = 0 ; i < st.length && i < STACK_MAX_CALLERS+1 ; i++ )
	        {
	            loc = st[i];
	            println(_errorOutputs, " . " + loc.getClassName() + "." + loc.getMethodName() + " : " + loc.getLineNumber() );
	        }
        }
        if ( ex.getCause() != null )
        {
            exeptionReport( ex.getCause(), true );
        }
    }

}