package org.objectweb.telosys.common;

/**
 * This class manages logging messages for the Java classes that are not TelosysObject descendent. <br>
 * 
 * Same principle as the trace management in TelosysObject <br>
 * but designed to be held as a static private attribute in the class that used it.
 * 
 * @author Laurent GUERIN
 *
 */
public class TelosysClassLogger 
{
    
    private String   _sClassName             = "???";
    
    private int      _iLastTraceConfigState  = -1 ; 
    
    private boolean  _bTrace                 = false ; // true if trace is activated for this classname


    //-----------------------------------------------------------------------------
    /**
     * Constructs a logs manager instance dedicated for the given Java class
     * @param cl the class to managed
     */
    public TelosysClassLogger(Class cl)
    {
        if ( cl != null )
        {
            _sClassName = cl.getName() ;            
        }
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Logs a standard TRACE message, using the active loggers, <br>
     * only if the trace flag is "true" for the Java class managed
     * 
     * @param s the message to log
     */
    public void trace(String s)
    {
        //--- If the configuration has changed since the last acces ...
        if ( TraceConfig.getCurrentState() != _iLastTraceConfigState )
        {
          //--- Refresh the flag from the configuration
          _bTrace = TraceConfig.flagIsOn(_sClassName);
          //--- Keep the current configuration state
          _iLastTraceConfigState = TraceConfig.getCurrentState();
        }
        if ( _bTrace == true )
        {
            Telosys.trace ( _sClassName, s );
        }        
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Logs a standard ERROR message, using the active loggers
     * @param s the message to log
     */
    public void error(String s)
    {
        Telosys.error ( _sClassName, s );
    }
    //-----------------------------------------------------------------------------
    /**
     * Logs the given exception, using the active loggers
     * 
     * @param exception : the exception to log
     */
    public void error(Throwable exception)
    {
        Telosys.error( _sClassName, exception );
    }

    //-----------------------------------------------------------------------------
    /**
     * Logs the given ERROR message and exception, using the active loggers
     * @param sMsg : the error message 
     * @param exception : the exception
     */
    public void error(String sMsg, Throwable exception)
    {
        this.error(sMsg);
        this.error(exception);
    }

    //-----------------------------------------------------------------------------
    /**
     * Logs a standard INFO message, using the active loggers
     * @param s the message to log
     */
    public void info(String s)
    {
        Telosys.info ( _sClassName, s );
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Logs a standard WARN message, using the active loggers
     * 
     * @param s the message to log
     */
    public void warn(String s)
    {
        Telosys.warn( _sClassName, s);
    }
    
}