package org.objectweb.telosys.common;

import org.objectweb.telosys.auth.IUserValidator;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.rpl.xml.StandardXmlWrapperRegistry;
import org.objectweb.telosys.screen.core.IScreenDataProvider;
import org.objectweb.telosys.screen.core.IScreenProvider;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.core.StandardScreenRegistry;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.IServiceProvider;
import org.objectweb.telosys.service.StandardServiceRegistry;

/**
 * Global TELOSYS factory. <br>
 * It encapsulates and uses the "external factory" provided by the application
 * 
 * @author Laurent GUERIN
 *  
 */
public class TelosysFactory extends TelosysObject
{
    private IExternalMainFactory _externalFactory = null;

    private IXmlWrapperProvider  _xmlWrapperProvider = null; // v 1.0.0
    
    private IServiceProvider     _serviceProvider    = null ;  // v 1.0.5
    
    private IScreenProvider      _screenProvider     = null ;  // v 1.0.5
    
    private IScreenDataProvider  _screenDataProvider = null ;  // v 1.0.5
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Constructs a new Telosys facory using the given external factory
     * 
     * @param extFactory the application factory 
     */
    public TelosysFactory(IExternalMainFactory extFactory)
    {
        trace("TelosysFactory constructor ...");
        if (extFactory != null)
        {
            _externalFactory = extFactory;
            
            //--- Get the XmlWrapperProvider defined in the external factory or Strandard provider if none
            _xmlWrapperProvider = _externalFactory.getXmlWrapperProvider() ; // v 1.0.0
            if ( _xmlWrapperProvider == null )
            {
                _xmlWrapperProvider = new StandardXmlWrapperRegistry();  // v 1.0.0
            }
            
            //--- Init the ScreenProvider ( ie StandardScreenRegistry ) with the Telosys properties // v 1.0.5 
            _screenProvider = _externalFactory.getScreenProvider();
            if ( _screenProvider != null )
            {
                _screenProvider.init(); // v 1.0.5
            }
            else
            {
                //--- Use the StandardScreenRegistry as the default ScreenProvider
                StandardScreenRegistry screenRegistry = new StandardScreenRegistry(); // v 1.0.5
                screenRegistry.init(); // v 1.0.5
                
                _screenProvider = screenRegistry ; // v 1.0.5
                _screenDataProvider = screenRegistry ; // v 1.0.5
            }

            //--- Init the ServiceProvider ( ie StandardServiceRegistry ) with the Telosys properties // v 1.0.5 
            _serviceProvider = _externalFactory.getServiceProvider();
            if ( _serviceProvider != null )
            {
                _serviceProvider.init(); // v 1.0.5
            }
            else
            {
                //--- Use the StandardServiceRegistry as the default ServiceProvider
                _serviceProvider = new StandardServiceRegistry(); // v 1.0.5
                _serviceProvider.init(); // v 1.0.5
            }
        }
        else
        {
            throw new TelosysRuntimeException("TelosysFactory constructor : external factory is NULL !");
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Creates a new screen context using the "external factory"
     * 
     * @param sScreenName
     * @param screenSession
     * @return
     * @throws TelosysException
     */
    public ScreenContext createScreen(ScreenSession screenSession, String sScreenName, int iScreenId ) throws TelosysException
    {
        trace("createScreen(" + sScreenName + ") ...");
//        ScreenContext screen = null;
        if ( sScreenName != null )
        {
//            //--- Use the external main factory
//            if (_externalFactory != null)
//            {
//                IScreenProvider screenProvider = _externalFactory.getScreenProvider();
//                if (screenProvider != null)
//                {
//                    //--- Use the screen provider to get a screen instance
//                    //screen = screenProvider.getScreen(sScreenName, screenSession);
//                    screen = screenProvider.getScreen(screenSession, sScreenName, iScreenId );
//                    if (screen != null)
//                    {
//                        trace("createScreen() : object created ");
//                    }
//                    else
//                    {
//                        error("createScreen() : cannot create object ");
//                    }
//                }
//                else
//                {
//                    error("createScreen() : screenProvider is null");
//                }
//            }
//            else
//            {
//                error("createScreen() : external factory is null");
//            }
            
            IScreenProvider screenProvider = getScreenProvider() ; // v 1.0.5
            if ( screenProvider != null )
            {
                //--- Use the screen provider to get a screen instance
                ScreenContext screen = screenProvider.getScreen( screenSession, sScreenName, iScreenId );
                if ( screen != null )
                {
                    //trace("createScreen() : screen object created ");
                    return screen;
                }
                else
                {
                    error("createScreen() : cannot create screen object ");
                }
            }
            else
            {
                error("createScreen() : cannot get ScreenProvider");
            }
        }
        else
        {
            error(" createScreen() : ScreenName is null !");
        }
        return null ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Creates a new ScreenData using the "external factory"
     * 
     * @param sScreenName
     * @return
     * @throws TelosysException
     */
    public ScreenData createScreenData(String sScreenName) throws TelosysException
    {
        trace("createScreenData(" + sScreenName + ") ...");
//        ScreenData screenData = null;
        if (sScreenName != null)
        {
//            //--- Use the external main factory
//            if (_externalFactory != null)
//            {
//                //--- Get the external "Screen Data Provider"
//                IScreenDataProvider screenDataProvider = _externalFactory.getScreenDataProvider();
//                if (screenDataProvider != null)
//                {
//                    screenData = screenDataProvider.getScreenData(sScreenName);
//                    if (screenData != null)
//                    {
//                        trace("createScreenData() : object created ");
//                    }
//                    else
//                    {
//                        error("createScreenData() : cannot create object ");
//                    }
//                }
//                else
//                {
//                    error("createScreenData() : screenDataProvider is null");
//                }
//            }
//            else
//            {
//                error("createScreenData() : external factory is null");
//            }
            
            IScreenDataProvider screenDataProvider = getScreenDataProvider() ; // v 1.0.5
            if ( screenDataProvider != null )
            {
                //--- Use the screen provider to get a screen instance
                ScreenData screenData = screenDataProvider.getScreenData(sScreenName);
                if (screenData != null)
                {
                    trace("createScreenData() : object created ");
                    return screenData;
                }
                else
                {
                    error("createScreenData() : cannot create object ");
                }
            }
            else
            {
                error("createScreen() : cannot get ScreenProvider");
            }
        }
        else
        {
            error(" createData() : ScreenName is null !");
        }
        return null ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the UserValidator from the "external factory"
     * @return
     */
    public IUserValidator getApplicationUserValidator()
    {
        IUserValidator validator = null;
        //--- Use the external main factory
        if (_externalFactory != null)
        {
            //--- Get the external "Screen Data Provider"
            validator = _externalFactory.getUserValidator();
            if (validator != null)
            {
                trace("getApplicationUserValidator() : OK");
            }
            else
            {
                error("getApplicationUserValidator() : cannot get validator ");
            }
        }
        else
        {
            error("getApplicationUserValidator() : no external factory");
        }
        return validator;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the ApplicationLogger from the "external factory"
     * @return
     */
    public IApplicationLogger getApplicationLogger()
    {
        IApplicationLogger logger = null ;
        //--- Use the external main factory
        if (_externalFactory != null)
        {
            //--- Get the external "Application Logger"
            logger = _externalFactory.getApplicationLogger();
            if (logger != null)
            {
                trace("getApplicationLogger() : logger found");
            }
            else
            {
                trace("getApplicationLogger() : logger not found");
            }
        }
        else
        {
            error("getApplicationLogger() : no external factory");
        }
        return logger ;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Return the XmlWrapperProvider from the "external factory" or the default one
     * 
     * @return
     */
    public IXmlWrapperProvider getXmlWrapperProvider()
    {
        IXmlWrapperProvider wrapperProvider = null;
        //--- Use the external main factory
        if (_externalFactory != null)
        {
            //--- Get the external "Screen Data Provider"
            wrapperProvider = _externalFactory.getXmlWrapperProvider();
            if (wrapperProvider != null)
            {
                trace("getXmlWrapperProvider() : OK");
            }
            else
            {
                return _xmlWrapperProvider; // v 1.0.0
                //error("getXmlWrapperProvider() : cannot get provider ");
            }
        }
        else
        {
            error("getXmlWrapperProvider() : external factory is null");
        }
        return wrapperProvider;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the Service provider from the "external factory" or the default one
     * 
     * @return
     */
    public IServiceProvider getServiceProvider()
    {
        IServiceProvider provider = null;
        //--- Use the external main factory
        if (_externalFactory != null)
        {
            //--- Get the external "Screen Data Provider"
            provider = _externalFactory.getServiceProvider();
            if (provider != null)
            {
                trace("getServiceProvider() : OK");
            }
            else
            {
                return _serviceProvider; // v 1.0.5
                //error("getServiceProvider() : cannot get provider ");
            }
        }
        else
        {
            error("getServiceProvider() : external factory is null");
        }
        return provider;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenProvider from the "external factory" or the default one
     * @return
     * @since 1.0.5
     */
    private IScreenProvider getScreenProvider()
    {
        //--- Use the external main factory
        if ( _externalFactory != null )
        {
            IScreenProvider provider = _externalFactory.getScreenProvider();
            if ( provider != null )
            {
                trace("getScreenProvider() : OK");
                return provider ;
            }
            else
            {
                return _screenProvider ; 
            }
        }
        else
        {
            error("getScreenProvider() : external factory is null");
        }
        return null ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenDataProvider from the "external factory" or the default one
     * @return
     * @since 1.0.5
     */
    private IScreenDataProvider getScreenDataProvider()
    {
        //--- Use the external main factory
        if ( _externalFactory != null )
        {
            IScreenDataProvider provider = _externalFactory.getScreenDataProvider();
            if ( provider != null )
            {
                trace("getScreenDataProvider() : OK");
                return provider ;
            }
            else
            {
                return _screenDataProvider ;
            }
        }
        else
        {
            error("getScreenProvider() : external factory is null");
        }
        return null;
    }
}