package org.objectweb.telosys.dal.dao;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * The context of a query ( contains the PreparedStatement and the ResultSet of the query )<br>
 * This is an internal framework class, designed to be used by the StandardDAO and all its descendents
 * 
 * @author Laurent GUERIN
 * 
 */
public class QueryContext extends TelosysObject
{
    //============================================================================
    // Constants
    //============================================================================
    public  static final boolean  RETURN_GENERATED_KEYS  = true;

    private static final int      BLOB_BUFFER_SIZE = 1024 ; // 1 kb            

    //============================================================================
    // Instance Attributes
    //============================================================================

    private PreparedStatement _ps                     = null;

    private ResultSet         _rs                     = null;

    private boolean           _bReturnGeneratedKeys   = false ;

    //============================================================================
    // Constructor(s)
    //============================================================================
    /**
     * Constructs a new query context for the given SQL request <br>
     * ( get a PreparedStatement )
     * 
     * @param con
     * @param sSql
     * @throws SQLException
     */
    public QueryContext(Connection con, String sSql) throws SQLException
    {
        super();
        this._ps = con.prepareStatement(sSql);
        this._rs = null;
    }

    //----------------------------------------------------------------------------
    /**
     * Constructs a new query context for the given SQL request <br>
     * ( get a PreparedStatement )
     * 
     * @param con
     * @param sSql
     * @param bReturnGeneratedKeys use RETURN_GENERATED_KEYS to return the generate key (useful for insertKeyGen) 
     * @throws SQLException
     */
    public QueryContext(Connection con, String sSql, boolean bReturnGeneratedKeys) throws SQLException
    {
        super();
        _bReturnGeneratedKeys = bReturnGeneratedKeys ;
        if ( _bReturnGeneratedKeys )
        {
            //--- Special Prepared Statement to retrieve Generated Keys
            this._ps = con.prepareStatement(sSql, Statement.RETURN_GENERATED_KEYS);
        }
        else
        {
            //--- Standard Prepared Statement
            this._ps = con.prepareStatement(sSql);
        }
        this._rs = null;
    }

    //============================================================================
    // Methods
    //============================================================================
    /**
     * Call the "executeUpdate" method of the current PreparedStatement 
     * @return
     * @throws SQLException
     */
    protected int executeUpdate() throws SQLException
    {
        // _rs = null ;
        return _ps.executeUpdate();
    }

    //----------------------------------------------------------------------------
    /**
     * Call the "executeQuery" method of the current PreparedStatement 
     * @return
     * @throws SQLException
     */
    protected ResultSet executeQuery() throws SQLException
    {
        _rs = _ps.executeQuery();
        return _rs;
    }
    
    //----------------------------------------------------------------------------
    /**
     * Executes an INSERT with KEY GENERATION  ( Autoincrement Key )
     * @return
     * @throws SQLException
     */
    protected Long executeInsertKeyGen() throws SQLException
    {
        Long generatedKey = null;
        //long lGeneratedKey = 0L;
        //--- Execute the SQL INSERT via PreparedStatement  
        _ps.executeUpdate();
        //--- If the Prepared Statement is built to retrieve generated keys 
        if ( _bReturnGeneratedKeys )
        {            
            //--- Retrieve the generated key 
	        ResultSet rs = _ps.getGeneratedKeys();
	        if ( rs.next() )
	        {
	            long lGeneratedKey = rs.getLong(1);
	            generatedKey = new Long( lGeneratedKey );
	        }
	        rs.close();
        }
        return generatedKey ;
    }

    //----------------------------------------------------------------------------
    /**
     * Close the ResultSet <br>
     * Doesn't throw Exception (just trace the error)
     */
    private void closeResultSet()
    {
        if (_rs != null)
        {
            try
            {
                _rs.close();
            } catch (SQLException e)
            {
                error("Cannot close the ResultSet ! SQLException : " + e.getMessage());
            }
        }
    }

    //----------------------------------------------------------------------------
    /**
     * Close the PreparedStatement <br>
     * Doesn't throw Exception (just trace the error)
     */
    private void closePreparedStatement()
    {
        if (_ps != null)
        {
            try
            {
                _ps.close();
            } catch (SQLException e)
            {
                error("Cannot close the PreparedStatement ! SQLException : " + e.getMessage());
            }
        }
    }

    //----------------------------------------------------------------------------
    /**
     * Close the PreparedStatement AND the ResultSet <br>
     * Doesn't throw Exception (just trace the error)
     */
    protected void close()
    {
        closeResultSet();
        closePreparedStatement();
    }

    //============================================================================
    // METHODES DE VALORISATION DES PARAMETRES DU "PREPARED STATEMENT"
    // Les exceptions sont gres au niveau des mthodes "executeSqlXxxx"
    //============================================================================
    
    /**
     * Set a generic Object parameter in the PreparedStatement <br>
     * A cast is done depending on the object's class.
     * @param i
     * @param oParam
     * @throws TelosysException
     */
    /* package */ void setParam(int i, Object oParam) throws TelosysException
    {
        if (_ps == null)
        {
            throw new TelosysException("setParam : _ps is null !");
        }
        else
        {
            try
            {
                if (oParam instanceof Boolean)
                {
                    setParamBoolean(i, ((Boolean) oParam).booleanValue());
                }
                //--- Integer Wrappers
                else if (oParam instanceof Integer)
                {
                    setParamInt(i, ((Integer) oParam).intValue());
                }
                else if (oParam instanceof Byte)
                {
                    setParamByte(i, ((Byte) oParam).byteValue());
                }
                else if (oParam instanceof Short)
                {
                    setParamShort(i, ((Short) oParam).shortValue());
                }
                else if (oParam instanceof Long)
                {
                    setParamLong(i, ((Long) oParam).longValue());
                }
                //--- Float/Double Wrappers
                else if (oParam instanceof Float)
                {
                    setParamFloat(i, ((Float) oParam).floatValue());
                }
                else if (oParam instanceof Double)
                {
                    setParamDouble(i, ((Double) oParam).doubleValue());
                }
                //--- Standards Objects
                else if (oParam instanceof String)
                {
                    setParamString(i, (String) oParam);
                }
                else if (oParam instanceof java.util.Date)
                {
                    setParamDate(i, (java.util.Date) oParam);
                }
                else if (oParam instanceof java.math.BigDecimal)
                {
                    setParamBigDecimal(i, (java.math.BigDecimal) oParam);
                }
                //--- Others
                else
                {
                    _ps.setObject(i, oParam);
                }
            } catch (SQLException e)
            {
                throw new TelosysException("setParam", e);
            }
        }
    }

    //-------------------------------------------------------------
    /**
     * Check PreparedStatement is useable, throws an exception if not
     * @throws TelosysException
     */
    private void checkPreparedStatement() throws TelosysException
    {
        if (_ps == null)
        {
            throw new TelosysException("PreparedStatement '_ps' is null !");
        }
    }

    //-------------------------------------------------------------
    // INT
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param iValue
     * @throws TelosysException
     */
    public void setParamInt(int i, int iValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setInt(i, iValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamInt", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param obj
     * @throws TelosysException
     */
    public void setParamInt(int i, java.lang.Integer obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setInt(i, obj.intValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.INTEGER );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamInt", e);
        }
    }

    //-------------------------------------------------------------
    // BYTE
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param value
     * @throws TelosysException
     */
    public void setParamByte(int i, byte value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setByte(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamByte", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param obj
     * @throws TelosysException
     */
    public void setParamByte(int i, java.lang.Byte obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setByte(i, obj.byteValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.TINYINT ); // JDBC : "TINYINT" => getByte/setByte
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamByte", e);
        }
    }

    //-------------------------------------------------------------
    // SHORT
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param value
     * @throws TelosysException
     */
    public void setParamShort(int i, short value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setShort(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamShort", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param obj
     * @throws TelosysException
     */
    public void setParamShort(int i, java.lang.Short obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setShort(i, obj.shortValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.SMALLINT ); // JDBC : "SMALLINT" => getShort/setShort
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamShort", e);
        }
    }

    //-------------------------------------------------------------
    // LONG
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param value
     * @throws TelosysException
     */
    public void setParamLong(int i, long value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setLong(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamLong", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param obj
     * @throws TelosysException
     */
    public void setParamLong(int i, java.lang.Long obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setLong(i, obj.longValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.BIGINT ); // JDBC : "BIGINT" => getLong/setLong
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamLong", e);
        }
    }

    //-------------------------------------------------------------
    // FLOAT
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param fValue
     * @throws TelosysException
     */
    public void setParamFloat(int i, float fValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setFloat(i, fValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamFloat", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param obj
     * @throws TelosysException
     */
    public void setParamFloat(int i, Float obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setFloat(i, obj.floatValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.FLOAT );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamFloat", e);
        }
    }

    //-------------------------------------------------------------
    // DOUBLE
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param dValue
     * @throws TelosysException
     */
    public void setParamDouble(int i, double dValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setDouble(i, dValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDouble", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param obj
     * @throws TelosysException
     */
    public void setParamDouble(int i, Double obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setDouble(i, obj.doubleValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.DOUBLE );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDouble", e);
        }
    }


    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param value
     * @throws TelosysException
     */
    public void setParamBigDecimal(int i, BigDecimal value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setBigDecimal(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBigDecimal", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param sValue
     * @throws TelosysException
     */
    public void setParamString(int i, String sValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setString(i, sValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamString", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement <br>
     * Converts the given java.util.Date to java.sql.Date and set it as Date parameter
     * @param i
     * @param dateValue
     * @throws TelosysException
     */
    public void setParamDate(int i, java.util.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if (dateValue != null)
            {
                _ps.setDate(i, new java.sql.Date(dateValue.getTime()));
            }
            else
            {
                _ps.setDate(i, null);
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDate", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement <br>
     * Converts the given java.util.Date to java.sql.Timestamp and set it as Timestamp parameter
     * @param i
     * @param dateValue
     * @throws TelosysException
     */
    public void setParamTimestamp(int i, java.util.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if (dateValue != null)
            {
                _ps.setTimestamp(i, new java.sql.Timestamp(dateValue.getTime()));
            }
            else
            {
                _ps.setTimestamp(i, null);
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTimestamp", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param timestamp
     * @throws TelosysException
     */
    public void setParamTimestamp(int i, java.sql.Timestamp timestamp ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setTimestamp(i, timestamp);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTimestamp", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement <br>
     * Converts the given java.util.Date to java.sql.Time and set it as Time parameter
     * @param i
     * @param dateValue
     * @throws TelosysException
     */
    public void setParamTime(int i, java.util.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if (dateValue != null)
            {
                _ps.setTime(i, new java.sql.Time(dateValue.getTime()));
            }
            else
            {
                _ps.setTime(i, null);
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTime", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param time
     * @throws TelosysException
     */
    public void setParamTime(int i, java.sql.Time time ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setTime(i, time);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTime", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param bytes
     * @throws TelosysException
     */
    public void setParamBytes(int i, byte[] bytes ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setBytes(i, bytes);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBytes", e);
        }
    }

    //-------------------------------------------------------------
    // BOOLEAN
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param bValue
     * @throws TelosysException
     */
    public void setParamBoolean(int i, boolean bValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
			_ps.setBoolean(i, bValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setBoolean", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param obj
     * @throws TelosysException
     */
    public void setParamBoolean(int i, java.lang.Boolean obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setBoolean(i, obj.booleanValue() );
            }
            else
            {
                _ps.setNull(i, java.sql.Types.BOOLEAN );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setBoolean/setNull", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement <br>
     * For a boolean stored as an String value in the table
     * @param i
     * @param bValue the boolean value
     * @param sTrueValue the String value to use if the boolean is TRUE ( e.g. "1" or "T" )
     * @param sFalseValue the String value to use if the boolean is FALSE ( e.g. "0" or "F" )
     * @throws TelosysException
     */
    public void setParamBooleanAsString(int i, boolean bValue, String sTrueValue, String sFalseValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            String sValue = ( bValue ? sTrueValue : sFalseValue ) ;
            _ps.setString(i, sValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setString", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement <br>
     * For a boolean stored as an integer value in the table
     * @param i
     * @param bValue the boolean value
     * @param iTrueValue the integer value to use if the boolean is TRUE ( e.g. 1 )
     * @param iFalseValue the integer value to use if the boolean is FALSE ( e.g. 0 )
     * @throws TelosysException
     */
    public void setParamBooleanAsInt(int i, boolean bValue, int iTrueValue, int iFalseValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            int iValue = ( bValue ? iTrueValue : iFalseValue ) ;
            _ps.setInt(i, iValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setInt", e);
        }
    }

    //-------------------------------------------------------------
    // CLOB 
    //-------------------------------------------------------------
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param param the CLOB object
     * @throws TelosysException
     */
    public void setParamClob(int i, Clob param ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setClob(i, param);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamClob", e);
        }
    }
    
    //-------------------------------------------------------------
    // BLOB
    //-------------------------------------------------------------
    //private static final ByteArrayInputStream VOID_STREAM = new ByteArrayInputStream(new byte[0]);
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param bytes the BLOB content (binary : array of bytes)
     * @throws TelosysException
     */
    public void setParamBlob(int i, byte[] bytes ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( bytes != null )
            {
                _ps.setBinaryStream(i, new ByteArrayInputStream (bytes), bytes.length );
            }
            else
            {
                //_ps.setBinaryStream(i, null, 0 );
                _ps.setNull(i, java.sql.Types.LONGVARBINARY );                
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBlob", e);
        }
    }
    
    /**
     * Set a parameter in the PreparedStatement
     * @param i
     * @param param the BLOB object
     * @throws TelosysException
     */
    public void setParamBlob(int i, Blob param ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setBlob(i, param);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBlob", e);
        }
    }


    //============================================================================
    // Mthodes de rcupration des donnes  partir du "RESULT SET"
    //============================================================================
    /**
     * Check ResultSet is useable, throws an exception if not
     * @throws TelosysException
     */
    private void checkResultSet() throws TelosysException
    {
        if (_rs == null)
        {
            throw new TelosysException("ResultSet '_rs' is null !");
        }
    }

    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public String getResultString(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getString(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultString", e);
        }
    }

    //-------------------------------------------------------------
    // INT
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public int getResultInt(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getInt(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultInt", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Integer getResultIntObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Integer )
                {
                    return (Integer) obj;
                }
                else
                {
                    // Not an error
                    // ie Oracle "NUMBER" col type returns "java.math.BigDecimal" or "java.lang.Double"
                    // warn("getResultIntObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Integer( _rs.getInt(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultIntObj", e);
        }
    }
    
    //-------------------------------------------------------------
    // BYTE
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public byte getResultByte(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getByte(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultByte", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Byte getResultByteObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Byte )
                {
                    return (Byte) obj;
                }
                else
                {
                    // Not an error
                    // ie Oracle "NUMBER" col type returns "java.math.BigDecimal" or "java.lang.Double"
                    // warn("getResultByteObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Byte( _rs.getByte(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultByteObject", e);
        }
    }
        
    //-------------------------------------------------------------
    // SHORT
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public short getResultShort(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getShort(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultShort", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Short getResultShortObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Short )
                {
                    return (Short) obj;
                }
                else
                {
                    // Not an error
                    // ie Oracle "NUMBER" col type returns "java.math.BigDecimal" or "java.lang.Double"
                    // warn("getResultShortObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Short( _rs.getShort(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultShortObject", e);
        }
    }
        
    
    //-------------------------------------------------------------
    // LONG
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public long getResultLong(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getLong(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultLong", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Long getResultLongObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Long )
                {
                    return (Long) obj;
                }
                else
                {
                    // Not an error
                    // ie Oracle "NUMBER" col type returns "java.math.BigDecimal" or "java.lang.Double"
                    // warn("getResultLongObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Long( _rs.getLong(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultLongObject", e);
        }
    }
        

    //-------------------------------------------------------------
    // FLOAT
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public float getResultFloat(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getFloat(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultFloat", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Float getResultFloatObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Float )
                {
                    return (Float) obj;
                }
                else
                {
                    // Not an error
                    // ie Oracle "NUMBER" col type returns "java.math.BigDecimal" or "java.lang.Double"
                    // warn("getResultFloatObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Float( _rs.getFloat(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultFloatObject", e);
        }
    }
    
    //-------------------------------------------------------------
    // DOUBLE
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public double getResultDouble(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getDouble(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDouble", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Double getResultDoubleObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Double ) 
                {
                    return (Double) obj;
                }
                else
                {
                    // Not an error
                    // ie Oracle "NUMBER" col type returns "java.math.BigDecimal" or "java.lang.Double"                  
                    // warn("getResultDoubleObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Double( _rs.getDouble(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDoubleObject", e);
        }
    }
    
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public BigDecimal getResultBigDecimal(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getBigDecimal(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBigDecimal", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet <br>
     * Returns a java.util.Date instance created with the java.sql.Date returned by the ResultSet
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public java.util.Date getResultDate(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            java.sql.Date date = _rs.getDate(iPosition);
            if ( date != null )
            {
                return new java.util.Date(date.getTime());
            }
            else
            {
                return null ;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDate", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet <br>
     * Returns the java.sql.Date returned by the ResultSet
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public java.sql.Date getResultDateSql(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getDate(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDateSql", e);
        }
    }    
    
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public java.sql.Time getResultTime(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getTime(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTime", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet <br>
     * For a TIME value in the table, returned as java.util.Date
     * 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public java.util.Date getResultTimeAsDate(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            java.sql.Time time = _rs.getTime(iPosition);
            if ( time != null )
            {
                return new java.util.Date(time.getTime());
            }
            else
            {
                return null ;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTimeAsDate", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public java.sql.Timestamp getResultTimestamp(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getTimestamp(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTimestamp", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet <br>
     * For a TIMESTAMP value in the table, returned as java.util.Date
     * 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public java.util.Date getResultTimestampAsDate(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            java.sql.Timestamp ts = _rs.getTimestamp(iPosition);
            if ( ts != null )
            {
                return new java.util.Date(ts.getTime());
            }
            else
            {
                return null ;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTimestampAsDate", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public byte[] getResultBytes(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getBytes(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBytes", e);
        }
    }

    //-------------------------------------------------------------
    // BOOLEAN
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public boolean getResultBoolean(int iPosition) throws TelosysException
    {
        boolean bRet = false;
        try
        {
            checkResultSet();
            bRet = _rs.getBoolean(iPosition); // return FALSE if "SQL NULL"
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBoolean --> rs.getBoolean", e);
        }
        return bRet;
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Boolean getResultBooleanObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition); // if "SQL NULL" -> return null
            if ( obj != null )
            {
                if ( obj instanceof Boolean )
                {
                    return (Boolean) obj;
                }
                else
                {
                    // warn("getResultBooleanObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    //return new Boolean( _rs.getBoolean(iPosition) );
                    return Boolean.valueOf( _rs.getBoolean(iPosition) ) ; // better space and time perf.
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBooleanObject", e);
        }
    }
        
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet <br>
     * For a boolean stored as an integer value in the table
     * @param iPosition
     * @param iTrueValue the integer value thar means "TRUE" ( e.g. 1 )
     * @return true if the ResultSet value equals the iTrueValue parameter, else false
     * @throws TelosysException
     */
    public boolean getResultBooleanFromInt(int iPosition, int iTrueValue) throws TelosysException
    {
        boolean bRet = false;
        try
        {
            checkResultSet();
            int iValue = _rs.getInt(iPosition); // return 0 if NULL
            bRet = ( iValue == iTrueValue ? true : false);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBooleanFromInt --> rs.getInt", e);
        }
        return bRet;
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet <br>
     * For a boolean stored as an String value in the table
     * @param iPosition
     * @param sTrueValue the String value thar means "TRUE" ( e.g. "1" or "T" or "true" )
     * @return true if the ResultSet value equals the sTrueValue parameter, else false
     * @throws TelosysException
     */
    public boolean getResultBooleanFromString(int iPosition, String sTrueValue) throws TelosysException
    {
        boolean bRet = false;
        try
        {
            checkResultSet();
            String s = _rs.getString(iPosition);
            if ( s != null && sTrueValue != null)
            {
                if ( s.equals(sTrueValue) )
                {
                    bRet = true ;
                }
            }
            else
            {
                if ( s == null && sTrueValue == null )
                {
                    bRet = true ;
                }
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBooleanFromString --> rs.getString", e);
        }
        return bRet;
    }

    //-------------------------------------------------------------
    // CLOB 
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Clob getResultClob(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getClob(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultClob", e);
        }
    }
    
    //-------------------------------------------------------------
    // BLOB 
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return the binary content of the BLOB ( can be null )
     * @throws TelosysException
     */
    public byte[] getResultBlobAsByteArray(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            //--- 1) Get the InputStream
            InputStream is = _rs.getBinaryStream(iPosition);
            
            //--- 2) Load the InputStream content in a buffer
            if (is != null)
            {
	            ByteArrayOutputStream os = new ByteArrayOutputStream(BLOB_BUFFER_SIZE);
	            byte [] buffer = new byte [BLOB_BUFFER_SIZE] ; 
	            int i = 0;
	            try
	            {
	                while ( ( i = is.read(buffer) ) > 0 )
	                {
	                    os.write(buffer, 0, i);                
	                }
	            } catch (IOException e)
	            {
	                error("getResultBlobAsByteArray ! IOException : " + e.getMessage());
	            }
	            return os.toByteArray();
            }
            else
            {
                // if the column value is NULL in the database, the InputStream returned is null 
                //return new byte[0] ;
                return null ; // Return null for NULL value ( not the same as "void" )
            }
            
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBlobAsByteArray", e);
        }
    }
    //-------------------------------------------------------------
    /**
     * Returns the value of the designated column in the current row of the ResultSet 
     * @param iPosition
     * @return
     * @throws TelosysException
     */
    public Blob getResultBlob(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getBlob(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBlob", e);
        }
    }
    //-------------------------------------------------------------


    //-------------------------------------------------------------
    //-------------------------------------------------------------
    /**
     * Moves the cursor down one row from its current position in the ResultSet<br>
     * Calls the "next" method of the current ResultSet 
     * @return
     * @throws TelosysException
     */
    protected boolean nextResult() throws TelosysException
    {
        if (_rs == null)
        {
            throw new TelosysException("nextResult: _rs is null !");
        }
        else
        {
            try
            {
                return _rs.next();
            } catch (SQLException e)
            {
                throw new TelosysException("nextResult", e);
            }
        }
    }

    //-------------------------------------------------------------
}