package org.objectweb.telosys.dal.sql;

import java.io.Serializable;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *  
 */
public class Criterion implements Serializable
{    
    private String  _sSqlExpression = null; // SQL where criterion ( ie : "agency = ?" )

    private int     _iParamType     = ParamType.UNKNOWN ;   // see ParamType class

    private String  _sParamValue    = null;

    private Object  _oParamValue    = null;

    private boolean _bUsed          = false;

    private String  _sParamName     = null;  // v 1.1.0

    /**
     * Creates a new Criterion object
     * @param sSqlExpression the SQL criterion expression ( eg : "agency = ?", "code > ?", ... )
     * @param iType the criterion parameter type ( eg : STRING, INTEGER, ... ) ( see "ParamType" )
     */
    public Criterion(String sSqlExpression, int iType)
    {
        super();
        _sSqlExpression = sSqlExpression ;
        _iParamType = iType;
        _sParamName = null;
    }

    /**
     * Creates a new Criterion object
     * @param sSqlExpression the SQL criterion expression ( eg : "agency = ?", "code > ?", ... )
     * @param iType the criterion parameter type ( eg : STRING, INTEGER, ... ) ( see "ParamType" )
     * @param sParamName the name of the parameter associated with the criterion 
     * @since 1.1.0
     */
    public Criterion(String sSqlExpression, int iType, String sParamName)
    {
        super();
        _sSqlExpression = sSqlExpression ;
        _iParamType = iType;
        _sParamName = sParamName;
    }

    /**
     * Returns the SQL expression of this criterion ( eg : "agency = ?", "name like ?", ... )
     * @return
     */
    public String getSqlExpression()
    {
        return _sSqlExpression;
    }

    /**
     * Returns the type of the parameter associated with this criterion <br>
     * See the constants in "ParamType" 
     * @return
     */
    public int getParamType()
    {
        return _iParamType;
    }

    /**
     * Returns the name of the parameter associated with this criterion <br>
     * @return
     */
    public String getParamName()
    {
        return _sParamName ;
    }

    /**
     * Returns the parameter value associated with this criterion 
     * @return
     */
    public String getParamValueAsString()
    {
        return _sParamValue ;
    }

    /**
     * Returns the parameter value associated with this criterion 
     * @return
     */
    public Object getParamValueAsObject()
    {
        return _oParamValue ;
    }

    /**
     * Declares this criterion as to be used with the given parameter value
     * @param sParamVal
     */
    public void useWithValue(String sParamVal)
    {
        _oParamValue = null ;
        _sParamValue = null ;
        try
        {
            _oParamValue = Cast.toObject(sParamVal, _iParamType) ;
            _sParamValue = sParamVal;
        } catch (TelosysException e)
        {
            throw new TelosysRuntimeException("Criteria : useWithValue(" + sParamVal + ") invalid parameter, type = " + _iParamType );
        }
        _bUsed = true;
    }

    /**
     * Declare this criterion as not to be used 
     */
    public void doNotUse()
    {
        _oParamValue = null ;
        _sParamValue = null;
        _bUsed = false;
    }

    /**
     * Returns true if this criterion is used in the SQL request
     * @return
     */
    public boolean isUsed()
    {
        return _bUsed ;
    }
}