package org.objectweb.telosys.dal.sql;

import java.util.*;
import java.sql.*;

/**
 * 
 * The standard connection pool 
 *
 * This class is a "ConnectionProvider"
 *  
 * @author Laurent GUERIN
 *
 */
public class SqlConnectionPool extends SqlConnectionProvider
{
    private Vector _vectConnections = null; // The set of connections 

    private int    _iInitialSize    = 10;   // The initial size of the pool

    //-----------------------------------------------------------------------------
    /**
     * Constructor 
     * @param sName the connection provider name
     * @param sDriver the JDBC driver to use
     * @param sUrl the JDBC URL
     * @param prop the connection properties
     * @param iIsolationLevel the "isolation level"
     * @param iSize the initial size of the pool
     */
    public SqlConnectionPool(String sName, String sDriver, String sUrl, Properties prop, int iIsolationLevel, int iSize)
    {
        //trace ( "Constructor ") ;
        super(sName, sDriver, sUrl, prop, iIsolationLevel);
        trace("Constructor ");
        _iInitialSize = iSize;
        _vectConnections = new Vector(_iInitialSize);

        //--- Creation des N connexions du pool
        createConnections();
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.SqlConnectionProvider#isPool()
     */
    public boolean isPool()
    {
        return true; // Yes it's a pool
    }

    //-----------------------------------------------------------------------------
    // Cration et ouverture de N connexions et stockage dans le pool
    //-----------------------------------------------------------------------------
    private synchronized void createConnections()
    {
        trace("openConnections(" + _iInitialSize + ") ...");

        for ( int i = 0 ; i < _iInitialSize ; i++ )
        {
            try
            {
                trace("Creating new DB connection ( #" + i + " ) ...");
                //--- Ajout dans le pool (vecteur)
                _vectConnections.addElement(createConnection());
                trace("Ok, new connection in the pool (new vector size = " + _vectConnections.size() + ")");
            } catch (Exception e)
            {
                error("Cannot create connection ( #" + i + " )");
            }
        }
    }

    //-----------------------------------------------------------------------------
    // Cration d'une connexion
    //-----------------------------------------------------------------------------
    private SqlConnection createConnection() throws Exception
    {
        Connection stdCon = createStandardJdbcConnection();
        return new SqlConnection(stdCon, this);
    }

    //-----------------------------------------------------------------------------
    // Finaliseur :
    // En cas de destruction de l'objet ConnectionPoolManager,
    // fermer toutes les connexions JDBC et vider le Vecteur
    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see java.lang.Object#finalize()
     */
    protected void finalize()
    {
        trace("finalize() : close JDBC connections ...");
        //--- Close all the JDBC connections
        closeAllConnections();
        //--- Try to call super.finalize()
        try
        {
            super.finalize();
        } catch (Throwable e)
        {
            error("Throwable in super.finalize() : " + e.getMessage() );
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * @return the number of free connections
     */
    public int getFreeConnections()
    {
        return count(false);
    }

    //-----------------------------------------------------------------------------
    /**
     * @return the number of used connections
     */
    public int getUsedConnections()
    {
        return count(true);
    }

    //-----------------------------------------------------------------------------
    private int count(boolean bInUse)
    {
        int n = 0;
        SqlConnection con = null;
        for ( int i = 0 ; i < _vectConnections.size() ; i++ )
        {
            con = (SqlConnection) _vectConnections.elementAt(i);
            if (bInUse)
            {
                if (con.isInUse())
                    n++;
            }
            else
            {
                if (!con.isInUse())
                    n++;
            }
        }
        return n;
    }

    //-----------------------------------------------------------------------------
    /**
     * @return the initial size of the pool
     */
    public int getInitialSize()
    {
        return _iInitialSize;
    }

    //-----------------------------------------------------------------------------
    /**
     * @return the current size of the pool
     */
    public int getCurrentSize()
    {
        return _vectConnections.size();
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.ConnectionProvider#getConnection()
     */
    public synchronized Connection getConnection() throws Exception
    {
        trace("getConnection()");

        SqlConnection con = findConnection();
        if (con == null) // Aucune connexion disponible dans le POOL
        {
            //--- Cration d'une nouvelle connexion
            con = createConnection();
            con.inUse(true);
            //--- Ajout dans le POOL
            _vectConnections.addElement(con);
            trace("Creating new DB connection #" + _vectConnections.size());
        }
        //trace("Returning the DB connection.");
        return con;
    }

    //-----------------------------------------------------------------------------
    private synchronized SqlConnection findConnection() throws Exception
    {
        SqlConnection con = null;
        //--- Parcours du POOL pour y trouver une connection disponible et
        // valide
        for ( int i = 0 ; i < _vectConnections.size() ; i++ )
        {
            con = (SqlConnection) _vectConnections.elementAt(i);
            //--- Si la connexion n'est pas dj utilise
            if (!con.isInUse())
            {
                //--- Si la connexion n'est pas bonne Connection is bad, remove
                // from pool
                //if ( isBad(con.connection) )
                if (con.isBad())
                {
                    trace("Bad connection found : #" + i);
                    con.tryToCloseJdbcConnection();

                    //--- Cration d'une nouvelle connexion
                    trace("Creating new DB connection #" + i);
                    con = createConnection();
                    if (con.isBad())
                    {
                        error("Cannot create a valid connection !");
                        throw new Exception("Cannot create a valid connection");
                    }
                    else
                    {
                        //--- Remplacement de la connexion dfectueuse
                        _vectConnections.set(i, con);
                    }
                }

                con.inUse(true);

                trace("Using existing DB connection #" + (i + 1));
                return con;
            }
        }
        return null; // Pas de connexion disponible dans le pool
    }

    //-----------------------------------------------------------------------------
    // Libration des ressources :
    // fermeture de toutes les connexions du POOL
    //-----------------------------------------------------------------------------
    private synchronized void closeAllConnections()
    {
        trace("closeAllConnections() : Nb = " + _vectConnections.size());
        SqlConnection con = null;
        //--- Fermeture des connexions
        for ( int i = 0 ; i < _vectConnections.size() ; i++ )
        {
            con = (SqlConnection) _vectConnections.elementAt(i);
            trace("Closing connection #" + i + " (" + con.toString() + ") ...");
            try
            {
                con.closeJdbcConnection();
                trace(" #" + i + ": Connection closed.");
            } catch (SQLException SqlEx)
            {
                trace(" #" + i + ": SQLException: Code=" + SqlEx.getErrorCode() + " - " + SqlEx.getSQLState());
            }
        }
        //--- Nettoyage du vecteur
        _vectConnections.clear();
        trace("END OF closeAllConnections() : Nb = " + _vectConnections.size());
    }
    
    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.ConnectionProvider#getType()
     */
    public int getType()
    {
        return ConnectionProvider.POOL ;
    }

}