package org.objectweb.telosys.rpl;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.util.web.GenericURI;

/**
 * The abstract/generic response class used as super class for "Screen Service Response"
 * and "Screen Action Response"
 * 
 * @author Laurent GUERIN
 * @since 1.0.5
 * 
 */
public abstract class GenericResponse extends TelosysObject
{
    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------

    private HttpServletRequest  _httpRequest     = null;

    private HttpServletResponse _httpResponse    = null; 
    
    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------

    private Hashtable       _htValues         = null; // Values to return in the response

    private LinkedList      _listExceptions   = null; // List of all the exceptions

    private LinkedList      _listErrors       = null; // List of all the errors

    private LinkedList      _listMessages     = null; // List of all the app. messages

    private int             _iReturnCode      = 0;   // default = "0" ( = OK )

    private String          _sReturnMessage   = "Ok"; // default = "Ok"
    
    private String          _sRedirection   = null ; // v 0.9.9

    //-----------------------------------------------------------------------------------------------
    // Constructor(s)
    //-----------------------------------------------------------------------------------------------
    /**
     * Constructs a new ServiceResponse object
     * @param httpRequest
     * @param httpResponse
     */
    public GenericResponse(HttpServletRequest httpRequest, HttpServletResponse httpResponse )
    {
        _httpRequest   = httpRequest;
        _httpResponse  = httpResponse;
    }

    //-----------------------------------------------------------------------------------------------
    // Methods
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "HttpServletRequest" object 
     * @return
     */
    public HttpServletRequest getHttpRequest()
    {
        return _httpRequest;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "HttpServletResponse" object 
     * @return
     * @since 1.0.5
     */
    public HttpServletResponse getHttpResponse() // v 1.0.5
    {
        return _httpResponse;
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set the "return code"
     * @param iCode
     */
    public void setReturnCode(int iCode)
    {
        _iReturnCode = iCode;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "return code"
     * @return
     */
    public int getReturnCode()
    {
        return _iReturnCode;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the main "return message"
     * @param sMsg
     */
    public void setReturnMessage(String sMsg)
    {
        _sReturnMessage = sMsg;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the main "return message"
     * @return
     */
    public String getReturnMessage()
    {
        return _sReturnMessage;
    }

    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    // Messages
    //-----------------------------------------------------------------------------------------------
    /**
     * Adds a message in the list
     * @param message
     */
    public void addMessage(ScreenMessage message)
    {
        if (_listMessages == null)
        {
            _listMessages = new LinkedList();
        }

        if (message != null)
        {
            _listMessages.add(message);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of messages 
     * @return
     */
    public int getNumberOfMessages()
    {
        if (_listMessages == null)
        {
            return 0;
        }
        else
        {
            return _listMessages.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the message associated with the given index
     * @param index
     * @return
     */
    public ScreenMessage getMessage(int index)
    {
        if (_listMessages == null)
        {
            return null;
        }
        else
        {
            Object obj = _listMessages.get(index);
            if (obj instanceof ScreenMessage)
            {
                return (ScreenMessage) obj;
            }
            else
            {
                error("getMessage(" + index + ") : not a ScreenMessage instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Errors
    //-----------------------------------------------------------------------------------------------
    /**
     * Adds an error in the list
     * @param error
     */
    public void addError(ScreenError error)
    {
        if (_listErrors == null)
        {
            _listErrors = new LinkedList();
        }

        if (error != null)
        {
            _listErrors.add(error);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of errors 
     * @return
     */
    public int getNumberOfErrors()
    {
        if (_listErrors == null)
        {
            return 0;
        }
        else
        {
            return _listErrors.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the error associated with the given index
     * @param index
     * @return
     */
    public ScreenError getError(int index)
    {
        if (_listErrors == null)
        {
            return null;
        }
        else
        {
            Object obj = _listErrors.get(index);
            if (obj instanceof ScreenError)
            {
                return (ScreenError) obj;
            }
            else
            {
                error("getError(" + index + ") : not a ScreenError instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Exceptions
    //-----------------------------------------------------------------------------------------------
    /**
     * Add an exception to the exceptions list
     * @param th
     */
    public void addException(Throwable th)
    {
        if (_listExceptions == null)
        {
            _listExceptions = new LinkedList();
        }

        if (th != null)
        {
            _listExceptions.add(th);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of exception(s) 
     * @return
     */
    public int getNumberOfExceptions()
    {
        if (_listExceptions == null)
        {
            return 0;
        }
        else
        {
            return _listExceptions.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the exception associated with the given id 
     * @param index
     * @return
     */
    public Throwable getException(int index)
    {
        if (_listExceptions == null)
        {
            return null;
        }
        else
        {
            Object obj = _listExceptions.get(index);
            if (obj instanceof Throwable)
            {
                return (Throwable) obj;
            }
            else
            {
                error("getException(" + index + ") : not a Throwable instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Values
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'String' value in the response (do nothing if the name or the value is null)
     * @param sName
     * @param sValue
     */
    public void setValue(String sName, String sValue)
    {
        if (_htValues == null)
        {
            _htValues = new Hashtable();
        }

        //--- NB : NullPointerException if the key or value is null.
        if (sName != null && sValue != null )
        {
            _htValues.put(sName, sValue);
        }
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set an 'int' value in the response 
     * @param sName
     * @param iValue
     */
    public void setValue(String sName, int iValue)
    {
        this.setValue(sName, ""+iValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'long' value in the response 
     * @param sName
     * @param lValue
     */
    public void setValue(String sName, long lValue)
    {
        this.setValue(sName, ""+lValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'short' value in the response 
     * @param sName
     * @param shValue
     */
    public void setValue(String sName, short shValue)
    {
        this.setValue(sName, ""+shValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'byte' value in the response 
     * @param sName
     * @param byValue
     */
    public void setValue(String sName, byte byValue)
    {
        this.setValue(sName, ""+byValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'float' value in the response 
     * @param sName
     * @param fValue
     */
    public void setValue(String sName, float fValue)
    {
        this.setValue(sName, ""+fValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'double' value in the response 
     * @param sName
     * @param dValue
     */
    public void setValue(String sName, double dValue)
    {
        this.setValue(sName, ""+dValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'boolean' value in the response 
     * @param sName
     * @param bValue
     */
    public void setValue(String sName, boolean bValue)
    {
        this.setValue(sName, ""+bValue );
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the response value associated with the given name
     * @param sName
     * @return
     */
    public String getValue(String sName)
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return (String) _htValues.get(sName);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of values stored in the response
     * @return
     */
    public int getNumberOfValues()
    {
        if (_htValues == null)
        {
            return 0;
        }
        else
        {
            return _htValues.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the values names 
     * @return
     */
    public Enumeration getValueNames()
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return _htValues.keys();
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------
    /**
     * Exposes an object in the standard request scope for future use in a view <br>
     * ( the object is stored as an attribute in the HttpServletRequest )
     * @param sName 
     * @param obj
     */
    public void expose(String sName, Object obj)
    {
        if ( _httpRequest != null )
        {
            _httpRequest.setAttribute(sName, obj);            
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Force the browser to redirect to a given URL after the AJAX call 
    //-----------------------------------------------------------------------------------------------
    /**
     * Force the client-side (the browser) to redirect to the given URL <br>
     * ( replace the current screen by the given URL )
     * @param sURL
     */
    public void forceRedirect ( String sURL )
    {
        if ( sURL != null )
        {
            if ( _httpResponse != null )
            {
                _sRedirection = sURL ;
                _httpResponse.setHeader(TelosysConst.TELOSYS_REDIRECT, _sRedirection );
            }
            else
            {
                throw new TelosysRuntimeException("forceRedirect : HttpResponse is null");
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Force the client-side (the browser) to redirect to the given generic URI <br>
     * ( replace the current screen by the given object URL )
     * @param uri
     */
    public void forceRedirect ( GenericURI uri )
    {
        if ( uri != null )
        {
            forceRedirect( uri.getFullURL( _httpRequest ) );
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the redirection URL set by 'forceRedirect' or null if none
     * @return
     */
    public String getRedirection()
    {
        return _sRedirection ;
    }


    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------

    public final static int RETURN      = 1 ;

    public final static int REDIRECTION = 2 ;

    public final static int ERRORS      = 4 ;
    
    public final static int EXCEPTIONS  = 8 ;
    
    public final static int MESSAGES    = 16 ;
    
    public final static int VALUES      = 32 ;
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Specific "toString" method designed to return then string part wanted
     * @param iFlag 
     * @return
     */
    public String toString(int iFlag)
    {
        StringBuffer sb = new StringBuffer(256);

        if ( ( iFlag & RETURN ) > 0 ) 
        {
            sb.append("\n . return : code='" + _iReturnCode + "' message='" + _sReturnMessage + "' " );
        }
        
        if ( ( iFlag & REDIRECTION ) > 0 ) 
        {
            sb.append("\n . redirection='" + _sRedirection + "'" );
        }

        if ( ( iFlag & ERRORS ) > 0 ) 
        {
	        sb.append("\n --- errors :" );
	        if ( _listErrors != null )
	        {
		        for ( int i = 0 ; i <_listErrors.size() ; i++ )
		        {
		            ScreenError error = (ScreenError) _listErrors.get(i);
		            sb.append( "\n . " + error );
		        }
	        }
	        else
	        {
	            sb.append( "\n   (no error)" );
	        }
        }
        
        if ( ( iFlag & EXCEPTIONS ) > 0 ) 
        {
	        sb.append("\n --- exceptions :" );
	        if ( _listExceptions != null )
	        {
		        for ( int i = 0 ; i <_listExceptions.size() ; i++ )
		        {
		            Throwable ex = (Throwable) _listExceptions.get(i);
		            sb.append( "\n . " + ex );
		        }
	        }
	        else
	        {
	            sb.append( "\n   (no exception)" );
	        }
        }
        
        if ( ( iFlag & MESSAGES ) > 0 ) 
        {
	        sb.append("\n --- messages :" );
	        if ( _listMessages != null )
	        {
		        for ( int i = 0 ; i <_listMessages.size() ; i++ )
		        {
		            ScreenMessage m = (ScreenMessage) _listMessages.get(i);
		            sb.append( "\n . " + m );
		        }
	        }
	        else
	        {
	            sb.append( "\n   (no exception)" );
	        }
        }
        
        if ( ( iFlag & VALUES ) > 0 ) 
        {
	        sb.append("\n --- values :" ); 
	        if ( _htValues != null )
	        {
	            Enumeration e = _htValues.keys();
	            while ( e.hasMoreElements() )
	            {
	                Object key = e.nextElement();
	                sb.append( "\n . '" + key + "' : '" +  _htValues.get(key) + "'" );
	            }
	        }
	        else
	        {
	            sb.append( "\n   (no value)" );
	        }
        }

        return sb.toString() ;
    }
    //-----------------------------------------------------------------------------------------------
}