package org.objectweb.telosys.rpl.xml;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.common.vo.InternalVOList;
import org.objectweb.telosys.rpl.ElementClassResolver;
import org.objectweb.telosys.rpl.ScreenElementProvider;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenRequestView;
import org.objectweb.telosys.util.BeanUtil;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;

/**
 * Event Handler for parsing XML ScreenRequest with a SAX 2 parser (can be used with JAXP 1.2) <br>
 * NB: This handler is not "thread safe" <br>
 * 
 * Optimized version (August 2007) with small footprint, <br>
 * including tag for bean attribute (<<attrib>>value<</attrib>>), usable since Telosys 0.9.8 
 * 
 * @author Laurent GUERIN
 * 
 */
//Changements SAX 2 (par rapport  SAX 1)
//. La classe "DefaultHandler" remplace "HandlerBase"
//. L'interface "Attributes" remplace "AttributeList"
//
//---------------------------------------------------------------------------
// XML TREE STRUCTURE ( limited to 5 levels ) :
//
// <request ...>  ( REQUEST_LEVEL )
//   <params>   ( PARAMS_LEVEL )
//     <param name="value" />
//   </params>
//   <views>    ( VIEWS_LEVEL )
//     <view name="..."  renderer="..." />
//   </views>
//   <data>  ( DATA_LEVEL )
//     <vo1 att1="..." att2=".." />  ( DATA_ELEMENT_LEVEL )
//     <vo2 att1="..." att2=".." />  ( DATA_ELEMENT_LEVEL )
//     <vo3 att1="..." att2=".." >   ( DATA_ELEMENT_LEVEL )
//       <att3><![CDATA[....]]></att3>
//     </vo3>
//   <volist1>   ( DATA_ELEMENT_LEVEL )
//     <vo att1="..." att2=".." />  (4)
//     <vo att1="..." att2=".." />  (4)
//       <att3><![CDATA[....]]></att3>  (5)
//   </volist1>
//   </data>
// </request>
//---------------------------------------------------------------------------
//
public final class ScreenRequestSaxHandler extends TelosysObject implements ContentHandler
{
    //============================================================================
    // CONSTANTS
    //============================================================================
    private final static int        MAX_LEVEL             = 5 ; // 1 to 5 
    
    private final static String     TAG_REQUEST           = "request";

    private final static String     TAG_PARAMS            = "params";

    private final static String     TAG_PARAM             = "param";

    private final static String     TAG_VIEWS             = "views";

    private final static String     TAG_VIEW              = "view";

    private final static String     TAG_DATA              = "data";

    //--- XML TREE TAGS LEVEL 
    private final static int        REQUEST_LEVEL         = 1;
    
    private final static int        PARAMS_LEVEL          = 2;

    private final static int        VIEWS_LEVEL           = 2;

    private final static int        DATA_LEVEL            = 2;

    private final static int        DATA_ELEMENT_LEVEL    = 3;

    
    //============================================================================
    // ATTRIBUTES 
    //============================================================================
    //--- Application Wrappers Provider
    private IXmlWrapperProvider     _wrapperProvider     = null;

    private boolean                 _bInTagParams        = false;

    private boolean                 _bInTagViews         = false;

    private boolean                 _bInTagData          = false;
    
    private int                     _indentLevel         = 0;
    
    private String                  _sLastTagOpened      = null; // NEW ( for "LONG TEXT" TAG )

    private StringBuffer            _sbCurrentTagContent = null ; // Not used in most cases => no creation

    //--- THE ScreenRequest Instance to populate form the XML flow 
    private ScreenRequest           _screenRequest       = null ;
    
    private int                     _iNbTagRequest       = 0;

    //--- USEFUL OBJECTS STACKS ( index = level )
    private IXmlWrapper[]           _wrappers = new IXmlWrapper[MAX_LEVEL+1] ;
    
    private Object[]                _beans    = new Object[MAX_LEVEL+1] ;
    
    private GenericVOList[]         _lists    = new GenericVOList[MAX_LEVEL+1] ;
    
    //============================================================================
    // CONSTRUCTOR(S)
    //============================================================================
    /**
     * Constructor
     * @param screenRequest the Screen Request to populate
     * @param wrapperProvider the XML wrappers provider
     */
    public ScreenRequestSaxHandler(ScreenRequest screenRequest, IXmlWrapperProvider wrapperProvider)
    {
        if ( screenRequest == null )
        {
            throw new TelosysRuntimeException("ScreenRequestSaxHandler constructor : ScreenRequest is null");
        }
        if ( wrapperProvider == null )
        {
            throw new TelosysRuntimeException("ScreenRequestSaxHandler constructor : WrapperProvider is null");
        }
        _screenRequest = screenRequest ;
        _wrapperProvider = wrapperProvider;
        for ( int i=0 ; i <= MAX_LEVEL ; i++ )
        {
            _wrappers[i] = null;
            _beans[i] = null;
            _lists[i] = null;
        }
    }

    //============================================================================
    // METHOD(S)
    //============================================================================
    /**
     * Returns the screen request instance built from the XML input stream
     * 
     * @return
     */
    public ScreenRequest getScreenRequest()
    {
        return _screenRequest;
    }
    
    //============================================================================
    // VO WRAPPERS STACK ( BY LEVEL )
    //============================================================================
    private void setWrapper(int iLevel, IXmlWrapper wrapper)
    {
        if ( iLevel > 0 && iLevel < _wrappers.length )
        {
            _wrappers[iLevel] = wrapper ;            
        }
    }
    private IXmlWrapper getWrapper(int iLevel)
    {
        if ( iLevel > 0 && iLevel < _wrappers.length )
        {
            return _wrappers[iLevel];            
        }
        return null ;
    }
    private IXmlWrapper getParentWrapper(int iLevel)
    {
        return getWrapper(iLevel-1) ;
    }
    
    //============================================================================
    // VO BEANS STACK ( BY LEVEL )
    //============================================================================
    private void setVOBean(int iLevel, Object bean)
    {
        if ( iLevel > 0 && iLevel < _beans.length )
        {
            _beans[iLevel] = bean ;            
        }
    }
    private Object getVOBean(int iLevel)
    {
        if ( iLevel > 0 && iLevel < _beans.length )
        {
            return _beans[iLevel];            
        }
        return null ;
    }
    private Object getParentVOBean(int iLevel)
    {
        return getVOBean(iLevel-1) ;
    }    
    
    //============================================================================
    // VO LISTS STACK ( BY LEVEL )
    //============================================================================
    private void setList(int iLevel, GenericVOList list)
    {
        if ( iLevel > 0 && iLevel < _lists.length )
        {
            _lists[iLevel] = list ;            
        }
    }
    private GenericVOList getList(int iLevel)
    {
        if ( iLevel > 0 && iLevel < _lists.length )
        {
            return _lists[iLevel];            
        }
        return null ;
    }
    private GenericVOList getParentList(int iLevel)
    {
        return getList(iLevel-1) ;
    }
    
    //============================================================================
    // SAX "Callback" methods (SAX events)
    //============================================================================

    //----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String)
     */
    public void endPrefixMapping(String s)
    {
    }

    //----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String, java.lang.String)
     */
    public void startPrefixMapping(String prefix, String uri)
    {
    }

    //----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int)
     */
    public void ignorableWhitespace(char[] ch, int start, int length)
    {
    }

    //----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String, java.lang.String)
     */
    public void processingInstruction(String target, String data)
    {
    }

    //----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char buf[], int offset, int len)
    {        
        trace("characters(buffer, " + offset + ", " + len + ") : LEVEL = " + _indentLevel);
        // Usefull only for data elements sub tags ( Rich Text Tag has content )
        if ( _bInTagData == true && _indentLevel > DATA_ELEMENT_LEVEL )
        {
	        // NB : this event can be called 1 or N times for the same tag 
	        // ==> concatenate buffers and set the content in the event "endElement" 
	        trace("characters(buffer, " + offset + ", " + len + ") : LEVEL = " + _indentLevel + " PROCESS ... ");
            if ( _sbCurrentTagContent == null )
            {
                _sbCurrentTagContent = new StringBuffer (len + 200);
            }
            _sbCurrentTagContent.append(buf, offset, len);
//	        //--- if the parent TAG has a specific Wrapper => it's a ValueObject 
//	        IXmlWrapper parentWrapper = getParentWrapper(_indentLevel);
//	        Object parentBean = getParentVOBean(_indentLevel);
//	        if ( parentWrapper != null && parentBean != null ) // is a bean child tag 
//	        {	            	            
//	            //--- Element field value : the tag content 
//	            String sFieldValue = new String(buf, offset, len);
//	            //--- Element field name : the current tag name 
//	            String sFieldName = _sLastTagOpened;
//	            //--- Set the element field value
//	            //parentWrapper.setXmlFieldValue(sFieldName, sFieldValue);
//	            parentWrapper.setXmlFieldValue(parentBean, sFieldName, sFieldValue);
//	            trace("characters(.,.,.) : setXmlFieldValue(" + sFieldName + "," + sFieldValue +")" );
//	        }
        }
    }

    //----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator)
     */
    public void setDocumentLocator(Locator locator)
    {
    }

    //----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String)
     */
    public void skippedEntity(String name)
    {
    }

    //----------------------------------------------------------------------------
    /* 
     * Beginning of XML document (before root tag opening)
     * throws SAXException
     * 
     * (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() // throws SAXException
    {
        trace("-> START DOCUMENT");
        //--- Init :
        _indentLevel = 0;
    }

    //----------------------------------------------------------------------------
    /* 
     * End of XML document (after root tag closing) 
     * throws SAXException
     * 
     * (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endDocument()
     */
    public void endDocument()
    {
        trace("-> END DOCUMENT");
        if (_indentLevel > 0)
        {
            error("Invalid XML document structure ! Unexpected end of document ( indentLevel > 0 )");
        }
    }

    //----------------------------------------------------------------------------
    /* 
     * New opening TAG (ie : "<tag ..." ) with its attributes
     * throws SAXException
     * 
     * (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(final String uri, final String sEltLocalName, final String qName, final Attributes attrs)
    {
        _indentLevel++;
        trace("startElement(" + uri + ", " + sEltLocalName + ", " + qName + ") : LEVEL = " + _indentLevel );
        _sLastTagOpened = sEltLocalName; // NEW for LONG TEXT TAG 
        String sTagName = sEltLocalName;
        if (sTagName == null)
        {
            error("Tag name is null !");
        }
        else
        {
            openTag(sTagName, _indentLevel, attrs);
        }
    }

    //----------------------------------------------------------------------------
    /* 
     * End of TAG ( ie : "...</tag>" or ".../>" in the current tag )
     * throws SAXException
     * 
     * (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(final String uri, final String sEltLocalName, final String qName)
    {
        String sTagName = sEltLocalName;

        trace("endElement(" + uri + ", " + sEltLocalName + ", " + qName + ")");
        closeTag(sTagName, _indentLevel);

        _indentLevel--; // Must be at the end of the method
        _sbCurrentTagContent = null ; // No current tag content 
    }
    
    //-----------------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------------
    /**
     * Processes the given opening tag <br>
     * Returns the XML Wrapper used for the given tag, or null if no wrapper needed.<br>
     * Method called by the XML SAX parser for each new tag ( SAX "startElement" event ) <br>
     * 
     * @param sTagName
     * @param iLevel
     * @param attributes
     * @return
     */
    private IXmlWrapper openTag(String sTagName, int iLevel, Attributes attributes) 
    {
        trace(" openTag(" + sTagName + ", " + iLevel + ")");
        // We don't know the type of tag ...
        setWrapper(iLevel, null);
        setVOBean(iLevel, null);
        setList(iLevel, null);

        //--- TAGS STANDARDS
        //-----------------------------------------------------------------------------------------------
        //  <request ... >  LEVEL 1
        //-----------------------------------------------------------------------------------------------
        if ( iLevel == REQUEST_LEVEL && sTagName.equals(TAG_REQUEST) )
        {
            processRequestTag(_screenRequest, attributes);
            if (_iNbTagRequest != 0)
            {
                error("Multiple tag <request> in XML stream !! ");
            }
            _iNbTagRequest++;
            return null; // No Wrapper
        }

        //-----------------------------------------------------------------------------------------------
        //  <params>  LEVEL 2
        // beginning of parameters list (no attribute => no wrapper)
        //-----------------------------------------------------------------------------------------------
        else if ( iLevel == PARAMS_LEVEL && sTagName.equals(TAG_PARAMS))
        {
            _bInTagParams = true;
            return null; // No Wrapper
        }
        
        //-----------------------------------------------------------------------------------------------
        //  <param ... >  LEVEL 3 
        //  parameter name and value
        //-----------------------------------------------------------------------------------------------
        else if ( _bInTagParams == true && sTagName.equals(TAG_PARAM))
        {
            processParamTag(_screenRequest, attributes);
            return null; // No Wrapper
        }

        //-----------------------------------------------------------------------------------------------
        //  <views>  LEVEL 2
        //  beginning of views list (no attribute => no wrapper)
        //-----------------------------------------------------------------------------------------------
        else if ( iLevel == VIEWS_LEVEL && sTagName.equals(TAG_VIEWS) )
        {
            _bInTagViews = true;
            return null; // No Wrapper
        }

        //-----------------------------------------------------------------------------------------------
        //  <view ... >  LEVEL 3 
        //-----------------------------------------------------------------------------------------------
        else if ( _bInTagViews == true && sTagName.equals(TAG_VIEW) )
        {
            processViewTag(_screenRequest, attributes);
            return null; // No Wrapper
        }

        //-----------------------------------------------------------------------------------------------
        //  <data ... >  LEVEL 2 
        //  no attribute => no wrapper
        //-----------------------------------------------------------------------------------------------
        else if ( iLevel == DATA_LEVEL && sTagName.equals(TAG_DATA) )
        {
            _bInTagData = true;
            ElementClassResolver.initScreenDataDefinition(_screenRequest.getScreenName());
            return null; // No Wrapper
        }

        //-----------------------------------------------------------------------------------------------
        // ScreenData elements tags ( first level in <data> section )  LEVEL "3" ( VO or VOList )
        //-----------------------------------------------------------------------------------------------
        else if ( _bInTagData == true && iLevel == DATA_ELEMENT_LEVEL )
        {
            //--- Is there an element with this name in the ScreenData definition ?
            Class elementClass = ElementClassResolver.getElementClass(_screenRequest.getScreenName(), sTagName);
            if (elementClass != null)
            {
                //--- Try to find a specific wrapper for this class 
                IXmlWrapper wrapper = _wrapperProvider.getWrapper(elementClass);
                if ( wrapper != null )
                {
                    //--- There's a specific wrapper => it's a specific VO TAG
                    Object bean = processBeanElementTag(sTagName, attributes, wrapper);
                    //--- Keep the current level wrapper and bean
                    setWrapper(iLevel, wrapper);
                    setVOBean(iLevel, bean);
                    //--- Store the VO in the ScreenRequest
                    _screenRequest.storeDataElement(sTagName, bean);
                }
                else
                {
	                //--- Is it a VOList class ?
	                if ( Tool.isVOListClass( elementClass ) )
	                {
	                    trace(" DATA ELEMENT " + sTagName + " class : " + elementClass.toString() + " : VOList ");
	                    GenericVOList volist = Tool.createVOListInstance(elementClass);
	                    //--- Keep the current level VOList
	                    setList(iLevel, volist);
	                    //--- Store the VOList in the ScreenRequest
	                    _screenRequest.storeDataElement(sTagName, volist);
	                }
	                else
	                {
	                    error ( "Unknown data tag '" + sTagName + "' class = '" + elementClass.getName() + "' (no wrapper and not a VOList) " );
	                }
                }
                return null ;
            }
            else
            {
                // Not an error : the VO is not used by the target ScreenData 
                //error ( "Unknown data element '" + sTagName + "' class not found." );
            }

            return null;
        }
        
        //-----------------------------------------------------------------------------------------------
        // LEVEL 4 : only 2 cases : Long Text Bean Attribute or VOList element 
        //-----------------------------------------------------------------------------------------------
        else if ( _bInTagData == true && iLevel == DATA_ELEMENT_LEVEL + 1 )
        {
            //--- Is the parent a VO ?
            if ( getParentWrapper(iLevel) != null ) // The parent is a VO
            {
                // The current tag is supposed to be a Long Text Bean Attribute 
                // Nothing to do (the parent wrapper will be used) 
                return null ;
            }
            
            //--- Is the parent a VO List ?
            GenericVOList parentList = getParentList(iLevel) ;
            if ( parentList != null ) // The parent is a VO List
            {
                // The current tag is supposed to be a VO bean
                Class elementClass = parentList.getElementType(); // the VO class
                if (elementClass != null)
                {
                    //--- Try to find a specific wrapper for the VO class 
                    IXmlWrapper wrapper = _wrapperProvider.getWrapper(elementClass);
                    if ( wrapper != null )
                    {
                        //--- There's a specific wrapper => it's a specific VO TAG
                        Object bean = processBeanElementTag(sTagName, attributes, wrapper);

                        //--- Keep the current level wrapper and bean
                        setWrapper(iLevel, wrapper);
                        setVOBean(iLevel, bean);
                        //--- Store the VO in the VOList
                        InternalVOList.addElement( parentList, bean );
                    }
                }
            }
            else
            {
                // ERROR invalid XML TREE
            }
        }
        
        //-----------------------------------------------------------------------------------------------
        // LEVEL 5 : only 1 case : Long Text Bean Attribute ( <volist> <vo> <voattrib>  )
        //-----------------------------------------------------------------------------------------------
//        else if ( _bInTagData == true && iLevel == DATA_ELEMENT_LEVEL + 2 )
//        {
//            // Nothing to do 
//            // The tag is a Long Text Bean Attribute
//        }
        return null;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Method called by the XML SAX parser for each end of tag <br>( SAX "endElement" event )
     * 
     * @param sTagName : the NAME of the closing tag 
     * @param iLevel : the LEVEL of the closing tag 
     */
    private void closeTag(String sTagName, int iLevel)
    {
        trace(" closeTag(" + sTagName + ", " + iLevel + ")");

        //--- TAGS STANDARDS
        //-----------------------------------------------------------------------------------------------
        //  </request ... >
        //-----------------------------------------------------------------------------------------------
        //        if (sTagName.equals("request"))
        //        {
        //            _screenRequest.endOfInitialization();
        //        }
        //        else if (sTagName.equals("params"))
        //-----------------------------------------------------------------------------------------------
        //  </params>
        //-----------------------------------------------------------------------------------------------
        if ( iLevel == PARAMS_LEVEL && sTagName.equals(TAG_PARAMS))
        {
			_bInTagParams = false;
        }
        //-----------------------------------------------------------------------------------------------
        //  </views>
        //-----------------------------------------------------------------------------------------------
        else if ( iLevel == VIEWS_LEVEL && sTagName.equals(TAG_VIEWS))
        {
            _bInTagViews = false;
        }
        //-----------------------------------------------------------------------------------------------
        //  </data>
        //-----------------------------------------------------------------------------------------------
        else if (iLevel == DATA_LEVEL && sTagName.equals(TAG_DATA))
        {
            _bInTagData = false;
        }
        //-----------------------------------------------------------------------------------------------
        //  Data Child Element Tag ( Rich Text Tag )
        //-----------------------------------------------------------------------------------------------
        else if ( iLevel > DATA_ELEMENT_LEVEL && _bInTagData )
        {
            if ( _sbCurrentTagContent != null )
            {
	            IXmlWrapper parentWrapper = getParentWrapper(iLevel);
	            Object parentBean = getParentVOBean(iLevel);
	            if ( parentWrapper != null && parentBean != null ) 
	            {
		            //--- Element field value : the tag content ( set by "characters" event ) 
		            String sFieldValue = _sbCurrentTagContent.toString() ;
		            //--- Element field name : the current tag name 
		            String sFieldName = _sLastTagOpened;
		            //--- Set the parent element field value
		            trace(" closeTag(" + sTagName + ", " + iLevel + ") : setXmlFieldValue(" + sFieldName + "," + sFieldValue +")" );	            
		            //parentWrapper.setXmlFieldValue(sFieldName, sFieldValue);
		            parentWrapper.setXmlFieldValue(parentBean, sFieldName, sFieldValue );
	            }
            }
        }
    }

    //============================================================================
    // FIXED TAGS PROCESSING : <request> <param> <view>
    //============================================================================
    //-----------------------------------------------------------------------------------------------
    // <request screen-name=".." screen-id=".." .... />
    //-----------------------------------------------------------------------------------------------
    private void processRequestTag(ScreenRequest screenRequest, Attributes attributes)
    {
        if (attributes != null)
        {
            for ( int i = 0 ; i < attributes.getLength() ; i++ ) // Pour chaque attribut
            {
                String sAttributeName = attributes.getLocalName(i);
                String sAttributeValue = attributes.getValue(i);
                trace("  processRequestTag() : attribute " + i + " : " + sAttributeName + " = \"" + sAttributeValue + "\"");
                processRequestField(screenRequest, sAttributeName, sAttributeValue);
            }
        }
    }
    
    private void processRequestField(ScreenRequest screenRequest, String sFieldName, String sFieldValue)
    {
        if (sFieldName.equalsIgnoreCase("screen-name"))
        {
            screenRequest.setScreenName(sFieldValue.trim());
        }
        else if (sFieldName.equalsIgnoreCase("screen-id"))
        {
            screenRequest.setScreenId(sFieldValue.trim());
        }
        else if (sFieldName.equalsIgnoreCase("action"))
        {
            screenRequest.setAction(sFieldValue.trim());
        }
        else if (sFieldName.equalsIgnoreCase("proc-name"))
        {
            screenRequest.setProcName(sFieldValue.trim());
        }
        else if (sFieldName.equalsIgnoreCase("elements"))
        {
            screenRequest.setElements(sFieldValue.trim());
        }
        else if (sFieldName.equalsIgnoreCase("set"))
        {
            screenRequest.setFlagSetData(sFieldValue.trim());
        }
        else if (sFieldName.equalsIgnoreCase("get"))
        {
            screenRequest.setFlagGetData(sFieldValue.trim());
        }
        else if (sFieldName.equalsIgnoreCase("clear"))
        {
            screenRequest.setFlagClearData(sFieldValue.trim());
        }
    }
    //-----------------------------------------------------------------------------------------------
    // <param name="value" />
    //-----------------------------------------------------------------------------------------------
    private void processParamTag(ScreenRequest screenRequest, Attributes attributes)
    {
        if (attributes != null)
        {
            for ( int i = 0 ; i < attributes.getLength() ; i++ )
            {
                String sAttributeName = attributes.getLocalName(i);
                String sAttributeValue = attributes.getValue(i);
                trace("  processParamTag() : attribute " + i + " : " + sAttributeName + " = \"" + sAttributeValue + "\"");
                //--- Store in the ScreenRequest
                screenRequest.setParameter( sAttributeName, sAttributeValue );
            }
        }
    }
    //-----------------------------------------------------------------------------------------------
    // <view name=".." renderer=".." .... />
    //-----------------------------------------------------------------------------------------------
    private void processViewTag(ScreenRequest screenRequest, Attributes attributes)
    {
        if (attributes != null)
        {
            if ( attributes.getLength() > 0 )
            {                
                //--- Create the View
	            ScreenRequestView view = new ScreenRequestView()  ;
                //--- Set the View attributes
	            for ( int i = 0 ; i < attributes.getLength() ; i++ )
	            {
	                String sAttributeName = attributes.getLocalName(i);
	                String sAttributeValue = attributes.getValue(i);
	                trace("  processRequestTag() : attribute " + i + " : " + sAttributeName + " = \"" + sAttributeValue + "\"");
	                processViewField(view, sAttributeName, sAttributeValue);
	            }
                //--- Store the View in the ScreenRequest
	            screenRequest.addView(view);
            }
            else
            {
                error("tag <view> : no attribute ");
            }
        }
    }
    private void processViewField(ScreenRequestView view, String sFieldName, String sFieldValue)
    {
		if (sFieldName.equals("name"))
		{
		    view.setName (sFieldValue);
			return;
		}
	    
		if (sFieldName.equals("renderer"))
		{
		    view.setRenderer (sFieldValue);
			return;
		}	    
    }
    
    //-----------------------------------------------------------------------------------------------
    // Value Object Tag :  <element att1=".." att2=".." .... /> 
    //-----------------------------------------------------------------------------------------------
    //private void processBeanElementTag(ScreenRequest screenRequest, Object bean, Attributes attributes, IXmlWrapper wrapper)
    private Object processBeanElementTag(String sTagName, Attributes attributes, IXmlWrapper wrapper)
    {
        //trace(" DATA ELEMENT " + sTagName + " class : " + elementClass.toString() + " : standard bean (VO)");
        trace("processBeanElementTag('"+sTagName+"',..,..) ");
        
        Object bean = null ;
        if (wrapper != null)
        {
            //--- (1) Bean creation ( delegates to the wrapper )
            try
            {
                bean = wrapper.createBean();
            } catch (Throwable t) // We are not sure of the wrapper quality ( external class ) 
            {
                throw new TelosysRuntimeException("Wrapper '"+wrapper.getClass()+"' cannot create bean (tag='"+sTagName+"')", t);
            }
            
            //--- ## v 0.9.9 (2008-03-30) --------------------------------------
            //--- (2) Bean initialization from the existing screen data element if any
            Object oScreenDataElement = null ;
            try
            {
                //--- Is there a current screen context data element for this tag ?  
                oScreenDataElement = ScreenElementProvider.getScreenDataElement( _screenRequest, sTagName ) ;                
            } catch (TelosysException e)
            {
                throw new TelosysRuntimeException("Cannot get screen data element for '"+sTagName+"' ", e);
            }
            if ( oScreenDataElement != null )
            {
                trace("processBeanElementTag('"+sTagName+"',..,..) : init bean from screen data element..");
                BeanUtil.shallowCopy(oScreenDataElement, bean);
            }
            //--- ## v 0.9.9 (2008-03-30) --------------------------------------
            
            //--- (3) Set the attributes sent by the client-side ( xml attribute --> bean attributes )
            if (attributes != null)
            {
                for ( int i = 0 ; i < attributes.getLength() ; i++ )
                {
                    //--- Nom de l'attribut
                    String sAttrName = attributes.getLocalName(i);
                    //--- Valeur de l'attribut
                    String sAttrValue = attributes.getValue(i);
                    //--- Trace
                    trace("  Attribut " + i + " : " + sAttrName + " = \"" + sAttrValue + "\"");
                    
                    //--- Delegates the field setting to the wrapper
                    try
                    {
                        wrapper.setXmlFieldValue(bean, sAttrName, sAttrValue);
                    } catch (Throwable t) // We are not sure of the wrapper quality ( external class ) 
                    {
                        throw new TelosysRuntimeException(wrapper.getClass()+" cannot set field value (tag:"+_sLastTagOpened+
                                " field:"+sAttrName+" val:"+sAttrValue+")" , t);
                    }
                }
            }
        }
        return bean ;
    }
    
}