package org.objectweb.telosys.rpl.xml.output;

import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.LinkedList;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.JavaObjectWriter;
import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.data.DataSet;
import org.objectweb.telosys.common.data.DataSetWriter;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.common.vo.GenericVOListWriter;
import org.objectweb.telosys.rpl.GenericResponse;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.uil.renderer.Renderers;
import org.objectweb.telosys.util.XmlUtil;

/**
 * Set of generic methods to generate the standard XML reponse fragments 
 * 
 * @author Laurent GUERIN
 * @sinc 1.0.5
 *
 */
public class GenericXmlOutput 
{
    //private static final TelosysClassLogger log = new TelosysClassLogger(GenericXmlOutput.class);
    
	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the opening "return" tag
	 * @param resp
	 * @param out
	 */
	public static void generateReturnOpeningTag ( GenericResponse resp, PrintWriter out)
	{
		out.println("<return code=\"" + resp.getReturnCode() 
		        + "\" message=\"" + XmlUtil.xmlString( resp.getReturnMessage() ) 
		        + "\" >" );
	}
	/**
	 * Generates the closing "return" tag
	 * @param out
	 */
	public static void generateReturnClosingTag ( PrintWriter out)
	{
		out.println("</return>" );
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the "exceptions" tag
	 * @param resp
	 * @param out
	 */
	public static void generateExceptions ( GenericResponse resp, PrintWriter out)
	{
		int iCount = resp.getNumberOfExceptions() ;
		if ( iCount > 0 )
		{
		    Throwable ex = null ;		     
			out.println("<exceptions>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        ex = resp.getException(i);
		        if ( ex != null )
		        {
					out.println(" <exception num=\"" + (i+1) 
					        + "\" class=\"" + ex.getClass().getName() 
					        + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) 
					        + "\" />" );		            
		        }
		    }
			out.println("</exceptions>" );
		}
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the "errors" tag
	 * @param resp
	 * @param out
	 */
	public static void generateErrors ( GenericResponse resp, PrintWriter out)
	{
	    int iCount = resp.getNumberOfErrors() ;
		if ( iCount > 0 )
		{
		    ScreenError error = null ;
			out.println("<errors>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        error = resp.getError(i);
		        if ( error != null )
		        {
					out.println(" <error num=\"" + (i+1) 
					        + "\" code=\"" + error.getCode() 
					        + "\" text=\"" + XmlUtil.xmlString( error.getText() ) 
					        + "\" />" );		            
		        }
		    }
			out.println("</errors>" );
		}
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the "messages" tag
	 * @param resp
	 * @param out
	 */
	public static void generateMessages ( GenericResponse resp, PrintWriter out)
	{
	    int iCount = resp.getNumberOfMessages() ;
		if ( iCount > 0 )
		{
		    ScreenMessage message = null ;
			out.println("<messages>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        message = resp.getMessage(i);
		        if ( message != null )
		        {
					out.println(" <message num=\"" + (i+1) 
					        + "\" type=\"" + message.getType()
					        + "\" code=\"" + message.getCode()
					        + "\" text=\"" + XmlUtil.xmlString( message.getText() ) 
					        + "\" />" );		            
		        }
		    }
			out.println("</messages>" );
		}
	}
	
	//--------------------------------------------------------------------------------------------------
	// <values> ... </values>
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the "values" tag
	 * @param resp
	 * @param out
	 */
	public static void generateValues ( GenericResponse resp, PrintWriter out)
	{
		if ( resp.getNumberOfValues() > 0 )
		{
		    String sValueName = null ;
			Enumeration eNames = resp.getValueNames();
			if ( eNames != null )
			{
				out.println("<values>" );
			    while ( eNames.hasMoreElements() )
			    {
			        sValueName = (String) eNames.nextElement() ;
					out.println("<value " + sValueName + "=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
			    }
				out.println("</values>" );			    
			}
		}
	}
	
    //------------------------------------------------------------------------------------
    /**
     * Generates a response view content using the given "renderer"
     * @param out
     * @param sViewName the view tag name 
     * @param sRenderer 
     * @param httpRequest
     * @param httpResponse
     * @return a ViewError object if an exception occurrs, null if there's no error
     */
    public final static ViewOutputError generateViewContent ( PrintWriter out, 
            String sViewName, String sRenderer,
            HttpServletRequest httpRequest, HttpServletResponse httpResponse )
    {
        ViewOutputError viewError = null ;
        
        //--- VIEW opening tag
		out.println("<"+ sViewName + " renderer=\"" + sRenderer + "\" >");    		
        //--- CDATA opening tag
		out.println("<![CDATA[");    		
        
        try
        {
            Renderers.generate( sRenderer, httpRequest, httpResponse);
        } catch (Throwable e)
        {
            viewError = new ViewOutputError(sViewName, sRenderer, e);
            out.println("<br><b>ERROR</b> in renderer <b>'" + sRenderer + "'</b><br>");
            printViewError(out, e);
        }
        finally {
            //--- CDATA closing tag
    		out.println("]]>");    		
            //--- VIEW closing tag
    		out.println("</"+ sViewName + ">");    		
        }
        return viewError ;
    }
	
	//--------------------------------------------------------------------------------------------------
	private static void printErrorCauses ( PrintWriter out, Throwable ex)
	{
	    if ( ex != null )
	    {
	        Throwable cause = getCause ( ex ) ;
	        while ( cause != null )
	        {
	            //--- Generate the cause tag
		        out.println("<b>Cause</b> : " + cause.getClass().getName() + " : " + cause.getMessage() + "<br>");
	            //--- Get the "cause of the cause"
	            cause = getCause ( cause ) ;
	        }
	    }
	}
	//--------------------------------------------------------------------------------------------------
	private static void printViewError ( PrintWriter out, Throwable ex)
	{
	    if ( ex != null ) 
	    {	        
	        out.println("<b>Exception</b> : " + ex.getClass().getName() + " : " + ex.getMessage() + "<br>");
	        printErrorCauses(out, ex);
	    }
	}

	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the "output-errors" if necessary from the given DataOutputError and ViewOutputError
	 * @param dataError
	 * @param viewError
	 * @param out
	 */
	public static void generateOutputErrors ( DataOutputError dataError, ViewOutputError viewError, PrintWriter out)
	{
	    if ( ( null == dataError ) && ( null == viewError ) ) return ;
		out.println("<output-errors>" );
		try {
		    //--- Generate 0..1 <data-error> tags
		    if ( dataError != null )
		    {
		        generateDataErrorTag ( dataError, out);		        
			}
		    
		    //--- Generate 0..1 <view-error> tags
		    if ( viewError != null )
		    {
		        generateViewErrorTag ( viewError, out);		        
			}
		}
		finally {
			out.println("</output-errors>" );
		}
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the "output-errors" if necessary from the given collections of DataError and ViewError
	 * @param dataErrors 
	 * @param viewErrors
	 * @param out
	 */
	public static void generateOutputErrors ( LinkedList dataErrors, LinkedList viewErrors, PrintWriter out)
	{
	    int n1 = dataErrors != null ? dataErrors.size() : 0 ;
	    int n2 = viewErrors != null ? viewErrors.size() : 0 ;
	    if ( n1 + n2 == 0 ) return ; // No error 
	    
		out.println("<output-errors>" );
		try {
		    //--- Generate 0..N <data-error> tags
		    generateOutputErrorTags ( dataErrors, out);
		    
		    //--- Generate 0..N <view-error> tags
		    generateOutputErrorTags ( viewErrors, out);
		}
		finally {
			out.println("</output-errors>" );
		}
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates the <data-error> tags or <view-error> tags for a collection of OutputError objects
	 * @param errors
	 * @param out
	 */
	private static void generateOutputErrorTags ( LinkedList errors, PrintWriter out)
	{
	    if ( null == errors ) return ; 
		int iCount = errors.size() ;
		if ( iCount > 0 )
		{
		    OutputError err = null ;
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        err = (OutputError) errors.get(i);
		        generateOutputErrorTag ( err, out);		        
		    }
		}
	}
	//--------------------------------------------------------------------------------------------------
//	/**
//	 * Generates the <view-errors> if necessary for a single ViewError
//	 * @param viewError
//	 * @param out
//	 */
//	private static void generateViewErrors ( ViewOutputError viewError, PrintWriter out)
//	{
//	    if ( viewError != null )
//	    {
//			out.println("<view-errors>" );
//		    generateViewErrorTag ( viewError, out);		        
//			out.println("</view-errors>" );
//		}
//	}
	//--------------------------------------------------------------------------------------------------
	private static void generateViewErrorTag ( ViewOutputError viewError, PrintWriter out)
	{
	    generateOutputErrorTag ( viewError, out);
//        if ( viewError != null )
//        {
//			out.print(" <view-error "
//			        + " view=\"" + viewError.getViewName() + "\""  
//			        + " renderer=\"" + viewError.getRenderer() + "\"" 
//			        );
//			
//            Throwable ex = viewError.getException();
//			if ( ex != null )
//			{
//				out.print(" exception=\"" + ex.getClass().getName() + "\"" );
//			}
//			out.print(" message=\"" + XmlUtil.xmlString( viewError.getMessage() ) + "\"" );
//			
//			if ( ex != null )
//			{
//				out.println(" >" );
//				generateExceptionCauseTag ( ex, out);
//				out.println("</view-error>");
//			}
//			else
//			{
//				out.println(" />" );
//			}
//        }
	}
	//--------------------------------------------------------------------------------------------------
	private static void generateDataErrorTag ( DataOutputError dataError, PrintWriter out)
	{
	    generateOutputErrorTag ( dataError, out);
	}
	//--------------------------------------------------------------------------------------------------
	private static void generateOutputErrorTag ( OutputError outputError, PrintWriter out)
	{
        if ( outputError != null )
        {
            //--- Open the tag
            if ( outputError instanceof ViewOutputError )
            {
                ViewOutputError viewError = (ViewOutputError) outputError ;
    			out.print(" <view-error "
    			        + " view=\"" + viewError.getViewName() + "\""  
    			        + " renderer=\"" + viewError.getRenderer() + "\"" 
    			        );
            }
            else if ( outputError instanceof DataOutputError )
            {
                DataOutputError dataError = (DataOutputError) outputError ;
				out.print(" <data-error "
				        + " name=\"" + dataError.getElementName() + "\""  
				        + " class=\"" + dataError.getDataClassName() + "\"" 
				        );
            }
            else
            {
                return ;
            }
            
            Throwable ex = outputError.getException();
    		if ( ex != null )
    		{
    			out.print(" exception=\"" + ex.getClass().getName() + "\"" );
    		}
    		out.print(" message=\"" + XmlUtil.xmlString( outputError.getMessage() ) + "\"" );
			out.println(" >" );
    		
            //--- Generate tag content if any ( exception causes )
			if ( ex != null )
			{
				generateExceptionCauseTag ( ex, out);
			}

            //--- Close the tag
			if ( outputError instanceof ViewOutputError )
            {
				out.println("</view-error>");
            }
            else if ( outputError instanceof DataOutputError )
            {
    			out.println("</data-error>");
            }
        }
	}
	//--------------------------------------------------------------------------------------------------
	/**
	 * Returns the cause of the given exception, managing the special case of "ServletException"
	 * @param ex
	 * @return
	 */
	private static Throwable getCause ( Throwable ex )
	{
	    if ( ex != null )
	    {	        
            if ( ex instanceof ServletException )
            {
                //--- ServletException : get "Root Cause"
                ServletException ex2 = (ServletException) ex ;
                return ex2.getRootCause();
            }
            else
            {
                //--- Other Exception : get "Standard Cause"
                return ex.getCause();
            }
	    }
	    return null ;
	}
	//--------------------------------------------------------------------------------------------------
	private static void generateExceptionCauseTag ( Throwable ex, PrintWriter out)
	{
	    if ( ex != null )
	    {
	        Throwable cause = getCause ( ex ) ;
	        while ( cause != null )
	        {
	            String sMessage = cause.getMessage() ;
	            if ( null == sMessage ) sMessage = "" ;
	            //--- Generate the cause tag
				out.println(" <cause "
				        + " exception=\"" + cause.getClass().getName() + "\""
				        + " message=\"" + sMessage + "\""
				        + " />" );		            
	            //--- Get the "cause of the cause"
	            cause = getCause ( cause ) ;
	        }
	    }
	}
	
	//--------------------------------------------------------------------------------------------------
    /**
     * Generates an element data tag for the given object instance <br>
     * The element can be : <br>
     * a Telosys collection ( VOList, DataSet ), <br>
     * a basic Java object ( String, Float, Boolean, Date, .. ), <br>
     * a specific application Value Object with a specific XML mapper
     * 
     * @param oDataElement
     * @param sElementName
     * @param out
     */
    public final static DataOutputError generateDataElement ( Object oDataElement, String sElementName, PrintWriter out )
    {
        DataOutputError error = null ;
        
	    if ( oDataElement != null )
	    {
	        boolean bGenerated = false ;
	        
	        try {
		        if ( ! bGenerated ) {
		        	//---  Try to generate a generic object ( VOList, DataSet )
		        	bGenerated = generateDataTelosysCollection ( oDataElement, sElementName, out) ;
		        }
		        
		        if ( ! bGenerated ) {
			        //--- Try to generate as a standard Java object 
			        bGenerated = generateDataJavaObject ( oDataElement, sElementName, out) ;
		        }
		        
		        if ( ! bGenerated ) {
			        //--- Try to generate as a specific object ( with specific mapper )
			        bGenerated = generateDataSpecificObject ( oDataElement, sElementName, out) ;
		        }
		        
		        if ( ! bGenerated ) {
			        String s = "<!-- Element " + sElementName + " : cannot generate this type of object (" 
	            				+ oDataElement.getClass().getName() +") -->" ;
			        out.println(s);
		        }
	
	        } catch (Throwable e)
	        {
	            error = new DataOutputError(oDataElement, sElementName, e);
	        }
//	        finally {
//	        }
	    }
	    else
	    {
    		out.println("<!-- Element " + sElementName + " is null -->" );
	    }
	    return error ;
    }
    
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates a tag for a Telosys collection : VOList or DataSet
	 * @param oDataElement
	 * @param sElementName
	 * @param out
	 * @return
	 */
	private static boolean generateDataTelosysCollection ( Object oDataElement, String sElementName, PrintWriter out) 
	{
	    if ( oDataElement instanceof GenericVOList )
	    {
	        GenericVOListWriter.toXml(out, (GenericVOList) oDataElement, sElementName );
	        return true ;
	    }
	    else if ( oDataElement instanceof DataSet )
	    {
	        DataSetWriter.toXml(out, (DataSet) oDataElement, sElementName );
	        return true ;
	    }
	    return false ;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates a tag for an application specific object ( VO ) that requires a specific XML mapper
	 * @param oDataElement
	 * @param sElementName
	 * @param out
	 * @return
	 */
	private static boolean generateDataSpecificObject ( Object oDataElement, String sElementName, PrintWriter out)
	{
	    boolean bRet = false ;
	    IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider() ;
        if (wrapperProvider == null)
		{
    		out.println("<!-- ERROR : cannot get XmlWrapperProvider -->" );
            //throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
		}
        else
        {
	        //--- Get the XML Wrapper for the given object
            IXmlWrapper wrapper = null;
            try
            {
                wrapper = wrapperProvider.getWrapper(oDataElement);
            } catch (TelosysRuntimeException e)
            {
                wrapper = null;
            }
	        if (wrapper != null)
	        {
	            //--- Print the XML tag
	            wrapper.toXml(out, oDataElement, sElementName);
	            bRet = true ;
	        }
	        else
	        {
	    		out.println("<!-- Cannot get mapper for class " + oDataElement.getClass().getName() + " -->" );
	        }
        }
        return bRet ;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates a tag for a basic Java object : String, Interger, Long, Double, Float, BigDecimal, Boolean, Date, ...
	 * @param oDataElement
	 * @param sElementName
	 * @param out
	 * @return
	 */
	private static boolean generateDataJavaObject ( Object oDataElement, String sElementName, PrintWriter out ) 
	{
	    String sClassName = oDataElement.getClass().getName() ;
		if ( sClassName.startsWith("java.") ) // java.lang, java.math, java.util 
	    {
		    //--- Try to generate this object ...
		    return JavaObjectWriter.toXml(out, oDataElement, sElementName);
	    }
	    return false ;
	}
	
    
}
