package org.objectweb.telosys.screen.core;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenRequestParameters;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * 
 * The ScreenActions interface <br>
 * Defines all the methods that must be implemented by an ACTION <br>
 * 
 * Implemented by "StandardScreenActions" (the default implementation provided by the framework) <br>
 * 
 * @author Laurent GUERIN
 *
 */
public interface ScreenActions
{
    //------------------------------------------------------------------------------------
    //--- Events 
    //------------------------------------------------------------------------------------

    /**
     * Event fired when the screen context has just been created by <br>
     * - a screenmap request with "open" action 
     * - a screenmap request with "use" action on a non existent screen context <br>
     * - an "open" screen action sent by the client-side ( in this case no parameters )
     *  
     * @param screenContext the ScreenContext instance created
     * @param params 
     * @param screenSession
     * @throws TelosysException
     */
    public void eventCreate(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession) throws TelosysException ;
    
    /**
     * Event fired when the screen context is reused  <br>
     * 
     * @param screenContext
     * @param params
     * @param screenSession
     * @throws TelosysException
     */
    public void eventReuse(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession) throws TelosysException ;
    
    
    //------------------------------------------------------------------------------------
    //--- Basic actions 
    //------------------------------------------------------------------------------------
    /**
     * "SET" action ( to set informations in the ScreenContext )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionSet(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    /**
     * "GET" action ( to get informations from the ScreenContext )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionGet(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;
    
    /**
     * "CLEAR" action ( to clear the ScreenContext )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionClear(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    //------------------------------------------------------------------------------------
    //--- Persistence actions 
    //------------------------------------------------------------------------------------
    /**
     * "LOAD" action ( to load informations from the database )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionLoad(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    /**
     * "SAVE" action ( to save informations in the database )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionSave(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    /**
     * "SAVE_CURRENT" action ( to save current informations in the database )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionSaveCurrent(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;
    
    /**
     * "INSERT" action ( to insert informations in the database )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionInsert(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    /**
     * "UPDATE" action ( to update informations in the database )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionUpdate(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    /**
     * "DELETE" action ( to delete informations in the database )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionDelete(ScreenContext screen, ScreenRequest request, ScreenResponse response ) throws TelosysException ;

    /**
     * "DELETE_CURRENT" action ( to delete current informations in the database )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionDeleteCurrent(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    //------------------------------------------------------------------------------------
    //--- List management 
    //------------------------------------------------------------------------------------
    /**
     * "LIST_SELECT" action ( to select an item in a list )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionListSelect(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;

    /**
     * "LIST_INSERT" action ( to insert an item in a list )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionListInsert(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;
    
    /**
     * "LIST_ADD" action ( to add an item at the end of a list )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionListAdd(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;
    
    /**
     * "LIST_REMOVE" action ( to remove an item from a list )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionListRemove(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;
    
    /**
     * "LIST_CLEAR" action ( to clear a list )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionListClear(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;
    
    /**
     * "LIST_SORT" action ( to sort a list )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionListSort(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException ;
    
    //------------------------------------------------------------------------------------
    //--- Specific 'procedure call'
    //------------------------------------------------------------------------------------
    /**
     * "EXEC" action ( to execute a specific "procedure" )
     * @param screen
     * @param request
     * @param response
     * @throws TelosysException
     */
    public void actionExec(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException;


}
