package org.objectweb.telosys.screen.core;

import org.objectweb.telosys.common.SymbolicConfigVar;
import org.objectweb.telosys.common.TelosysException;

/**
 * Each instance of this class defines a convention to determine the full name of the ScreenContext classes 
 * 
 * @author Laurent Guerin
 * @since 1.0.5
 */
/* package */ class ScreenClassesConvention
{
    
    //--- Default class names suffixes
    private String _sScreenDataExtension      = "Data" ;
    
    private String _sScreenManagerExtension   = "Mgr" ;
    
    private String _sScreenTriggerExtension   = "Trg" ;
    
    private String _sScreenProcedureExtension = "Proc" ;
    
    //--- Class names
    private String _sScreenDataClassName      = null ;
    
    private String _sScreenManagerClassName   = null ;
    
    private String _sScreenTriggerClassName   = null ;
    
    private String _sScreenProcedureClassName = null ;
    
    //----------------------------------------------------------------------------------------------------
    private String checkGenericClassName(String s) throws TelosysException
    {
        if ( null == s )
        {
            throw new TelosysException("constructor error : the genericClassName parameter is null !");
        }
        return s.trim();
    }
    
    //----------------------------------------------------------------------------------------------------
    private String blankIfNull (String s)
    {
        return ( s != null ? s : "" ) ;
    }
    
    //----------------------------------------------------------------------------------------------------
    /**
     * Build a full class name by concatenation ( package + "." + class )
     * @param sPackageName
     * @param sClassName
     * @return
     * @throws TelosysException
     */
    private String concat(String sPackageName, String sClassName ) throws TelosysException
    {
        if ( null == sPackageName )
        {
            throw new TelosysException("constructor error : the package name is null !");
        }
        if ( null == sClassName )
        {
            throw new TelosysException("constructor error : the class name is null !");
        }
        return sPackageName.trim() + "." + sClassName.trim() ;
    }
    
    //----------------------------------------------------------------------------------------------------
    /**
     * Constructor based on generic class name with default extensions
     * @param genericClassName generic class name pattern ( ie "demo.screen.${SCREENNAME_LC}.${SCREENNAME}" )
     * @throws TelosysException
     */
    public ScreenClassesConvention( String genericClassName ) throws TelosysException
    {
        super();
        String s = checkGenericClassName( genericClassName );
        _sScreenDataClassName      = s + _sScreenDataExtension ;
        _sScreenManagerClassName   = s + _sScreenManagerExtension ; 
        _sScreenTriggerClassName   = s + _sScreenTriggerExtension ;
        _sScreenProcedureClassName = s + _sScreenProcedureExtension ;
    }
    
    //----------------------------------------------------------------------------------------------------
    /**
     * Constructor with specific class names extensions
     * @param genericClassName generic class name pattern ( ie "demo.screen.${SCREENNAME_LC}.${SCREENNAME}" )
     * @param screenDataExtension specific extension for ScreenData 
     * @param screenManagerExtension specific extension for ScreenManager
     * @param screenTriggerExtension specific extension for ScreenTrigger
     * @param screenProcedureExtension specific extension for ScreenProcedure
     * @throws TelosysException
     */
    public ScreenClassesConvention( String genericClassName, 
            String screenDataExtension, String screenManagerExtension, 
            String screenTriggerExtension, String screenProcedureExtension ) throws TelosysException
    {
        super();
        String s = checkGenericClassName( genericClassName );
        _sScreenDataClassName      = s + blankIfNull(screenDataExtension);
        _sScreenManagerClassName   = s + blankIfNull(screenManagerExtension);
        _sScreenTriggerClassName   = s + blankIfNull(screenTriggerExtension);
        _sScreenProcedureClassName = s + blankIfNull(screenProcedureExtension);
    }

    //----------------------------------------------------------------------------------------------------
    /**
     * Constructor for a convention based on 4 specific class name patterns
     * 
     * @param screenDataPattern
     * @param screenManagerPattern
     * @param screenTriggerPattern
     * @param screenProcedurePattern
     */
    public ScreenClassesConvention( 
            String screenDataPattern, String screenManagerPattern, 
            String screenTriggerPattern, String screenProcedurePattern ) // throws TelosysException
    {
        super();
        _sScreenDataClassName      = screenDataPattern ;
        _sScreenManagerClassName   = screenManagerPattern ;
        _sScreenTriggerClassName   = screenTriggerPattern ;
        _sScreenProcedureClassName = screenProcedurePattern ;
    }
    
    //----------------------------------------------------------------------------------------------------
    /**
     * Constructor for a convention based on 4 specific package name patterns and 4 specific class name patterns
     * 
     * @param screenDataPackageName
     * @param screenDataClassName
     * @param screenManagerPackageName
     * @param screenManagerClassName
     * @param screenTriggerPackageName
     * @param screenTriggerClassName
     * @param screenProcedurePackageName
     * @param screenProcedureClassName
     * @throws TelosysException
     */
    public ScreenClassesConvention( 
            String screenDataPackageName, String screenDataClassName,
            String screenManagerPackageName, String screenManagerClassName,
            String screenTriggerPackageName, String screenTriggerClassName,
            String screenProcedurePackageName, String screenProcedureClassName ) throws TelosysException
    {
        super();
        _sScreenDataClassName      = concat ( screenDataPackageName, screenDataClassName ) ;
        _sScreenManagerClassName   = concat ( screenManagerPackageName, screenManagerClassName ) ;
        _sScreenTriggerClassName   = concat ( screenTriggerPackageName, screenTriggerClassName ) ;
        _sScreenProcedureClassName = concat ( screenProcedurePackageName, screenProcedureClassName ) ;
    }
    
//    //----------------------------------------------------------------------------------------------------    
//    /**
//     * Replaces the first occurrence of ${SCREENNAME}, ${SCREENNAME_LC} or ${SCREENNAME_UC}
//     * @param sOriginal
//     * @param sValue
//     * @return
//     */
//    private static String replace(String sOriginal, String sValue) 
//    {
//        if ( sOriginal.indexOf( TelosysConfigVar.SCREENNAME ) >= 0 )
//        {
//            return StrUtil.replaceVar(sOriginal, TelosysConfigVar.SCREENNAME, sValue );
//        }
//        else if ( sOriginal.indexOf(TelosysConfigVar.SCREENNAME_UC) >= 0 )
//        {
//            return StrUtil.replaceVar(sOriginal, TelosysConfigVar.SCREENNAME_UC, sValue.toUpperCase() );
//        }
//        else if ( sOriginal.indexOf(TelosysConfigVar.SCREENNAME_LC) >= 0 )
//        {
//            return StrUtil.replaceVar(sOriginal, TelosysConfigVar.SCREENNAME_LC, sValue.toLowerCase() );
//        }
//        return sOriginal ;
//    }
    
//    //----------------------------------------------------------------------------------------------------
//    private static boolean containsVarToReplace(String s) 
//    {
//        if ( s.indexOf( TelosysConfigVar.SCREENNAME   ) >= 0 ) return true ;
//        if ( s.indexOf( TelosysConfigVar.SCREENNAME_UC) >= 0 ) return true ;
//        if ( s.indexOf( TelosysConfigVar.SCREENNAME_LC) >= 0 ) return true ;
//        return false ;
//    }
    
    //----------------------------------------------------------------------------------------------------
    /**
     * Replaces all the occurrences of the symbolic variables 
     * "${SCREENNAME}", "${SCREENNAME_UC}" and "${SCREENNAME_LC}" in the given pattern
     * 
     * @param pattern
     * @param screenName
     * @return
     */
    private String getClassName( String pattern, String screenName )
    {
//        if ( pattern != null && screenName != null )
//        {
//            //--- Replace all the symbolic variables ${xxx} in the pattern
//            String s = pattern ;
//            while ( containsVarToReplace(s) )
//            {
//                s = replace( s, screenName );
//            }
//            return s ;
//        }
//        return null ;
        
        return SymbolicConfigVar.applyScreenName( pattern, screenName );

    }
    
    //----------------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenData class name for the given screen name
     * @param screenName
     * @return
     */
    public String getScreenDataClassName( String screenName )
    {
        return getClassName(_sScreenDataClassName, screenName);
    }

    //----------------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenManager class name for the given screen name
     * @param screenName
     * @return
     */
    public String getScreenManagerClassName( String screenName )
    {
        return getClassName(_sScreenManagerClassName, screenName);
    }
    
    //----------------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenTrigger class name for the given screen name
     * @param screenName
     * @return
     */
    public String getScreenTriggerClassName( String screenName )
    {
        return getClassName(_sScreenTriggerClassName, screenName);
    }

    //----------------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenProcedure class name for the given screen name
     * @param screenName
     * @return
     */
    public String getScreenProcedureClassName( String screenName )
    {
        return getClassName(_sScreenProcedureClassName, screenName);
    }
}
