package org.objectweb.telosys.screen.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * A temporary collection of ScreenDataElement instances <br>
 * <br>
 * Used to store : <br>
 * . the data elements of a ScreenRequest ( XML "data" section received from the client side ), <br>
 * . the data elements of a ScreenResponse ( XML "data" section send to the client side ) <br>
 * . a copy of the ScreenContext elements 
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenDataElements extends TelosysObject
{
    private ArrayList _elements = null;

    //-----------------------------------------------------------------------------------------------
    /**
     * Default constructor
     */
    public ScreenDataElements()
    {
        super();
        _elements = new ArrayList();
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Constructor from elements collection
     * @param elements
     */
    public ScreenDataElements(Collection elements)
    {
        super();
        _elements = new ArrayList(elements);        
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Adds an element in the elements collection
     * @param oElement
     */
    public void add(ScreenDataElement oElement)
    {
        _elements.add(oElement);
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Adds an element in the elements collection
     * @param sElementName 
     * @param oBean
     */
    public void add( String sElementName, Object oBean)
    {
        if ( sElementName == null )
        {
            throw new TelosysRuntimeException("ScreenDataElements - add() : element name is null ");
        }
        if ( oBean == null )
        {
            throw new TelosysRuntimeException("ScreenDataElements - add() : bean object is null ");
        }
        //--- Element creation
        ScreenDataElement element = new ScreenDataElement( sElementName, oBean.getClass() ) ;
        element.setInstance(oBean);            

        //--- Add the element ...
        _elements.add(element);
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the size of the elements collection
     * @return
     */
    public int size()
    {
        return _elements.size();
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the element located at the given index
     * @param index
     * @return
     */
    public ScreenDataElement get(int index)
    {
        return (ScreenDataElement) _elements.get(index);
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the element having the given name (or null if not found) <br>
     * If there's multiple elements with the same name, the first element found is returned.
     * 
     * @param sElementName
     * @return
     */
    public ScreenDataElement get(String sElementName)
    {
        if ( null == sElementName ) return null ;
        
        Iterator iter = _elements.iterator();
        if ( iter != null )
        {
            while ( iter.hasNext() )
            {
                ScreenDataElement elt = (ScreenDataElement) iter.next();
                if ( elt != null )
                {
                    if ( sElementName.equals( elt.getName() ) )
                    {
                        return elt ;
                    }
                }
            }            
        }
        return null ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the element object having the given name and instance of the given class (or null if not found) <br>
     * If there's multiple elements with the same name, the first element found is returned.
     * 
     * @param sElementName
     * @param clazz if null the type of the object is not checked
     * @return
     */
    public Object getDataElementInstance(String sElementName, Class clazz)
    {
        if ( null == sElementName ) return null ;

        Iterator iter = _elements.iterator();
        if ( iter != null )
        {
            while ( iter.hasNext() )
            {
                ScreenDataElement e = (ScreenDataElement) iter.next();
                if ( e != null )
                {
                    if ( sElementName.equals( e.getName() ) ) // Name found
                    {
            			Object o = e.getInstance();
            			if ( o != null )
            			{
	            			if ( clazz != null ) // Check "instance of"
	            			{
	            				if ( clazz.isInstance( o ) ) // Class matching OK
	            				{
	            					return o ;
	            				}
	            			}
	            			else // Do not check the class type
	            			{
	            			    return o ;
	            			}
            			}
            			else
            			{
            			    return o ; // null => considered as found
            			}
                    }
                }
            }
		}
        return null ;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns an iterator for the elements collection
     * @return
     */
    public Iterator iterator()
    {
        return _elements.iterator();
    }
    
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer(256);
        sb.append("ScreenDataElements ( " + _elements.size() + " elements ) : ");
        Iterator iter = _elements.iterator();
        if ( iter != null )
        {
            while ( iter.hasNext() )
            {
                ScreenDataElement elt = (ScreenDataElement) iter.next();
                sb.append("\n . " + elt.toString() );                
            }            
        }
        return sb.toString();
    }
}