package org.objectweb.telosys.screen.core;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * The standard ScreenData implementation.
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardScreenData extends TelosysObject implements ScreenData
{
    private final HashMap       _hmElements      = new HashMap(4); // serializable (std in Java)

    private boolean             _bFound          = false;

    //-----------------------------------------------------------------------------------------------
    /**
     * Defines a screen data element <br>
     * The element is only defined, not created.
     * 
     * @param sElementName : the logical name of the element
     * @param elementClass : the class of the element 
     * @throws TelosysException
     */
    public void defineElement(String sElementName, Class elementClass) throws TelosysException
    {
        _hmElements.put(sElementName, new ScreenDataElement(sElementName, elementClass));
    }

//    //-----------------------------------------------------------------------------------------------
//    public void defineElement(String sElementName, Class elementClass, String sRendererPath) throws TelosysException
//    {
//        _hmElements.put(sElementName, new ScreenDataElement(sElementName, elementClass, sRendererPath));
//    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Binds an "item element" to a "list element"
     * Not used (!)
     * 
     * @param sElementName
     * @param sListName
     * @throws TelosysException
     */
    public void bindElementToList(String sElementName, String sListName) throws TelosysException
    {
        trace("bindElementToList(" + sElementName + "," + sListName + ")");
        //--- Get the "ITEM" ScreeDataElement
        ScreenDataElement oElement = getScreenDataElement(sElementName);
        if (oElement == null)
        {
            throw new TelosysRuntimeException("bindElementToList() : unknown element name '" + sElementName + "'");
        }

        //--- Get the "LIST" ScreeDataElement
        ScreenDataElement oList = getScreenDataElement(sListName);
        if (oList == null)
        {
            throw new TelosysRuntimeException("bindElementToList() : unknown list name '" + sListName + "'");
        }

        //--- Bind the 2 ScreenDataElements
        oElement.bindToList(oList);
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#copyElementsTo(org.objectweb.telosys.screen.core.ScreenData)
     */
    public void copyElementsTo(ScreenData screenData) //throws TelosysException
    {
        trace("copyElementsTo(ScreenData)");
        if ( screenData == null )
        {
            return ;
        }
        
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //--- For each element defined : create a new instance
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    if (element != null)
                    {
                        //--- Set the same element instance in the ScreenData destiniation
                        screenData.setElement(element.getName(), element.getInstance());
                    }
                    else
                    {
                        throw new TelosysRuntimeException("Invalid screen data definition (element entry is null) ");
                    }
                }
            }
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#copyTo(org.objectweb.telosys.screen.core.ScreenData)
     */
    public void copyTo(ScreenData screenDataImage)
    {
        if ( screenDataImage != null )
        {
            //--- Copy the elements collection
            copyElementsTo(screenDataImage);
            //--- Copy the "Found Flag"
            if ( isFound() )
            {
                screenDataImage.setFound();
            }
            else
            {
                screenDataImage.setNotFound();
            }
        }
    }
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#setElement(java.lang.String, java.lang.Object)
     */
    public void setElement(String sElementName, Object oElementInstance)
    {
        trace("setElement('" + sElementName + "', Object )");
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("setElement : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            element.setInstance(oElementInstance);
        }
        else
        {
            throw new TelosysRuntimeException("setElement : Unknown element name '" + sElementName + "'");
        }
    }

    //-----------------------------------------------------------------------------------------------
    //    public void applyBindings()
    //    {
    //        //--- For each ScreenDataElement try to apply binding
    //        Collection coll = _hmElements.values();
    //        ScreenDataElement element = null ;
    //        Iterator iter = coll.iterator();
    //        while ( iter.hasNext() )
    //        {
    //            element = (ScreenDataElement) iter.next();
    //            if ( element != null )
    //            {
    //                //--- Apply binding if any
    //                element.applyBinding();
    //            }
    //        }
    //    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getElement(java.lang.String)
     */
    public Object getElement(String sElementName)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("getElement : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            return element.getInstance();
        }
        else
        {
            throw new TelosysRuntimeException("getElement : Unknown element '" + sElementName + "' (not defined).");
        }
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#isElementDefined(java.lang.String)
     */
    public boolean isElementDefined(String sElementName)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("isElementDefined : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        return (element != null ? true : false ) ;
    }
    
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getElementClass(java.lang.String)
     */
    public Class getElementClass(String sElementName)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("getElementClass : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            return element.getInstanceClass();
        }
        else
        {
            throw new TelosysRuntimeException("getElementClass : Unknown element name '" + sElementName + "'");
        }
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getScreenDataElement(java.lang.String)
     */
    public ScreenDataElement getScreenDataElement(String sElementName)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("getScreenDataElement : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            return element;
        }
        else
        {
            throw new TelosysRuntimeException("getElementClass : Unknown element name '" + sElementName + "'");
        }
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getElements()
     */
    public ScreenDataElements getElements()
    {
        Collection c = _hmElements.values();
        if (c != null)
        {
            return new ScreenDataElements(c);
        }
        return new ScreenDataElements();
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#clearElements()
     */
    public void clearElements()
    {
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //--- For each element defined : clea
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    if (element != null)
                    {
                        element.clearInstance();
                    }
                }
            }
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#clear()
     */
    public void clear()
    {
        clearElements();
        _bFound = false ;
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#setFound()
     */
    public void setFound()
    {
        _bFound = true;
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#setNotFound()
     */
    public void setNotFound()
    {
        _bFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#isFound()
     */
    public boolean isFound()
    {
        return _bFound;
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer(256);
        sb.append("ScreenData (" + this.getClass().getName() + ") : ");
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //--- For each element defined : create a new instance
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    sb.append("\n . " + element.toString() );
                    /***
                    Object oElementInstance = null;
                    String sElementName = null;
                    if (element != null)
                    {
                        sElementName = element.getName();
                        String sElementClass = "(null)";
                        if (element.getInstanceClass() != null)
                        {
                            sElementClass = element.getInstanceClass().getName();
                        }
                        String sElementToString = "(null)";
                        oElementInstance = element.getInstance();
                        if (oElementInstance != null)
                        {
                            sElementToString = oElementInstance.toString();
                            sElementClass = oElementInstance.getClass().getName();
                        }
                        //--- Add element informations
                        sb.append("\n . " + sElementName + "(" + sElementClass + ") : " + sElementToString);
                    }
                    ***/
                }
            }
        }
        else
        {
            sb.append("(no element)");
        }
        return sb.toString();
    }
    
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#exposeData(javax.servlet.http.HttpServletRequest)
     */
    public void exposeData( HttpServletRequest httpRequest )
    {
        if ( httpRequest == null )
        {
            throw new TelosysRuntimeException("exposeData() : HttpServletRequest parameter is null ");
        }        
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //--- For each element in the Map
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    if (element != null)
                    {
                        if (element.getInstance() != null)
                        {
                            //--- Expose the element as a request attribute 
                            httpRequest.setAttribute(element.getName(), element.getInstance());
                        }
                    }
                }
            }
        }
    }

}