package org.objectweb.telosys.service;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Properties;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysProperties;

/**
 * This is the standard ancestor class designed to build a "Service Registry" ( "Service Provider" ) <br>
 * 
 * It provides the "register" and "registerServicePackage" methods designed <br>
 * to be used in the child class.
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardServiceRegistry extends TelosysObject implements IServiceProvider
{
    //=========================================================================================
    // INDEPENDENT (static) INNER CLASS 
    //=========================================================================================
    private static class ServiceComponents
    {
        private final static String ERR      = "ServiceProvider : cannot create service ";
        
        private String              _serviceName;

        private Class               _serviceClass;

        private String              _serviceRenderer ;

        private String              _serviceSessionKey ; 

        public ServiceComponents(String serviceName, Class serviceClass, String sRenderer, String sSessionKey ) throws TelosysException
        {
            final String sErrMsg = "Register Service '" + serviceName + "' : Invalid class ";
          
            try
            {
                //--- Service Name ------------------------------------
                _serviceName = serviceName;
                if (_serviceName == null)
                {
                    throw new TelosysException("Register Service : Service name is NULL ");
                }

                //--- Service class ------------------------------------
                _serviceClass = serviceClass;
                if (_serviceClass != null)
                {
                    if ( ScreenService.class.isAssignableFrom(_serviceClass) != true)
                    {
                        throw new TelosysException(sErrMsg);
                    }
                }
                
                //--- Service renderer ------------------------------------
                _serviceRenderer = sRenderer ; // can be null

                //--- Service key to store result in session---------------
                _serviceSessionKey = sSessionKey ; // can be null
                
            } catch (Throwable e)
            {
                throw new TelosysException("Register Service '"+ _serviceName + "' : Exception", e);
            }
        }

        /**
         * Create an instance of ScreenService
         * 
         * @return
         * @throws TelosysException
         */
        private ScreenService createService() throws TelosysException
        {
            try
            {
                //--- Create the screen instance with the class
                ScreenService service = (ScreenService) _serviceClass.newInstance();
                if (service != null)
                {
                    //--- Init the service
                    service.init( _serviceName, _serviceRenderer, _serviceSessionKey );
                    return service;
                }
                else
                {
                    throw new TelosysException(ERR + _serviceName + " (class.newInstance return null)");
                }
            } catch (InstantiationException e)
            {
                throw new TelosysException(ERR + _serviceName, e);
            } catch (IllegalAccessException e)
            {
                throw new TelosysException(ERR + _serviceName, e);
            } catch (Throwable e)
            {
                throw new TelosysException(ERR + _serviceName, e);
            }
        }

        private String getSessionKey() 
        {
            return _serviceSessionKey ;
        }
    }

    //=========================================================================================

    private final HashMap hmRegistry = new HashMap(40); // the REGISTRY (contains the services definitions)

    
    //------------------------------------------------------------------------------------
    /**
     * Constructor 
     * 
     * @since 1.0.5
     */
    public StandardServiceRegistry()
    {
        super();
        info("StandardServiceRegistry created.");
    }
    
    //------------------------------------------------------------------------------------
    // Interface implementation 
    //------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.service.IServiceProvider#init()
     */
    public void init() // v 1.0.5
    {
        info("StandardServiceRegistry initialization...");
        //--- Init the service packages conventions if defined in the "telosys.properties" file
        Properties prop = Telosys.getProperties();
        if ( prop != null )
        {
            String s = prop.getProperty( TelosysProperties.SERVICE_PACKAGE_CONVENTION ); // v 1.0.5
            if ( s != null )
            {
                registerServicePackage( s.trim() );
            }
            
            for ( int i = 0 ; i < 100 ; i++ )
            {
                s = prop.getProperty( TelosysProperties.SERVICE_PACKAGE_CONVENTION + i ); // v 1.0.5
                if ( s != null )
                {
                    registerServicePackage( s.trim() );
                }
            }
        }
        else
        {
            error("Cannot get Telosys properties !");
        }
    }

    //------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.service.IServiceProvider#getService(java.lang.String)
     */
    public ScreenService getService(String sServiceName) throws TelosysException
    {
        trace(" getService('" + sServiceName + "')");

        ServiceComponents def = lookup(sServiceName);
        if (def != null)
        {
            return def.createService();
        }
        return null;
    }
    
    //------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.service.IServiceProvider#getSessionKey(java.lang.String)
     */
    public String getSessionKey(String sServiceName) throws TelosysException
    {
        trace(" getSessionKey(" + sServiceName + ")");

        ServiceComponents def = lookup(sServiceName);
        if (def != null)
        {
            return def.getSessionKey();
        }
        return null;
    }

    
    //------------------------------------------------------------------------------------
    /**
     * Register the service for future use.
     * @param serviceName the logical name of the service 
     * @param serviceClass the class where the service is implemented
     * @param sRenderer the renderer used to build the "result view" (no view if null) 
     * @param sSessionKey the key used to store the result in session (not stored if null)
     */
    protected final void register(String serviceName, Class serviceClass, String sRenderer, String sSessionKey  )
    {
        trace("register('" + serviceName + "', " + serviceClass + ", '" + sRenderer + "', '" + sSessionKey + "')");
        privateRegister(serviceName, serviceClass, sRenderer, sSessionKey );
    }
    
    //------------------------------------------------------------------------------------
    private final ServiceComponents privateRegister(String serviceName, Class serviceClass, String sRenderer, String sSessionKey  )
    {
        trace("privateRegister('" + serviceName + "', " + serviceClass + ", '" + sRenderer + "', '" + sSessionKey + "')");
        ServiceComponents sc = null ;
        try
        {
            sc = new ServiceComponents(serviceName, serviceClass, sRenderer, sSessionKey );
            hmRegistry.put(serviceName, sc);
        } catch (TelosysException e)
        {
            sc = null ;
            error("Cannot register service '" + serviceName + "'");
            error(e.getMessage());
        }
        return sc ;
    }
    
    //------------------------------------------------------------------------------------
    private final ServiceComponents lookup(String sServiceName) throws TelosysException
    {
        trace("lookup('" + sServiceName + "')");

        //--- 1) Search in the services registry 
        ServiceComponents servComp = (ServiceComponents) hmRegistry.get(sServiceName);
        if (servComp != null)
        {
            return servComp;
        }
        else
        {
            //--- 2) Search in the services packages 
            servComp = lookupInPackages(sServiceName);
            if (servComp != null)
            {
                return servComp;
            }
            else
            {
                throw new TelosysException("Service '" + sServiceName + "' not found");
            }
        }
    }


    //------------------------------------------------------------------------------------
    // Unregistered services loading ( packages management ) 
    //------------------------------------------------------------------------------------
    
    private final LinkedList _servicePackages = new LinkedList();

    /**
     * Register the given package as a "service package"<br>
     * ( a package that can contain Telosys Service Classes )
     * @param sPackage
     */
    protected final void registerServicePackage( String sPackage )
    {
        trace("registerServicePackage('" + sPackage + "')");
        _servicePackages.add(sPackage);
    }
    
    //------------------------------------------------------------------------------------
    private final ServiceComponents lookupInPackages(String sServiceName)
    {
        trace("lookupInPackages('" + sServiceName + "')");
        
        Class serviceClass = loadServiceClass (sServiceName);
        if (serviceClass != null)
        {
            //--- Register the service for future re-use
            ServiceComponents sc = privateRegister(sServiceName, serviceClass, null, null );
            return sc ;
        }
        return null ;
    }
    
    //------------------------------------------------------------------------------------
    private Class loadServiceClass(String sServiceName) //throws TelosysException
    {
        trace("loadServiceClass('" + sServiceName + "')");
        Class serviceClass = null ;
        String[] sServiceClassNames = getServiceClassNames(sServiceName);
        if ( sServiceClassNames != null )
        {
            trace("loadServiceClass('" + sServiceName + "') : " + sServiceClassNames.length + " class name(s).");
            for ( int i = 0 ; i < sServiceClassNames.length ; i++ )
            {
                serviceClass = tryToLoadClass( sServiceClassNames[i] );
                if ( serviceClass != null )
                {
                    // OK : Class found
                    return serviceClass ; 
                }
            }
        }
        else
        {
            trace("loadServiceClass('" + sServiceName + "') : no class names.");
        }
	    return null ; // Class not found !
    }
    
    //------------------------------------------------------------------------------------
    /**
     * Returns all the possible full class names for the given service name 
     * @param sServiceName ie 'MyService'
     * @return ie [ org.package1.MyService, org.package2.MyService, net.package3.MyService ]
     */
    private String[] getServiceClassNames(String sServiceName)
    {
        trace("getServiceClassNames('" + sServiceName + "')");
        if ( _servicePackages == null ) return null ;
        if ( _servicePackages.isEmpty() ) return null ;
        
        int n = _servicePackages.size();
        String[] names = new String[n];
        for ( int i = 0 ; i < n ; i++ )
        {
            names[i] = ((String) _servicePackages.get(i)) + "." + sServiceName ;
        }
        return names ;
    }
    
    //------------------------------------------------------------------------------------
    /**
     * Try to load the service Java class using its class name
     * @param sServiceClassName
     * @return the class or null the class cannot be loaded
     */
    private Class tryToLoadClass(String sServiceClassName)
    {
        trace("tryToLoadClass('" + sServiceClassName + "')");
        
	    ClassLoader classLoader = this.getClass().getClassLoader() ;
        Class serviceClass = null ;
	    if ( classLoader != null )
	    {
	        try
	        {
	            serviceClass = classLoader.loadClass(sServiceClassName);
	        } catch (ClassNotFoundException e)
	        {
	            serviceClass = null ;	        
	        }
	    }
	    else
	    {
	        error("Cannot get class loader from the current class !");	        
	    }
	    return serviceClass ;
    }
    
}