package org.objectweb.telosys.service.search;

import java.util.Map;

import org.objectweb.telosys.dal.dataset.SQLDataSetDefinition;
import org.objectweb.telosys.dal.sql.Criteria;
import org.objectweb.telosys.dal.tools.Joker;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;
import org.objectweb.telosys.service.StandardScreenService;

/**
 * Generic abstract class for SQL search services <br>
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class SQLSearchAncestor extends StandardScreenService
{
    //-----------------------------------------------------------------------------------------------
    // Const
    //-----------------------------------------------------------------------------------------------

    protected final static int          MAXI_UNLIMITED      = -1;

    protected final static int          MAXI_NOT_SET        = 0;

    protected final static int          MAXI_DEFAULT        = 1000;

    private final static SearchMessages STD_MSG             = new StandardSearchMessages();

    //-----------------------------------------------------------------------------------------------
    // Attributes
    //-----------------------------------------------------------------------------------------------

    protected boolean                   _bDynamicParameters = false;

    protected Criteria                  _criteria           = null;

    private int                         _iMaxi              = MAXI_NOT_SET;                // max page size ( 0 = not set )

    protected SQLDataSetDefinition      _dsDefinition       = null;

    protected SearchMessages            _messages           = STD_MSG;

    //protected String                    _sSessionKey        = null;                        // Default : Not stored in session

    private Joker                       _joker              = null;

    //-----------------------------------------------------------------------------------------------
    /**
     * Initializes the SQL search service (no dynamic parameters)
     * 
     * @param def the SQLDataset definition to use in this service
     */
    protected void init(SQLDataSetDefinition def)
    {
        _dsDefinition = def;
        _criteria = null;
        _bDynamicParameters = false;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Initializes the SQL search service (with dynamic parameters)
     * 
     * @param def the SQLDataset definition to use in this service
     * @param criteria the SQL criteria to use 
     */
    protected void init(SQLDataSetDefinition def, Criteria criteria)
    {
        _dsDefinition = def;
        _criteria = criteria;
        _bDynamicParameters = true;
    }

    //-----------------------------------------------------------------------------------------------
    // MAX ROWS 
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns true if the maximum number of rows is set
     * @return
     */
    protected boolean maxRowsIsSet()
    {
        return (_iMaxi != MAXI_NOT_SET ? true : false);
    }

    /**
     * Returns true if the maximum number of rows is not set
     * @return
     */
    protected boolean maxRowsIsNotSet()
    {
        return (_iMaxi == MAXI_NOT_SET ? true : false);
    }

    /**
     * Set the default maximum number of rows
     * @return
     */
    protected int setDefaultMaxRows()
    {
        return _iMaxi = MAXI_DEFAULT;
    }

    /**
     * Returns the default value for the maximum number of rows
     * @return
     */
    protected int getDefaultMaxRows()
    {
        return MAXI_DEFAULT;
    }

    /**
     * Set the maximum number of rows
     * @param iMaxi
     */
    protected void setMaxRows(int iMaxi)
    {
        _iMaxi = iMaxi;
    }

    /**
     * Returns the current value for the maximum number of rows
     * @return
     */
    protected int getMaxRows()
    {
        return _iMaxi;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the messages provider to use 
     * @param messages
     */
    protected void setMessages(SearchMessages messages)
    {
        _messages = messages;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Do not use !
     * @param s
     * @deprecated
     */
    protected void setSessionKey(String s)
    {
        // _sSessionKey = s;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the specific "joker" to use <br>
     * By default there's no joker 
     * @param joker
     */
    protected void setJoker(Joker joker)
    {
        _joker = joker;
    }
    
    /**
     * Set the default "joker" ( all the "*" are replaced by "%" ) <br>
     * By default there's no joker 
     */
    protected void setJoker()
    {
        _joker = new Joker('*', '%', Joker.REPLACE_ALL);
    }
    
    /**
     * Returns the current "joker" 
     * @return
     */
    protected Joker getJoker()
    {
        return _joker ;
    }
    
    //-----------------------------------------------------------------------------------------------
    protected String getParamValue(Map requestParameters, String sParamName)
    {
        String[] val = (String[]) requestParameters.get(sParamName);
        if (val != null)
        {
            if (val.length > 0)
            {
                return val[0];
            }
        }
        return null;
    }

    protected String getParamValue(Map requestParameters, int iParamNum)
    {
        String[] val = (String[]) requestParameters.get("p" + iParamNum);
        if (val != null)
        {
            if (val.length > 0)
            {
                return val[0];
            }
        }
        return null;
    }

    protected int countParams(Map requestParameters)
    {
        int i = 1;
        while (requestParameters.get("p" + i) != null)
        {
            i++;
        }
        // "p4" == null => i == 4 ==> count == 3
        return i - 1;
    }

    /**
     * Return a String array of parameters
     * 
     * @param requestParameters
     * @return parameters array
     */
    protected String[] getStringParams(Map requestParameters)
    {
        int iSize = countParams(requestParameters);
        trace(" countParams : " + iSize);
        String strParams[] = new String[iSize];
        for ( int i = 1 ; i <= iSize ; i++ )
        {
            strParams[i - 1] = getParamValue(requestParameters, i);
        }
        return strParams;
    }

    /**
     * Set the dynamic criterias according with the input parameters
     * 
     * @param requestParameters
     * @return
     */
    protected void setCriterias(Map requestParameters)
    {
        int iSize = _criteria.getCount();
        for ( int i = 1 ; i <= iSize ; i++ )
        {
            //--- If there's a value for this parameter
            String sVal = getParamValue(requestParameters, i);
            if (sVal != null)
            {
                trace("VAL " + i + " = '" + sVal + "'");
                sVal = sVal.trim();
                if (sVal.length() > 0)
                {
//                    if (sVal.endsWith("*"))
//                    {
//                        sVal = sVal.substring(0, sVal.length() - 1) + "%";
//                    }
                    //--- If a Joker replacement is defined => use it
                    if ( _joker != null )
                    {
                        sVal = _joker.replace(sVal);
                    }
                    //--- Parameter set => use this criteria
                    _criteria.useWithValue(i, sVal);
                    trace("USE " + i + " : " + sVal);
                }
                else
                {
                    _criteria.doNotUse(i);
                    trace("DO NOT USE " + i);
                }
            }
            else
            {
                //--- No parameter => Criteria not used
                _criteria.doNotUse(i);
                trace("DO NOT USE " + i);
            }
        }
    }
    
    /**
     * Hook method called before the search operation  <br>
     * ( designed to be overridden in subclasses if necessary ) 
     * @param screenSession
     * @param serviceRequest
     * @param serviceResponse
     */
    public void before(ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse)
    {
    }
    
    /**
     * Hook method called after the search operation <br>
     * ( designed to be overridden in subclasses if necessary ) 
     * @param screenSession
     * @param serviceRequest
     * @param serviceResponse
     */
    public void after(ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse)
    {
    }
    
}