package org.objectweb.telosys.uil.screenmap;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.TelosysClassLogger;

/**
 * Utility class providing methods to parse the specific screen informations <br>
 * defined in the screen body file ( in the .jsp or .jspx file )<br>
 * 
 * The specific screen configuration can be located everywhere in the file<br>
 * It must start by a "[ScreenConfig]" line and must end by a "[/ScreenConfig]" line
 * 
 * @author Laurent Guerin
 * @since 1.0.5
 * 
 */
public class ScreenConfigParser
{
    private static final TelosysClassLogger log = new TelosysClassLogger(ScreenConfigParser.class);

    /**
     * No public constructor
     */
    private ScreenConfigParser()
    {
    }

    /**
     * Parse the ScreenBody using a BufferedReader
     * @param br
     * @param screenDefinition
     * @param sScreenType
     * @throws IOException
     */
    private static void parse( BufferedReader br, ScreenDefinition screenDefinition, String sScreenType ) throws IOException
    {
        boolean bInScreenConfig = false ;
        //HashMap hm = new HashMap();
        
        String line = null ;
        while ( ( line = br.readLine() ) != null )
        {
            String s = line.trim();
            if ( "[ScreenConfig]".equals(line) )
            {
                bInScreenConfig = true ;
            }
            else if ( "[/ScreenConfig]".equals(line) )
            {
                bInScreenConfig = false ;
                return ; // Stop as soon as possible
            }
            else
            {
                if ( bInScreenConfig )
                {
                    if ( s.length() > 0 && ( ! s.startsWith("//") ) ) // Not void and not a comment
                    {
                        String parts[] = s.split("=");
                        if ( parts.length == 2 )
                        {
                            String name  = parts[0].trim() ;
                            String value = parts[1].trim() ;
                            
                            if ( "contextname".equals(name) )
                            { 
                                screenDefinition.setScreenContextName(value);
                                log.trace("  --> contextname = '" + value + "'" );
                            }
                            else if ( "contextid".equals(name) )
                            {
                                screenDefinition.setScreenContextId(value);
                                log.trace("  --> contextid = '" + value + "'" );
                            }
                            else if ( "contextaction".equals(name) ) 
                            {
                                screenDefinition.setScreenContextAction(value); 
                                log.trace("  --> contextaction = '" + value + "'" );
                            }
                            else if ( "template".equals(name) )
                            {
                                //--- Is there a specific screen location for the current type ?
                                ScreenDefLocation screenDefLocation = screenDefinition.getScreenDefLocation(sScreenType);
                                if ( null == screenDefLocation )
                                {
                                    //--- No specific location => use the common location
                                    screenDefLocation = screenDefinition.getScreenDefLocation(); // Never null
                                }
                                if ( screenDefLocation != null )
                                {
                                    //--- Set the template file
                                    screenDefLocation.setScreenTemplate(value);
                                    log.trace("  --> template = '" + value + "'" );
                                }
                                else
                                {
                                    log.error("Cannot get ScreenDefLocation for screen '" + screenDefinition.getScreenName() + "'");
                                }
                            }
                            else if ( name.startsWith("$") )
                            {
                                //--- Template element 
                                String elementName = name.substring(1);
                                ScreenMapElement e = new ScreenMapElement(elementName, value, null );
                                screenDefinition.addElement(e);
                                log.trace("  --> '$' template element : '" + elementName + "' = '" + value + "'" );
                            }
                            else if ( name.startsWith("%") )
                            {
                                //--- Template part  
                                String elementName = name.substring(1);
                                ScreenMapElement e = new ScreenMapElement(elementName, null, value );
                                screenDefinition.addElement(e);
                                log.trace("  --> '%' template part : '" + elementName + "' = '" + value + "'" );
                            }
                            else
                            {
                                // Error 
                                log.error(" [ScreenConfig] Invalid name : '" + name + "'");
                            }
                        }
                    }
                }
            }
        }
    }
    
	/**
	 * Parse the ScreenBody resource
	 * 
	 * @param servletContext
	 * @param sResourcePath the resource to parse
	 * @param screenDefinition
	 * @param sScreenType
	 */
	private static void parse( ServletContext servletContext, String sResourcePath, ScreenDefinition screenDefinition, String sScreenType )
	{
	    log.trace("parse() : '" + sResourcePath + "'");

	    InputStream is = servletContext.getResourceAsStream(sResourcePath);
	    if ( is != null )
	    {
	        BufferedReader br = new BufferedReader( new InputStreamReader(is) );
	        
	        try
            {
                parse( br, screenDefinition, sScreenType );
            } catch (IOException e)
            {
                // Nothing to do in catch ( close in finally )
            } finally
            {
                try
                {
                    //--- Try to close ...
                    br.close();
                    is.close();
                } catch (IOException e1)
                {
                    // Nothing to do
                }
            }
	        
	    }
	}
	
	/**
	 * Parse the ScreenBody file (.jsp or .jspx) and update the given ScreenDefinition
	 * 
	 * @param servletContext
	 * @param rootDir
	 * @param screenDefinition
	 */
	public static void parseScreenBody( ServletContext servletContext, ScreenDefRootDir rootDir, ScreenDefinition screenDefinition )
	{
	    log.trace("parseScreenBody()...");
	    
	    String sScreenType = rootDir.getScreenType() ;
	    
	    //--- Build the Screen Body path
	    String sScreenBody = screenDefinition.getScreenBody( sScreenType );
	    String sScreenBodyPath = rootDir.getBodyPath(sScreenBody);
	    
	    //--- Parse the Screen Body
	    parse( servletContext, sScreenBodyPath, screenDefinition, sScreenType );
	    
	    //--- Flag as parsed
	    screenDefinition.setFlagScreenBodyParsed(true);
	}
}
