package org.objectweb.telosys.uil.screenmap;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.SymbolicConfigVar;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.util.web.WebUtil;

/**
 * 
 * This class contains the "conventions" to use by default ( for "Convention Over Configuration" ). <br>
 * <br>
 * It provides all the attributes loaded from the "screen_conventions" tag of the "screens.xml" file.<br>
 * It allows to build a ScreenDefinition by applying the conventions <br>
 * There's one instance of this class ( held by ScreenDefinitions )  <br>
 * 
 * @author Laurent GUERIN
 * @since 1.0.5
 *  
 */
public class ScreenConventions extends TelosysObject
{
    // <screen_default body="xxx"
    private String _body      = null ;

    // <screen_default script="xxx"
    private String _script    = null ;

    // <screen_default css="xxx"
    private String _css       = null ;

    // <screen_default template="xxx"
    private String _template  = null ;

    // <screen_default contextid="xxx"
    private String _contextid      = null ;

    // <screen_default contextname="xxx"
    private String _contextname    = null ;

    // <screen_default contextaction="xxx"
    private String _contextaction  = null ;

    //---------------------------------------------------------------------------
    /**
     * Constructor
     */
    protected ScreenConventions()
    {
        super();
        
        //--- No default values for the conventions ( can be defined or not )
        // _contextaction        = "none";
    }

    //---------------------------------------------------------------------------
    // "body"
    //---------------------------------------------------------------------------
    /**
     * Set the default "body resource" to use by convention
     * @param s the resource ( e.g. "/screen/${SCREENNAME_UC}/${SCREENNAME_UC}.jsp" )
     */
    /* package */ void setBody(String s)
    {
        if ( ! nullOrVoid(s) )
        {
            _body = s.trim() ;  
            return ;
        }
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the default "body resource" to use by convention
     * @return the resource ( e.g. "/screen/${SCREENNAME_UC}/${SCREENNAME_UC}.jsp" ), or null if none 
     */
    public String getBody()
    {
        return _body;
    }
    
    //---------------------------------------------------------------------------
    // "script"
    //---------------------------------------------------------------------------
    /**
     * Set the default "javascript resource" to use by convention
     * @param s the resource ( e.g. "/screen/${SCREENNAME_UC}/${SCREENNAME_UC}.js" )
     */
    /* package */ void setScript(String s)
    {
        if ( ! nullOrVoid(s) )
        {
            _script = s.trim() ;  
            return ;
        }
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the default "javascript resource" to use by convention
     * @return the resource ( e.g. "/screen/${SCREENNAME_UC}/${SCREENNAME_UC}.js" ), or null if none 
     */
    public String getScript()
    {
        return _script;
    }

    //---------------------------------------------------------------------------
    // "css"
    //---------------------------------------------------------------------------
    /**
     * Set the default "CSS resource" to use by convention
     * @param s the resource ( e.g. "/screen/${SCREENNAME_UC}/${SCREENNAME_UC}.css" )
     */
    /* package */ void setCss(String s)
    {
        if ( ! nullOrVoid(s) )
        {
            _css = s.trim() ;  
            return ;
        }
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the default "CSS resource" to use by convention
     * @return the resource ( e.g. "/screen/${SCREENNAME_UC}/${SCREENNAME_UC}.css" ), or null if none 
     */
    public String getCss()
    {
        return _css ;
    }

    //---------------------------------------------------------------------------
    // "template"
    //---------------------------------------------------------------------------
    /**
     * Set the default "template resource" to use by convention
     * @param s the resource ( e.g. "mytemplate.jsp" )
     */
    /* package */ void setTemplate(String s)
    {
        if ( ! nullOrVoid(s) )
        {
            _template = s.trim() ;  
            return ;
        }
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the default "template resource" to use by convention
     * @return the resource ( e.g. "mytemplate.jsp" ) , or null if none
     */
    public String getTemplate()
    {
        return _template ;
    }

    //---------------------------------------------------------------------------
    // "contextid"
    //---------------------------------------------------------------------------
    /**
     * Set the "contextid"
     * @param s
     */
    /* package */ void setContextId(String s)
    {
        if ( ! nullOrVoid(s) )
        {
            _contextid = s.trim() ;  
            return ;
        }
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the default "context id"
     * @return the id, or null if none
     */
    public String getContextId()
    {
        return _contextid ;
    }

    //---------------------------------------------------------------------------
    // "contextname"
    //---------------------------------------------------------------------------
    /**
     * Set the "contextname"
     * @param s the context name ( e.g. "${SCREENNAME_UC}" ), or null if none
     */
    /* package */ void setContextName(String s)
    {
        if ( ! nullOrVoid(s) )
        {
            _contextname = s.trim() ;  
            return ;
        }
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the default "context name"
     * @return the context name ( e.g. "${SCREENNAME_UC}" ), or null if none
     */
    public String getContextName()
    {
        return _contextname ;
    }

    //---------------------------------------------------------------------------
    // "contextaction"
    //---------------------------------------------------------------------------
    /**
     * Set the "contextaction"
     * @param s the action : "none" or "open" or "use" ( not set if other values )
     */
    /* package */ void setContextAction(String s)
    {
        if ( s != null )
        {
            String s2 = s.trim().toLowerCase() ;
            if ( s2.equals("none") ||  s2.equals("open") ||  s2.equals("use") )
            {
                _contextaction = s2;
                return ;
            }
        }
        throw new TelosysRuntimeException("Config : invalid context action : '" + s + "'");
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the default "context action"
     * @return the action : "none" or "open" or "use"  ( or null if not defined )
     */
    public String getContextAction()
    {
        return _contextaction;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Build a ScreenDefinition replacing the SCREENNAME by the given screen name parameter <br>
     * 
     * The screen existence is checked here, <br> 
     * if the required resources for the screen are not found this method returns null
     * 
     * @param servletContext
     * @param sScreenName
     * @param rootDir
     * @return a ScreenDefinition or null if the required resources don't exist
     * @since 1.0.5
     */
    protected ScreenDefinition buildScreenDefinition( ServletContext servletContext, String sScreenName, ScreenDefRootDir rootDir )
    {
        //--- Use Context ID and Context Action "as is" ( no variable replacement )
        String sContextId     = _contextid ;
        String sContextAction = _contextaction ;
        
        //--- Replace the SCREENNAME or SCREENNAME_UC or SCREENNAME_LC in context name pattern
        String sContextName = replaceSCREENNAME( _contextname, sScreenName );
        
        //--- Replace the SCREENNAME or SCREENNAME_UC or SCREENNAME_LC in the screen locations
        String sTemplate    = replaceSCREENNAME( _template, sScreenName ) ;
        String sBody        = replaceSCREENNAME( _body,     sScreenName ) ;
        String sScript      = replaceSCREENNAME( _script,   sScreenName ) ;
        String sCss         = replaceSCREENNAME( _css,      sScreenName ) ;
        
        //--- Get the full paths using the "root directories"
        String templatePath = rootDir.getTemplatePath( sTemplate );
        String bodyPath     = rootDir.getBodyPath( sBody );
        String scriptPath   = rootDir.getScriptPath( sScript );
        String cssPath      = rootDir.getCssPath( sCss );

        // String pagePath     = rootDir.getPagePath( xxx ); // TODO ?
        
        //--- Create the global ScreenDefLocation if the requiered resources exist
        ScreenDefLocation screenLoc = null ;
        if ( WebUtil.checkExistence( servletContext, templatePath ) && WebUtil.checkExistence( servletContext, bodyPath ) )
        {
            trace("buildScreenDefinition : '" + sScreenName + "' : template and body found ") ;
            screenLoc = new ScreenDefLocation();
            screenLoc.setScreenTemplate( sTemplate );
            screenLoc.setScreenBody    ( sBody );
        }
//        else if ( WebUtil.checkExistence( servletContext, pagePath ) ) // TODO ?
//        {
//            screenLoc = new ScreenDefLocation();
//            screenLoc.setScreenPage(sPage);
//        }
        else
        {
            trace("buildScreenDefinition : '" + sScreenName + "' : cannot be created : required resource(s) not found") ;
            trace("buildScreenDefinition : '" + sScreenName + "' : template = '" + templatePath + "'" ) ;
            trace("buildScreenDefinition : '" + sScreenName + "' : body = '" + bodyPath + "'" ) ;
        }
        
        //--- Add SCRIPT and CSS if they exist and create the ScreenDefinition
        if ( screenLoc != null )
        {
            //--- Is there a Javascript file for this screen ?
            if ( WebUtil.checkExistence( servletContext, scriptPath ) )
            {
                trace("buildScreenDefinition : '" + sScreenName + "' : script found ") ;
                screenLoc.setScreenScript  ( sScript );
            }
            else
            {
                trace("buildScreenDefinition : '" + sScreenName + "' : script not found '" + scriptPath + "'") ;
            }
            
            //--- Is there a CSS file for this screen ?
            if ( WebUtil.checkExistence( servletContext, cssPath ) )
            {
                trace("buildScreenDefinition : '" + sScreenName + "' : CSS found ") ;
                screenLoc.setScreenCss( sCss );
            }
            else
            {
                trace("buildScreenDefinition : '" + sScreenName + "' : CSS not found '" + cssPath + "'") ;
            }
            
            //--- Create the ScreenDefinition and return it
            ScreenDefinition screenDefinition = new ScreenDefinition(sScreenName, screenLoc, sContextId, sContextName, sContextAction);
            return screenDefinition ;
        }
        
//        //--- Create the global ScreenDefLocation 
//        ScreenDefLocation screenLoc = new ScreenDefLocation();
//        screenLoc.setScreenTemplate( sTemplate );
//        screenLoc.setScreenBody    ( sBody );
//        screenLoc.setScreenScript  ( sScript );
//        screenLoc.setScreenCss     ( sCss );
//
//        //--- Create the ScreenDefinition
//        ScreenDefinition screenDefinition = new ScreenDefinition(sScreenName, screenLoc, sContextId, sContextName, sContextAction);
//        
//        return screenDefinition ;
        
        return null ;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Replaces all the occurrences of the symbolic variables <br>
     * "${SCREENNAME}", "${SCREENNAME_UC}" and "${SCREENNAME_LC}" in the given pattern
     * 
     * @param pattern
     * @param sScreenNameValue
     * @return
     */
    private String replaceSCREENNAME( String pattern, String sScreenNameValue )
    {
        //return Tool.replaceSCREENNAME( pattern, sScreenNameValue );
        return SymbolicConfigVar.applyScreenName( pattern, sScreenNameValue );
    }
    
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    private boolean nullOrVoid(final String s)
    {
        if (s == null)
        {
            return true ;
        }
        else
        {
            if ( s.trim().length() == 0 )
            {
                return true ;
            }
        }
        return false ;
    }

}