package org.objectweb.telosys.uil.screenmap;

import org.objectweb.telosys.common.TelosysObject;

/**
 * Each instance of this class contains the screen components defined <br>
 * in the "screen" tag <br>
 * or in the "html" or "xul" child tags <br>
 * 
 * Informations stored : "template", "body", "page", "url", "script", "css" and the "ouput type" <br>
 * 
 * The "output type" depends on the tag where the definition was found : <br>
 * "screen" => unknown type<br>
 * "screen/html" => HTML type<br>
 * "screen/xul"  => XUL type<br>
 * 
 * @author Laurent GUERIN
 *  
 */
/* package */class ScreenDefLocation extends TelosysObject
{

    private final static int UNKNOWN_TYPE     = -1;

    private final static int HTML_TYPE        = 1;

    private final static int XUL_TYPE         = 2;

    private int              _iScreenType     = UNKNOWN_TYPE; // HTML or XUL

    private String           _sScreenTemplate = null;

    private String           _sScreenBody     = null;

    private String           _sScreenPage     = null;

    private String           _sScreenUrl      = null;

    private String           _sScreenCss      = null;

    private String           _sScreenScript   = null;

    //---------------------------------------------------------------------------
    //    private String buildURL(String sUrl, String sRootDir)
    //    {
    //        if (sUrl != null)
    //        {
    //            if (sUrl.indexOf('/') < 0) // No "/" in the URL
    //            {
    //                return sRootDir + "/" + sUrl;
    //            }
    //            else
    //            {
    //                return sUrl;
    //            }
    //        }
    //        else
    //        {
    //            return null;
    //        }
    //    }

    //---------------------------------------------------------------------------
    /**
     * Constructor for standard screen location parameters
     */
    protected ScreenDefLocation()
    {
        super();
        _iScreenType = UNKNOWN_TYPE ;
    }

    //---------------------------------------------------------------------------
    /**
     * Constructor for specific type location parameters ( HTML, XUL )
     * 
     * @param sType ( ie : "html", "xul" )
     */
    protected ScreenDefLocation(String sType)
    {
        super();
        _iScreenType = UNKNOWN_TYPE;
        String sTrimType = sType.trim();
        //if (sTrimType.equalsIgnoreCase("html"))
        if ( sTrimType.equalsIgnoreCase(ScreenMapConst.HTML_SCREEN_TYPE) )
        {
            _iScreenType = HTML_TYPE;
        }
        //if (sTrimType.equalsIgnoreCase("xul"))
        if ( sTrimType.equalsIgnoreCase(ScreenMapConst.XUL_SCREEN_TYPE) )
        {
            _iScreenType = XUL_TYPE;
        }
    }

//    //---------------------------------------------------------------------------
//    // Screen Type
//    //---------------------------------------------------------------------------
//    public final int getType()
//    {
//        return _iScreenType;
//    }

    //---------------------------------------------------------------------------
    // 
    //---------------------------------------------------------------------------
    private String getValidAttribute(String s)
    {
        if ( s == null )
        {
            return null ;
        }
        String sTrim = s.trim() ;
        if ( sTrim.length() == 0 )
        {
            return null ;
        }
        return sTrim ;
    }
    
    //---------------------------------------------------------------------------
    // template
    //---------------------------------------------------------------------------
    /**
     * Returns the screen template ( from "template" attribute )
     * @return the screen template, or null if not defined
     */
    public String getScreenTemplate()
    {
        return _sScreenTemplate;
    }

    /**
     * Set the screen template ( from "template" attribute )
     * @param s
     */
    protected void setScreenTemplate(String s)
    {        
        _sScreenTemplate = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // body
    //---------------------------------------------------------------------------
    /**
     * Returns the screen body ( from "body" attribute )
     * @return the screen body, or null if not defined
     */
    public String getScreenBody()
    {
        return _sScreenBody;
    }

    /**
     * Set the screen body ( from "body" attribute )
     * @param s
     */
    protected void setScreenBody(String s)
    {
        _sScreenBody = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // page
    //---------------------------------------------------------------------------
    /**
     * Returns the screen page  ( from "page" attribute )
     * @return the screen page, or null if not defined
     */
    public String getScreenPage()
    {
        return _sScreenPage;
    }

    /**
     * Set the screen page ( from "page" attribute )
     * @param s
     */
    protected void setScreenPage(String s)
    {
        _sScreenPage = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // url
    //---------------------------------------------------------------------------
    /**
     * Returns the screen URL  ( from "url" attribute )
     * @return the screen URL, or null if not defined
     */
    public String getScreenUrl()
    {
        return _sScreenUrl;
    }

    /**
     * Set the screen URL  ( from "url" attribute )
     * @param s
     */
    protected void setScreenUrl(String s)
    {
        _sScreenUrl = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // css
    //---------------------------------------------------------------------------
    /**
     * Returns the screen CSS ( from "css" attribute )
     * @return the screen CSS, or null if not defined
     */
    public String getScreenCss()
    {
        return _sScreenCss;
    }

    /**
     * Set the screen CSS ( from "css" attribute )
     * @param s
     */
    protected void setScreenCss(String s)
    {
        _sScreenCss = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    // script
    //---------------------------------------------------------------------------
    /**
     * Returns the screen script  ( from "script" attribute ) 
     * @return the screen script, or null if not defined
     */
    public String getScreenScript()
    {
        return _sScreenScript;
    }

    /**
     * Set the screen script  ( from "script" attribute ) 
     * @param s
     */
    protected void setScreenScript(String s)
    {
        _sScreenScript = getValidAttribute(s);
    }

    //---------------------------------------------------------------------------
    /**
     * Return the target for the current definition
     * Target determination by priority : 
     *  1. template attribute ( if it is specified )
     *  2. page attribute ( if it is specified and there is no template )
     *  3. url attribute ( if no template and no page )
     * @return the target (or null)
     */
    public String getScreenTarget()
    {
        if (_sScreenTemplate != null)
        {
            return _sScreenTemplate;
        }
        else if (_sScreenPage != null)
        {
            return _sScreenPage;
        }
        else
        {
            return _sScreenUrl;
        }
    }

    //---------------------------------------------------------------------------
//    public boolean isTemplate()
//    {
//        if (_sScreenTemplate != null)
//        {
//            return true;
//        }
//        return false;
//    }

    //---------------------------------------------------------------------------
//    public boolean isStandardScreen()
//    {
//        if (_iScreenType == SCREEN_TYPE)
//        {
//            return true;
//        }
//        return false;
//    }

    //---------------------------------------------------------------------------
    /**
     * Returns true if the screen type is HTML
     * @return
     */
    public boolean isHtml()
    {
        if (_iScreenType == HTML_TYPE)
        {
            return true;
        }
        return false;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns true if the screen type is XUL
     * @return
     */
    public boolean isXul()
    {
        if (_iScreenType == XUL_TYPE)
        {
            return true;
        }
        return false;
    }
    //---------------------------------------------------------------------------
}