package org.objectweb.telosys.uil.screenmap;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.i18n.Translator;
import org.objectweb.telosys.util.StrUtil;

/**
 * Each instance of this class represents a screen definition loaded from the "screens.xml" file <br>
 * or defined by convention (since v 1.1.0 )<br>
 * 
 * One instance for each screen.<br> 
 * 
 * @author Laurent GUERIN
 * 
 */
public class ScreenDefinition extends TelosysObject
{
    private String            _sScreenName     = null; // never null after construction

    private ScreenDefLocation _screenLocation  = null; // from <screen ... > : never null after construction

    private ScreenDefLocation _htmlLocation    = null; // from <screen> <html ... /> </screen> : usually null

    private ScreenDefLocation _xulLocation     = null; // from <screen> <xul ... /> </screen> : usually null

    private HashMap           _hmElements      = new HashMap(); // 0..N <screen> <element .../> </screen>
    
    private Translator        _translator      = null ;
    
    private String            _sContextId      = null; // v 1.0.5 
    
    private String            _sContextName    = null; // v 1.0.5 
    
    private String            _sContextAction  = null; // v 1.0.5 
    
    private boolean           _bScreenBodyParsed = false ; // v 1.0.5
    
    //---------------------------------------------------------------------------

    /**
     * Creates a new screen definition with the attributes located at the "screen" tag level
     * 
     * @param screenName the screen name (cannot be null) 
     * @param screenLoc the common Screen Def Location (cannot be null)
     * @param sContextId   can be null or void 
     * @param sContextName  can be null or void 
     * @param sContextAction can be null or void 
     */
    protected ScreenDefinition( String screenName, ScreenDefLocation screenLoc, 
            String sContextId,    // v 1.0.5
            String sContextName,    // v 1.0.5
            String sContextAction )  // v 1.0.5
    {
        super();
        
        if ( null == screenName ) // v 1.0.5
        {
            throw new TelosysRuntimeException("ScreenDefinition constructor : screen name parameter is null");
        }
        if ( null == screenLoc ) // v 1.0.5
        {
            throw new TelosysRuntimeException("ScreenDefinition constructor : ScreenDefLocation parameter is null");
        }
        
        _sScreenName = screenName;
        _screenLocation = screenLoc;
        
        _sContextId     = StrUtil.nullOrVoid(sContextId) ? null : sContextId.trim() ; // v 1.0.5
        _sContextName   = StrUtil.nullOrVoid(sContextName) ? null : sContextName.trim() ; // v 1.0.5
        _sContextAction = StrUtil.nullOrVoid(sContextAction) ? null : sContextAction.trim()   ; // v 1.0.5
        
        trace("ScreenDefinition created for '" + screenName + "' : " + sContextId + " " + sContextName + " " + sContextAction );
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenMap name
     * @return
     */
    public final String getScreenName()
    {
        return _sScreenName;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns true if the Screen Body file has been parsed 
     * 
     * @return
     */
    public final boolean getFlagScreenBodyParsed()
    {
        return _bScreenBodyParsed ;
    }
    
    /**
     * Set the flag saying if the Screen Body file has been parsed
     * @param v
     */
    protected final void setFlagScreenBodyParsed(boolean v)
    {
        _bScreenBodyParsed = v ;
    }
    
    //---------------------------------------------------------------------------
    
    /**
     * Returns the id of the screen context associated with this ScreenMap<br>
     * The expected values are : <br>
     * . UNDEFINED_CONTEXT_ID <br>
     * . DYNAMIC_CONTEXT_ID <br>
     * . 0 .. N  ( explicit context id ) <br>
     * 
     * @return 
     */
    public final int getScreenContextId()
    {
        if ( _sContextId != null )
        {
            return StrUtil.getInt(_sContextId, ScreenMapConst.UNDEFINED_CONTEXT_ID);
        }
        else
        {
            return ScreenMapConst.UNDEFINED_CONTEXT_ID ;
        }
    }

    /**
     * Set the id of the screen context associated with this ScreenMap
     * @param v
     * @since 1.0.5
     */
    protected final void setScreenContextId( String v)
    {
        _sContextId = v ;
    }

    //---------------------------------------------------------------------------
    
    /**
     * Returns the name of the screen context associated with this ScreenMap<br>
     * or null if none
     * @return
     */
    public final String getScreenContextName()
    {
        return _sContextName;
    }

    /**
     * Set the name of the screen context associated with this ScreenMap
     * @param v
     * @since 1.0.5
     */
    protected final void setScreenContextName( String v)
    {
        _sContextName = v ;
    }

    //---------------------------------------------------------------------------

    /**
     * Returns the default screen context action for this ScreenMap, or null if none
     * @return
     */
    public final String getScreenContextAction()
    {
        return _sContextAction;
    }

    /**
     * Set the default screen context action for this ScreenMap
     * @param v
     * @since 1.0.5
     */
    protected final void setScreenContextAction( String v )
    {
        _sContextAction = v ;
    }

    //---------------------------------------------------------------------------
    // Screen Location
    //---------------------------------------------------------------------------
    /**
     * Set a specific "html" or "xul" definition <br>
     * This method is used by the loader when a subtag "screen/html" or "screen/xul" is encountered
     * 
     * @param screenLoc
     */
    protected void setScreenDefLocation(ScreenDefLocation screenLoc)
    {
        if (screenLoc.isHtml())
        {
            _htmlLocation = screenLoc;
        }
        else if (screenLoc.isXul())
        {
            _xulLocation = screenLoc;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the common screen location (used for both "html" and "xml" types)
     * @return
     * @since 1.0.5
     */
    protected final ScreenDefLocation getScreenDefLocation() // v 1.0.5
    {
        return _screenLocation ;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Returns the screen definition associated with the given type, or null if no specific definition has been defined
     * @param sScreenTypeParam "html" or "xul" ( not case sensitive )
     * @return
     */
    //private final ScreenDefLocation getScreenDefLocation(String sScreenTypeParam )
    protected final ScreenDefLocation getScreenDefLocation(String sScreenTypeParam ) // v 1.0.5
    {
//        ScreenDefLocation ret = null;
//        if (sType.equals(ScreenMapConst.HTML_SCREEN_TYPE))
//        {
//            ret = _htmlLocation;
//        }
//        else if (sType.equals(ScreenMapConst.XUL_SCREEN_TYPE))
//        {
//            ret = _xulLocation;
//        }
//        return ret;
        
        String sScreenType = ( sScreenTypeParam != null ? sScreenTypeParam : TelosysUIL.getDefaultScreenType() ) ;
        
        if ( ScreenMapConst.HTML_SCREEN_TYPE.equalsIgnoreCase( sScreenType) ) 
        {
            return _htmlLocation;
        }
        if ( ScreenMapConst.XUL_SCREEN_TYPE.equalsIgnoreCase( sScreenType) ) 
        {
            return _xulLocation;
        }
        return null ;            
    }

    //---------------------------------------------------------------------------
    // Screen Map Elements
    //---------------------------------------------------------------------------
    /**
     * Adds a screen element <br>
     * This method is used by the loader when a subtag "screen/element" is encountered
     * 
     * @param elem
     */
    protected void addElement(ScreenMapElement elem)
    {
        if (elem != null)
        {
            if (elem.getName() != null)
            {
                _hmElements.put(elem.getName(), elem);
            }
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Set the elements  
     * @param hm
     */
    protected void setElements(HashMap hm)
    {
        _hmElements = hm;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns all the elements defined 
     * @return
     */
    public HashMap getElements()
    {
        return _hmElements;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Returns the number of elements
     * @return
     */
    public int getElemCount()
    {
        if ( _hmElements == null ) 
        {
            return 0 ;
        }
        else
        {
            return _hmElements.size();
        }
    }
    
    //---------------------------------------------------------------------------
    // Screen Map Translator
    //---------------------------------------------------------------------------
    /**
     * Set the translator to use
     * @param t
     */
    protected void setTranslator(Translator t)
    {
        _translator = t;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the translator to use 
     * @return
     */
    public Translator getTranslator()
    {
        return _translator;
    }

    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    /**
     * Returns true if the screen is composed of a template and a screen body
     * @param sType
     * @return
     */
    public boolean isBodyTarget(String sType)
    {
        //--- If this screen definition contains a template and a body ...
        if ( ( getScreenTemplate(sType) != null ) && ( getScreenBody(sType) != null ) )
        {
            return true ;
        }
        return false ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns true if the screen is a single resource ( just one JSP, no template + body )
     * @param sType
     * @return
     */
    public boolean isPageTarget(String sType)
    {
        //--- If this screen definition contains a "page" attribute ...
        if ( getScreenPage(sType) != null  )
        {
            return true ;
        }
        return false ;
    }
    //---------------------------------------------------------------------------
    private String getScreenTemplate(String sType)
    {
        String sRet = _screenLocation.getScreenTemplate();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenTemplate() != null )
            {
                sRet = loc.getScreenTemplate();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    private String getScreenPage(String sType)
    {
        String sRet = _screenLocation.getScreenPage();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenPage() != null )
            {
                sRet = loc.getScreenTemplate();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the screen target ( template, simpe page or URL )
     * @param sType
     * @return the path to the target
     */
    public String getScreenTarget(String sType)
    {
        String sRet = _screenLocation.getScreenTarget();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenTarget() != null )
            {
                sRet = loc.getScreenTarget();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the "screen body" file part for the given type 
     * ( without the "root dir" part ) 
     * @param sType
     * @return
     */
    public String getScreenBody(String sType)
    {
        String sRet = _screenLocation.getScreenBody();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenBody() != null )
            {
                sRet = loc.getScreenBody();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the CSS to use 
     * @param sType
     * @return
     */
    public String getScreenCss(String sType)
    {
        String sRet = _screenLocation.getScreenCss();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenCss() != null )
            {
                sRet = loc.getScreenCss();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the Javascript to use
     * @param sType
     * @return
     */
    public String getScreenScript(String sType)
    {
        String sRet = _screenLocation.getScreenScript();
        ScreenDefLocation loc = getScreenDefLocation(sType);
        if ( loc != null )
        {
            if ( loc.getScreenScript() != null )
            {
                sRet = loc.getScreenCss();
            }
        }
        return sRet ;
    }
    //---------------------------------------------------------------------------
}
