package org.objectweb.telosys.uil.screenmap;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.uil.TelosysUIL;

/**
 * An instance of this class contains all the screen definitions loaded from the "screens.xml" file.<br>
 * 
 * It's the ScreenMap provider. <br>
 * The screens definitions (and the default configuration values) are used to build <br>
 * all the elements composing a ScreenMap ( body, script, css, template, and so on )  
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenDefinitions extends TelosysObject
{
    //private final static int         UNKNOWN_TYPE = -1;

    //private final static int SCREEN_TYPE = 0;

    //private final static int         HTML_TYPE    = 1;

    //private final static int         XUL_TYPE     = 2;

    private final TelosysClassLogger _log            = new TelosysClassLogger(ScreenDefinitions.class);

    private ScreenDefConfig          _config         = new ScreenDefConfig();

    private ScreenConventions        _screenConventions = new ScreenConventions(); // v 1.0.5

    private ScreenDefRootDir         _htmlRootDir    = new ScreenDefRootDir( ScreenMapConst.HTML_SCREEN_TYPE );

    private ScreenDefRootDir         _xulRootDir     = new ScreenDefRootDir( ScreenMapConst.XUL_SCREEN_TYPE);

    //--- Screens Definitions : "screen name" --> ScreenDefinition instance
    private HashMap                  _hmScreenDefinitions  = new HashMap(20);

    /**
     * Constructor
     */
    public ScreenDefinitions()
    {
        super();
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the instance containing the attributes of the <config> tag 
     * @return
     */
    public ScreenDefConfig getConfig()
    {
            return _config;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the instance containing the attributes of the <sccreen_defaults> tag 
     * @return
     */
    public ScreenConventions getScreenConventions()
    {
            return _screenConventions;
    }

//    //---------------------------------------------------------------------------
//    private boolean isHtmlType(String sScreenType)
//    {
//        //return sScreenType.equalsIgnoreCase("html");
//        return sScreenType.equalsIgnoreCase( ScreenMapConst.HTML_SCREEN_TYPE );
//    }
//
//    //---------------------------------------------------------------------------
//    private boolean isXulType(String sScreenType)
//    {
//        //return sScreenType.equalsIgnoreCase("xul");
//        return sScreenType.equalsIgnoreCase( ScreenMapConst.XUL_SCREEN_TYPE );
//    }

    //---------------------------------------------------------------------------
    /**
     * Returns the <rootdir> attributes for "HTML" screen type
     * @return
     */
    public ScreenDefRootDir getHtmlRootDir()
    {
            return _htmlRootDir;
    }
    /* package */ void setHtmlRootDir(ScreenDefRootDir x)
    {
        _htmlRootDir = x;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Returns the <rootdir> attributes for "XUL" screen type
     * @return
     */
    public ScreenDefRootDir getXulRootDir()
    {
            return _xulRootDir;
    }
    /* package */ void setXulRootDir(ScreenDefRootDir x)
    {
        _xulRootDir = x;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the <rootdir> attributes for the given screen type ( "HTML" or "XUL" )
     * @param sScreenTypeParam  "html" or "xul" ( not case sensitive )
     * @return
     */
    public ScreenDefRootDir getRootDir(String sScreenTypeParam)
    {
        String sScreenType = ( sScreenTypeParam != null ? sScreenTypeParam : TelosysUIL.getDefaultScreenType() ) ;
        if ( ScreenMapConst.HTML_SCREEN_TYPE.equalsIgnoreCase( sScreenType) ) 
        {
            return _htmlRootDir;
        }
        if ( ScreenMapConst.XUL_SCREEN_TYPE.equalsIgnoreCase( sScreenType) ) 
        {
            return _xulRootDir;
        }
        return null ;
        
//        if ( sScreenType != null )
//        {
//            //if (isHtmlType(sScreenType))
//            if ( ScreenMapConst.HTML_SCREEN_TYPE.equalsIgnoreCase( sScreenType) ) 
//            {
//                return _htmlRootDir;
//            }
//            //else if (isXulType(sScreenType))
//            if ( ScreenMapConst.XUL_SCREEN_TYPE.equalsIgnoreCase( sScreenType) ) 
//            {
//                return _xulRootDir;
//            }
//        }
//        else
//        {
//            if ( ScreenMapConst.HTML_SCREEN_TYPE.equalsIgnoreCase( TelosysUIL.getDefaultScreenType() ) )
//            {
//                return _htmlRootDir;
//            }
//            if ( ScreenMapConst.XUL_SCREEN_TYPE.equalsIgnoreCase( TelosysUIL.getDefaultScreenType() ) )
//            {
//                return _xulRootDir;
//            }
//        }
//        return null ;            
    }

    //---------------------------------------------------------------------------
    /**
     * Creates a ScreenMap instance for the given screen informations ( name, type, context id, context name )
     * 
     * @param servletContext
     * @param sScreenName   cannot be null 
     * @param sScreenType   can be null 
     * @param iScreenContextId   can be UNDEFINED_CONTEXT_ID ( same as null ) 
     * @param sScreenContextName   can be null (if null the screen name is used)
     * @param sScreenContextAction  can be null (if null the default action is used)
     * @return
     */
    public ScreenMap getScreenMap( ServletContext servletContext, // v 1.0.5 ( + servletContext )
            String sScreenName, String sScreenType, 
            int iScreenContextId, 
            String sScreenContextName,
            String sScreenContextAction ) // v 1.0.5  ( + sScreenContextAction )
    {
        // TODO : trim on sScreenName, sScreenType, ...
        _log.trace("getScreenMap(" + sScreenName + "," + sScreenType + "," + iScreenContextId + "," + sScreenContextName + ")");

        //--- Search a screen definition for this screen name
        ScreenDefinition screenDefinition = (ScreenDefinition) _hmScreenDefinitions.get(sScreenName);
        if ( null == screenDefinition )
        {
            // Not found in the definitions list => try to load the definition using the conventions
            screenDefinition = getScreenDefinitionByConvention ( servletContext, sScreenName, sScreenType );
        }

        if ( null == screenDefinition ) // Still null ?
        {
            _log.error("No screen definition for the screen : '" + sScreenName + "' ");
            return null;
        }
        
        //--- Get the "root dir" definition for this type of screen
        ScreenDefRootDir rootDir = getRootDir(sScreenType);
        if (rootDir == null)
        {
            _log.error("No root dir for screen type : '" + sScreenType + "' ");
            return null;
        }
        
        //--- Parse the ScreenBody if not yet done 
        if ( screenDefinition.getFlagScreenBodyParsed() != true )
        {
            _log.trace("getScreenMap() : parse screen body ...");
            ScreenConfigParser.parseScreenBody(servletContext, rootDir, screenDefinition);
        }
        else
        {
            _log.trace("getScreenMap() : no screen body parsing (already done) ");
        }
        
        //--- Create and return the ScreenMap instance
        //return new ScreenMap(rootDir, screenDef, sScreenType, iScreenContextId, sScreenContextName ); // v 0.9.9
        return new ScreenMap(rootDir, screenDefinition, sScreenType, iScreenContextId, sScreenContextName, sScreenContextAction ); // v 1.0.5
    }

    //---------------------------------------------------------------------------
    /**
     * Try to get a ScreenDefinition using the conventions defined in "screen_conventions"
     * 
     * @param servletContext
     * @param sScreenName
     * @param sScreenType
     * @return
     * @since 1.0.5
     */
    private ScreenDefinition getScreenDefinitionByConvention ( ServletContext servletContext, String sScreenName, String sScreenType )
    {
        trace("getScreenDefinitionByConvention(...," + sScreenName + "," + sScreenType + ")" ) ;
        if ( _screenConventions != null )
        {
            ScreenDefRootDir rootDir = getRootDir(sScreenType);

            //--- Build a ScreenDefinition based on SCREENNAME replacement
            ScreenDefinition screenDefinition = _screenConventions.buildScreenDefinition(servletContext, sScreenName, rootDir);
            if ( screenDefinition != null )
            {
                trace("getScreenDefinitionByConvention : ScreenDefinition created => register it " ) ;
                
//                
//                //--- Check the existence of the screen body 
//                String body = rootDir.getBodyPath( screenDefinition.getScreenBody(sScreenType) );
//                //String body = screenDefinition.getScreenBody(sScreenType);
//                
//                trace("getScreenDefinitionByConvention : screenDefinition.getScreenBody(" + sScreenType + ") --> '" + body + "'") ;
//                
//                if ( WebUtil.checkExistence( servletContext, body ) )
//                {
//                    trace("getScreenDefinitionByConvention : '" + body + "' exists ") ;
//                    //--- The screen body exists => this definition can be registered
//                    addScreenDefinition( screenDefinition );
//                    return screenDefinition ;
//                }
//                else
//                {
//                    trace("getScreenDefinitionByConvention : '" + body + "' doesn't exist ") ;
//                }
                
                addScreenDefinition( screenDefinition );
                return screenDefinition ;
            }
        }
        return null ;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Adds a new screen definition (loaded form "screens.xml" )
     * @param screenDef
     */
    /* package */ void addScreenDefinition(ScreenDefinition screenDef)
    {
        _log.trace("addScreenDefinition(" + screenDef.getScreenName() + ")");
        _hmScreenDefinitions.put(screenDef.getScreenName(), screenDef);
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the screen definition for the given screen name, or null if none
     * @param sScreenName
     * @return
     */
    public ScreenDefinition getScreenDefinition(String sScreenName)
    {
        _log.trace("getScreenDefinition(" + sScreenName + ")");

        //--- Search a screen definition for this screen name
        return (ScreenDefinition) _hmScreenDefinitions.get(sScreenName);
    }

    //---------------------------------------------------------------------------
    /**
     * Returns an iterator for all the screen names currently defined
     * @return
     */
    public Iterator getScreenNames()
    {
        Set keys = _hmScreenDefinitions.keySet();
        return ( keys != null ? keys.iterator() : null );
    }

    //---------------------------------------------------------------------------
    /**
     * Returns an array of all the screen names currently defined
     * @return
     */
    public String[] getScreenNamesArray()
    {
        Set keys = _hmScreenDefinitions.keySet();
        if ( keys != null )
        {
            int size = keys.size();
            String[] names = new String[size] ;
            int index = 0 ;
            Iterator iter = keys.iterator() ;
            while ( iter.hasNext() )
            {
                names[index++] = (String) iter.next();
            }
            return names ;
        }
        else
        {
            return new String[0] ;
        }
    }
    
    
    //---------------------------------------------------------------------------
    /**
     * Returns the current number of screen definitions <br>
     * it can evolve if the Convention over Configuration is used ( see <screen_conventions> )
     * @return
     */
    public int getScreenDefCount()
    {
        return _hmScreenDefinitions.size();
    }
    //---------------------------------------------------------------------------
}