package org.objectweb.telosys.upload;

import java.io.File;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.util.DirUtil;
import org.objectweb.telosys.util.FileUtil;

/**
 * File management methods
 * 
 */
public class FileManager
{
    private static final String ERROR_NOT_A_DIRECTORY     = "Not a directory : ";
    private static final String ERROR_NOT_A_FILE          = "Not a file : ";
    private static final String FILE_PARAMETER_IS_NULL    = "File parameter is null ";
    private static final String UPLOAD_DIRECTORY_IS_NULL  = "Upload directory is null ";

    /**
     * Retuns the FileUploaded object stored in the Http Session 
     * @param req
     * @param sSessionKey the session attribute key 
     * @return
     */
    public final static FileUploaded getFileFromHttpSession(HttpServletRequest req, String sSessionKey )
    {
        HttpSession session = req.getSession(false); // No session creation 
        if ( session != null )
        {
            Object obj = session.getAttribute(sSessionKey);
    		if ( obj != null )
    		{
        		if ( obj instanceof FileUploaded )
        		{
        		    return (FileUploaded) obj ;
        		}
    		}
        }
        return null ;
    }
    
    /**
     * Retuns the FileUploaded object stored as a Screen Context attribute 
     * @param request
     * @param iContextId
     * @param sContextAttribute
     * @return
     */
    public final static FileUploaded getFileFromScreenContext(HttpServletRequest request, int iContextId, String sContextAttribute )
    {
        ScreenContext screenContext = null ;
        try
        {
            screenContext = ScreenContextManager.getScreenContext(request, iContextId);
        } catch (TelosysException e)
        {
            return null ;
        }
        if (screenContext != null)
        {
            Object obj = screenContext.getAttribute(sContextAttribute);
    		if ( obj != null )
    		{
        		if ( obj instanceof FileUploaded )
        		{
        		    return (FileUploaded) obj ;
        		}
    		}
        }
        return null ;
    }
    
    /**
     * Returns a File object for the given parameters
     * @param psUploadDirectory
     * @param psDir
     * @param psFileName
     * @return
     * @throws TelosysException
     */
    public final static File getFileFromFilesystem(String psUploadDirectory, String psDir, String psFileName ) throws TelosysException
    {
        String sFilePath = FileManager.buildFilePath(psUploadDirectory, psDir, psFileName );
        //if ( FileUtil.isFile(sFilePath) )
        if ( FileUtil.exists(sFilePath) ) // v 1.1.0
        {
            return new File( sFilePath );
        }
        else
        {
            throw new TelosysException( ERROR_NOT_A_FILE + sFilePath );
        }
    }
    
    

    /**
     * Builds a file path using the given parameters
     * @param psUploadDirectory the "upload directory" (cannot be null)
     * @param psDir the subdirectory in the "upload directory" ( if null : none )
     * @param psFileName the file name 
     * @return
     * @throws TelosysException
     */
    public final static String buildFilePath( String psUploadDirectory, String psDir, String psFileName ) throws TelosysException
    {
        if ( psUploadDirectory == null )
        {
            throw new TelosysException( UPLOAD_DIRECTORY_IS_NULL );
        }
        if ( psFileName == null )
        {
            throw new TelosysException( FILE_PARAMETER_IS_NULL );
        }
        String sDirectory = ( psDir != null ? psDir : "" ) ;
        sDirectory = sDirectory.trim();

    	//--- Build the full directory
        if (sDirectory.startsWith("/"))
        {
            sDirectory = sDirectory.substring(1); // remove 1st char
        }
        if ( sDirectory.equals(".") ) // "." = save in the standard upload dir
        {
            sDirectory = ""; // no subdirectory                 
        }
        sDirectory = psUploadDirectory + sDirectory;
        if (!DirUtil.exists(sDirectory))
        {
            throw new TelosysException(ERROR_NOT_A_DIRECTORY + sDirectory);
        }
        
        String sFilePath = null ;
        if ( sDirectory.endsWith("/") )
        {
            sFilePath = sDirectory + psFileName ;
        }
        else
        {
            sFilePath = sDirectory + "/" + psFileName ;
        }
        return sFilePath ;
    }
    
    /**
     * Removes the FileUploaded object from the Http Session if any 
     * @param req
     * @param sSessionKey the session attribute key
     * @return
     */
    public final static boolean removeFileFromHttpSession(HttpServletRequest req, String sSessionKey )
    {
        HttpSession session = req.getSession(false); // No session creation 
        if ( session != null )
        {
            Object obj = session.getAttribute(sSessionKey);
    		if ( obj != null )
    		{
        		if ( obj instanceof FileUploaded )
        		{
                    session.removeAttribute(sSessionKey);
                    return true ;
        		}
    		}
        }
        return false ;
    }
    
//    /**
//     * Generates the Http Response from the file content 
//     * @param response
//     * @param out
//     * @param file
//     * @return true if the response has been generated, false if nothing done
//     * @throws IOException
//     */
//    public final static boolean generateFileContent(HttpServletResponse response, ServletOutputStream out, FileUploaded file ) throws IOException
//    {
//        if ( response == null ) return false ;
//        if ( out == null ) return false ;
//        if ( file == null ) return false ;
//        byte[] data = file.getData() ;
//        if ( data == null ) return false ;
//        
//        // Write the response  
//        String sType = file.getFileType() ;
//        response.setContentType(sType);
//        response.setContentLength(data.length);
//        //ServletOutputStream out = response.getOutputStream(); // Cannot be called more than 1 time !!
//        out.write(data);
//        
//        return true ;
//    }
}
