package org.objectweb.telosys.upload;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysProperties;

/**
 * Multipart request parser
 */
/* package */ class MultipartParser
{

    //private static final String DEFAULT_CHARSET = "ISO-8859-1";
    
    private String             _sCharset = "ISO-8859-1"; 
    
    /** input stream to read parts from */
    private ServletInputStream _in = null ;

    /** MIME boundary that delimits parts */
    private String             _sBoundary = null ;

    /** reference to the last file part we returned */
    //private Part               _lastFilePart = null;

    /** buffer for readLine method */
    private byte[]             _buffer = new byte[8 * 1024];

//    /**
//     * Constructor without limit length
//     */
//    public MultipartParser(HttpServletRequest req, long maxPostSize) throws IOException
//    {
//        this(req, maxPostSize, true, true);
//    }

    /**
     * Constructor with specific limit length
     * @param request
     * @param maxPostSize
     * @throws IOException
     */
    public MultipartParser(HttpServletRequest request, long maxPostSize) throws IOException
    {
        /***
        // Check the content type to make sure it's "multipart/form-data"
        // Access header two ways to work around WebSphere oddities
        String contentType = null;
        String type1 = req.getHeader("Content-Type");
        String type2 = req.getContentType();
        // If one value is null, choose the other value
        if (type1 == null && type2 != null)
        {
            contentType = type2;
        }
        else if (type2 == null && type1 != null)
        {
            contentType = type1;
        }
        // If neither value is null, choose the longest value
        else if (type1 != null && type2 != null)
        {
            contentType = (type1.length() > type2.length() ? type1 : type2);
        }
        if (contentType == null || !contentType.toLowerCase().startsWith("multipart/form-data"))
        {
            throw new IOException("Posted content type isn't multipart/form-data");
        }
        ***/
        
        //--- Init the charset ( v 1.0.5 )
        Properties prop = Telosys.getProperties();
        if ( prop != null )
        {
            String s = prop.getProperty( TelosysProperties.UPLOAD_CHARSET );
            if ( s != null )
            {
                _sCharset = s.trim() ;
            }
        }
        
        //--- Check the content length to prevent denial of service attacks
        int length = request.getContentLength();
        if (length > maxPostSize)
        {
            throw new IOException("Request content too large (" + length + ")");
        }
        
        //--- Get the Content-Type http header
        // something like "multipart/form-data; boundary=---------------------------7da336323080e"
        String contentType = extractRequestContentType(request) ; // v 1.0.5
        
        //--- Get the boundary string ( included in the content type ) 
        // something like "------------------------12012133613061"
        String boundary = extractBoundary(contentType);
        if (boundary == null)
        {
            throw new IOException("Separation boundary was not specified");
        }
        
        // Save our values for later
        this._in = request.getInputStream();
        this._sBoundary = boundary;
        
        // Read the first line, should be the first boundary
        String line = readLine();
        if (line == null)
        {
            throw new IOException("Corrupt form data: premature ending");
        }
        // Verify that the line is the boundary
        if (!line.startsWith(boundary))
        {
            throw new IOException("Corrupt form data: no leading boundary");
        }
    }

    /**
     * Reads and returns the next part in the stream
     * @return : the next 'Part' or null at the end
     * @throws IOException
     */
    /* package */ Part readNextPart() throws IOException
    {
        // Read the first line, should look like this:
        // content-disposition: form-data; name="field1"; filename="file1.txt"
        String line = readLine();
        if (line == null)
        {
            // No parts left, we're done
            return null;
        }
        else if (line.length() == 0)
        {
            // IE4 on Mac sends an empty line at the end; treat that as the end.
            return null;
        }
        
        //--- Parse the content-disposition line
        String[] dispInfo = extractContentDisposition(line);
        // String disposition = dispInfo[0]; // not used
        String name = dispInfo[1];
        String filename = dispInfo[2];
        // Now onto the next line. This will either be empty
        // or contain a Content-Type and then an empty line.
        line = readLine();
        if (line == null)
        {
            // No parts left, we're done
            return null;
        }
        
        //--- Get the content type, or null if none specified
        String contentType = extractContentType(line);
        if (contentType != null)
        {
            // Eat the empty line
            line = readLine();
            if (line == null || line.length() > 0)
            { // line should be empty
                throw new IOException("Malformed line after content type: " + line);
            }
        }
        else
        {
            // The default content type is "text/plain" ( see rfc1867 )
            contentType = "text/plain";
        }
        
        //--- New part management since v 1.0.5
        // Now, finally, we read the content (end after reading the boundary)
        // _lastFilePart = new Part(name, _in, _sBoundary, contentType, filename);
        
        byte[] content = readPartContent();
        Part part = null ;
        if ( filename != null )
        {
            //--- This part is a file
            part = new Part(name, contentType, filename, content );
        }
        else
        {
            //--- This part is a parameter
            String value = new String(content, _sCharset);
            part = new Part(name, value );
        }
        return part;
    }

    /**
     * Reads and returns the next part content
     * @return
     * @throws IOException
     * @since v 1.0.5
     */
    private byte[] readPartContent() throws IOException
    {
        PartInputStream pis = new PartInputStream(_in, _sBoundary);
        ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
        byte[] buf = new byte[128];
        int read;
        while ((read = pis.read(buf)) != -1)
        {
            baos.write(buf, 0, read);
        }
        pis.close();
        baos.close();
        return baos.toByteArray();
    }
    
    /**
     * Returns the Content-Type of the given http request
     * @param request
     * @return
     * @throws IOException
     */
    private String extractRequestContentType(HttpServletRequest request) throws IOException
    {
        // Check the content type to make sure it's "multipart/form-data"
        // Access header two ways to work around WebSphere oddities
        String contentType = null;
        String type1 = request.getHeader("Content-Type");
        String type2 = request.getContentType();
        // If one value is null, choose the other value
        if (type1 == null && type2 != null)
        {
            contentType = type2;
        }
        else if (type2 == null && type1 != null)
        {
            contentType = type1;
        }
        // If neither value is null, choose the longest value
        else if (type1 != null && type2 != null)
        {
            contentType = (type1.length() > type2.length() ? type1 : type2);
        }
        if ( contentType == null || !contentType.toLowerCase().startsWith("multipart/form-data") )
        {
            throw new IOException("Posted content type is not multipart/form-data");
        }
        return contentType ;
    }
    
    /**
     * Extracts and returns the 'boundary token' from the given line
     * @param contentType ( example : "multipart/form-data; boundary=---------------------------7da336323080e" )
     * @return
     */
    private String extractBoundary(String contentType)
    {
        //--- Use lastIndexOf() because old browsers like IE 4 send the "boundary=" string multiple times.
        int index = contentType.lastIndexOf("boundary=");
        if (index == -1)
        {
            return null;
        }
        String boundary = contentType.substring(index + 9); // 9 for "boundary="
        //--- The real boundary is always preceeded by an extra "--"
        boundary = "--" + boundary;
        return boundary;
    }

    /**
     * Extracts and returns disposition info from a line <br>
     * Line example : Content-Disposition: form-data; name="field1"; filename="D:\dir\file1.txt"
     * @param line
     * @return : the line's disposition data as a String array : 0:disposition, 1:name, 2:filename
     * @throws IOException
     */
    private String[] extractContentDisposition(String line) throws IOException
    {
        // Convert the line to a lowercase string without the ending \r\n
        // Keep the original line for error messages and for variable names.
        String origline = line;
        line = origline.toLowerCase();
        
        //--- Get the content disposition, should be "form-data"
        int start = line.indexOf("content-disposition: ");
        int end = line.indexOf(";");
        if (start == -1 || end == -1)
        {
            throw new IOException("Content disposition corrupt: " + origline);
        }
        String disposition = line.substring(start + 21, end);
        if (!disposition.equals("form-data"))
        {
            throw new IOException("Invalid content disposition: " + disposition);
        }
        
        //--- Get the field name
        start = line.indexOf("name=\"", end); // start at last semicolon
        end = line.indexOf("\"", start + 7); // skip name=\"
        if (start == -1 || end == -1)
        {
            throw new IOException("Content disposition corrupt: " + origline);
        }
        String name = origline.substring(start + 6, end);
        
        //--- Get the filename, if given
        String filename = null;
        start = line.indexOf("filename=\"", end + 2); // start after name
        end = line.indexOf("\"", start + 10); // skip filename=\"
        if (start != -1 && end != -1)
        { // note the !=
            filename = origline.substring(start + 10, end);
            // The filename may contain a full path. Cut to just the filename.
            int slash = Math.max(filename.lastIndexOf('/'), filename.lastIndexOf('\\'));
            if (slash > -1)
            {
                filename = filename.substring(slash + 1); // past last slash
            }
        }
        
        //--- Return a String array: disposition, name, filename ( empty filename = no file posted )
        String[] retval = new String[3];
        retval[0] = disposition;
        retval[1] = name;
        retval[2] = filename;
        return retval;
    }

    /**
     * Extracts and returns the content type from the given line
     * @param sLine
     * @return : the content type ( i.e. "text/plain", ... )
     * @throws IOException
     */
    private String extractContentType(String sLine) throws IOException
    {
        String contentType = null;
        // Convert the line to a lowercase string
        String origline = sLine;
        sLine = origline.toLowerCase();
        // Get the content type, if any
        if (sLine.startsWith("content-type"))
        {
            int start = sLine.indexOf(" ");
            if (start == -1)
            {
                throw new IOException("Content type corrupt: " + origline);
            }
            contentType = sLine.substring(start + 1);
        }
        else if (sLine.length() != 0)
        { // no content type, so should be empty
            throw new IOException("Malformed line after disposition: " + origline);
        }
        return contentType;
    }

    /**
     * Reads the next input line.
     * @return 
     * @throws IOException
     */
    private String readLine() throws IOException
    {
        StringBuffer sbuf = new StringBuffer();
        int result;
        do
        {
            result = _in.readLine(_buffer, 0, _buffer.length); // does +=
            if (result != -1)
            {
                //sbuf.append(new String(_buffer, 0, result, "ISO-8859-1")); 
                sbuf.append(new String(_buffer, 0, result, _sCharset)); 
            }
        } while (result == _buffer.length); // loop only if the buffer was filled

        if (sbuf.length() == 0)
        {
            return null; // nothing read, must be at the end of stream
        }
        sbuf.setLength(sbuf.length() - 2); // cut off the trailing \r\n
        return sbuf.toString();
    }

}

