package org.objectweb.telosys.upload;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysProperties;
import org.objectweb.telosys.util.StrUtil;

/**
 * Multipart request manager<br>
 * Parses the given HttpServletRequest and holds all the parts contained in the request.<br>
 * Provides "file parts" and "parameter part"
 */
/* package */ class MultipartRequest
{

    private static final int   DEFAULT_MAX_POST_SIZE = 100 * 1024 * 1024; // 100 Mb

    public static final String MULTIPART             = "multipart/";

    private Hashtable          _htParameters         = new Hashtable();  // name - ParamPart

    private Hashtable          _htFiles              = new Hashtable();  // name - FilePart

    //private MultipartParser    _parser               = null ;  // parser

    /**
     * Constructor with default max post size ( 100 Mb or value specified in .properties )
     * @param request
     * @throws IOException
     */
    public MultipartRequest(HttpServletRequest request) throws IOException
    {
        // this(request, DEFAULT_MAX_POST_SIZE);
        //--- Init the maximum size ( v 1.0.5 )
        int iMaxPostSize = DEFAULT_MAX_POST_SIZE ;
        Properties prop = Telosys.getProperties();
        if ( prop != null )
        {
            String s = prop.getProperty( TelosysProperties.UPLOAD_MAX_SIZE );
            if ( s != null )
            {
                int i = StrUtil.getInt(s,-1);
                if ( i > 0 )
                {
                    iMaxPostSize = i * 1024 * 1024 ; // Mega bytes 
                }
            }
        }
        initAndParse( request,  iMaxPostSize);
    }

    /**
     * Constructor with generic max post size
     * @param request
     * @param maxPostSize
     * @throws IOException
     */
    public MultipartRequest(HttpServletRequest request, long maxPostSize) throws IOException
    {
        initAndParse( request,  maxPostSize);
    }
    
    /**
     * Init the instance and parse the http request
     * @param request
     * @param maxPostSize
     * @throws IOException
     */
    private void initAndParse(HttpServletRequest request, long maxPostSize) throws IOException
    {
        //--- Check arguments
        if (request == null)
        {
            throw new IllegalArgumentException("MultipartRequest : request cannot be null");            
        }
        if (maxPostSize <= 0)
        {
            throw new IllegalArgumentException("MultipartRequest : maxPostSize must be > 0");
        }
        
        //--- Create the parser
        MultipartParser parser = new MultipartParser(request, maxPostSize);
        
        //--- Use it to get all the PARTS contained in the request
        Part part;
        while ((part = parser.readNextPart()) != null)
        {
            //--- Add the part in the correct Hashtable
            String sPartName = part.getName();
            if (part.isFile())
            {
                // It's a FILE part
                _htFiles.put(sPartName, part);
            }
            else
            {
                // It's a PARAMETER part
                _htParameters.put(sPartName, part);
            }
        }
    }

    /**
     * Returns all the names of the "parameter parts" held
     * @return
     */
    public Enumeration getParameterNames()
    {
        return _htParameters.keys();
    }

    /**
     * Returns all the names of the "file parts" held
     * @return
     */
    public Enumeration getFileNames()
    {
        return _htFiles.keys();
    }

    /**
     * Returns the "parameter part" from the given name
     * @param name
     * @return
     */
    public Part getParameter(String name)
    {
        return (Part) _htParameters.get(name);
    }

    /**
     * Returns the "file part" from the given name<br>
     * The "name" parameter is the HTML "name" attribute of the "input" tag ( i.e. input type="file" name="file"  )
     * @param name
     * @return
     */
    public Part getFile(String name)
    {
        return (Part) _htFiles.get(name);
    }

    /**
     * Check if the given http request is really "multipart"
     * @param request
     * @return : true if "multipart", else false
     */
    public static final boolean isMultipartContent(HttpServletRequest request)
    {
        //--- The HTTP method must be "POST"
        if (!"post".equals(request.getMethod().toLowerCase()))
        {
            return false;
        }
        
        //--- The CONTENT TYPE must starts with "multipart/" 
        String contentType = request.getContentType();
        if (contentType == null)
        {
            return false;
        }
        if (contentType.toLowerCase().startsWith(MULTIPART))
        {
            return true;
        }
        return false;
    }

}