package org.objectweb.telosys.util;

import java.lang.reflect.Field;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * Utility class for JAVA CLASS operations ( set of static methods )
 * 
 * @author Laurent GUERIN
 * 
 */
public final class ClassUtil
{
    /**
     * No constructor for this class !
     */
    private ClassUtil()
    {
    }

    //-----------------------------------------------------------------------------------
    /**
     * Return the value of a static field for a given class.
     * @param cl : the class
     * @param sFieldName : the field name to find in the class
     * @return : the field value
     * @throws TelosysException
     */
    public static Object getFieldValue ( Class cl, String sFieldName ) throws TelosysException
    {
        String sErrMsg = "getStaticFieldValue(" + cl.getName() + ", " + sFieldName + ") : ";
        try
        {
            Field field = cl.getDeclaredField(sFieldName);
            return field.get(null) ;
        } catch (SecurityException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        } catch (NoSuchFieldException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        } catch (IllegalArgumentException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        } catch (IllegalAccessException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        }
        //return null ;        
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Returns the "simple name" of the given class <br>
     * Just the same as Class.getSimpleName() in Java 5+<br>
     * e.g. returns "String" for "java.lang.String" class
     * @param cl
     * @return 
     */
    public static String getSimpleName(Class cl)
    {
        if ( cl != null )
        {
            return getSimpleName( cl.getName() ) ;
        }
        else
        {
            throw new TelosysRuntimeException("getSimpleName(Class) : null parameter");
        }
    }

    //-----------------------------------------------------------------------------------
    /**
     * Returns the "simple name" of the given class name <br>
     * Just the same as Class.getSimpleName() in Java 5+<br>
     * e.g. returns "String" for "java.lang.String"
     * @param sClassName the name of the class
     * @return 
     */
    public static String getSimpleName(String sClassName)
    {
        if ( sClassName != null )
        {
            int i = sClassName.lastIndexOf('.');
            if ( i >= 0 )
            {
                return sClassName.substring(i+1);
            }
            else
            {
                return sClassName;
            }
        }
        else
        {
            throw new TelosysRuntimeException("getSimpleName(String) : null parameter");
        }
    }
    
	/**
     * Returns the "package name" of the given class name <br>
     * e.g. returns "java.lang" for "java.lang.String"
	 * @param sFullClassName
	 * @return
	 */
	public static String getPackageName(String sFullClassName) 
	{
		if ( sFullClassName != null )
		{
			String s = sFullClassName.trim();
			int i = s.lastIndexOf('.');
			if ( i >= 0 )
			{
				return s.substring(0,i);
			}
			return "" ;
		}
		return null ;
	}
	
    //------------------------------------------------------------------------------------
	/**
	 * Try to load the Java class for the given class name without ClassNotFoundException
	 * @param sFullClassName
	 * @return the class loaded or null if not found 
	 */
	public static Class tryToLoadClass(String sFullClassName)
    {
        // trace("tryToLoadClass('" + sServiceClassName + "')");
	    Class c = null ;
        try
        {
            c = Class.forName(sFullClassName); 
        } catch (ClassNotFoundException e)
        {
            c = null ;	        
        }
        return c ;
        
//	    ClassLoader classLoader = ClassUtil.class.getClassLoader() ;
//        Class c = null ;
//	    if ( classLoader != null )
//	    {
//	        try
//	        {
//	            c = classLoader.loadClass(sFullClassName);
//	        } catch (ClassNotFoundException e)
//	        {
//	            c = null ;	        
//	        }
//	    }
//	    else
//	    {
//	        //error("Cannot get class loader from the current class !");
//	        return null ;
//	    }
//	    return c ;
    }
    
}